@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -20,16 +20,8 @@
 #include <ags/X/ags_export_soundcard.h>
 #include <ags/X/ags_export_soundcard_callbacks.h>
 
-#include <ags/object/ags_application_context.h>
-#include <ags/object/ags_connectable.h>
-#include <ags/object/ags_soundcard.h>
-
-#include <ags/audio/ags_sound_provider.h>
-#include <ags/audio/ags_devout.h>
-
-#include <ags/audio/pulse/ags_pulse_devout.h>
-
-#include <ags/audio/jack/ags_jack_devout.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
 
 #include <ags/X/ags_export_window.h>
 
@@ -48,9 +40,11 @@ void ags_export_soundcard_get_property(GObject *gobject,
 				       GValue *value,
 				       GParamSpec *param_spec);
 void ags_export_soundcard_finalize(GObject *gobject);
+
 void ags_export_soundcard_connect(AgsConnectable *connectable);
 void ags_export_soundcard_disconnect(AgsConnectable *connectable);
 void ags_export_soundcard_show(GtkWidget *widget);
+
 gboolean ags_export_soundcard_delete_event(GtkWidget *widget, GdkEventAny *event);
 
 /**
@@ -128,7 +122,7 @@ ags_export_soundcard_class_init(AgsExportSoundcardClass *export_soundcard)
    *
    * The assigned #AgsSoundcard acting as default sink.
    * 
-   * Since: 0.7.119
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("soundcard",
 				   i18n_pspec("assigned soundcard"),
@@ -185,12 +179,21 @@ ags_export_soundcard_init(AgsExportSoundcard *export_soundcard)
 		   0, 1,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
-
+  
+#ifdef AGS_WITH_CORE_AUDIO
+  gtk_combo_box_text_append_text(export_soundcard->backend,
+				 "core-audio");
+#endif
+  
+#ifdef AGS_WITH_PULSE
   gtk_combo_box_text_append_text(export_soundcard->backend,
 				 "pulse");
+#endif
   
+#ifdef AGS_WITH_JACK
   gtk_combo_box_text_append_text(export_soundcard->backend,
 				 "jack");
+#endif
   
 #ifdef AGS_WITH_ALSA
   gtk_combo_box_text_append_text(export_soundcard->backend,
@@ -348,6 +351,20 @@ ags_export_soundcard_get_property(GObject *gobject,
   }
 }
 
+void
+ags_export_soundcard_finalize(GObject *gobject)
+{
+  AgsExportSoundcard *export_soundcard;
+
+  export_soundcard = (AgsExportSoundcard *) gobject;
+
+  if(export_soundcard->soundcard != NULL){
+    g_object_unref(export_soundcard->soundcard);
+  }
+  
+  G_OBJECT_CLASS(ags_export_soundcard_parent_class)->finalize(gobject);
+}
+
 void
 ags_export_soundcard_connect(AgsConnectable *connectable)
 {
@@ -385,38 +402,24 @@ ags_export_soundcard_disconnect(AgsConnectable *connectable)
   export_soundcard->flags &= (~AGS_EXPORT_SOUNDCARD_CONNECTED);
 
   g_object_disconnect(G_OBJECT(export_soundcard->backend),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_export_soundcard_backend_callback),
 		      export_soundcard,
 		      NULL);
 
   g_object_disconnect(G_OBJECT(export_soundcard->card),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_export_soundcard_card_callback),
 		      export_soundcard,
 		      NULL);
   
   g_object_disconnect(G_OBJECT(export_soundcard->file_chooser_button),
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_export_soundcard_file_chooser_button_callback),
 		      export_soundcard,
 		      NULL);
 }
 
-void
-ags_export_soundcard_finalize(GObject *gobject)
-{
-  AgsExportSoundcard *export_soundcard;
-
-  export_soundcard = (AgsExportSoundcard *) gobject;
-
-  if(export_soundcard->soundcard != NULL){
-    g_object_unref(export_soundcard->soundcard);
-  }
-  
-  G_OBJECT_CLASS(ags_export_soundcard_parent_class)->finalize(gobject);
-}
-
 /**
  * ags_export_soundcard_set_backend:
  * @export_soundcard: the #AgsExportSoundcard
@@ -424,7 +427,7 @@ ags_export_soundcard_finalize(GObject *gobject)
  * 
  * Set backend.
  * 
- * Since: 0.7.119
+ * Since: 2.0.0
  */
 gboolean
 ags_export_soundcard_set_backend(AgsExportSoundcard *export_soundcard,
@@ -471,7 +474,7 @@ ags_export_soundcard_set_backend(AgsExportSoundcard *export_soundcard,
  * 
  * Refresh cards.
  * 
- * Since: 0.7.119
+ * Since: 2.0.0
  */
 void
 ags_export_soundcard_refresh_card(AgsExportSoundcard *export_soundcard)
@@ -480,7 +483,7 @@ ags_export_soundcard_refresh_card(AgsExportSoundcard *export_soundcard)
   
   GtkTreeModel *model;
   
-  GList *soundcard;
+  GList *start_soundcard, *soundcard;
   GList *card, *card_start;
   
   gchar *backend;
@@ -489,23 +492,25 @@ ags_export_soundcard_refresh_card(AgsExportSoundcard *export_soundcard)
   export_window = gtk_widget_get_ancestor(export_soundcard,
 					  AGS_TYPE_EXPORT_WINDOW);
  
-  soundcard = NULL;
+  start_soundcard = NULL;
   
   if(export_window != NULL &&
      export_window->application_context != NULL){
-    soundcard = ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(export_window->application_context));
+    start_soundcard = ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(export_window->application_context));
   }
 
   card_start = NULL;
   backend = gtk_combo_box_text_get_active_text(export_soundcard->backend);
 
+  soundcard = start_soundcard;
+  
   if(backend != NULL){
     if(!g_ascii_strncasecmp(backend,
 			    "alsa",
 			    5)){
       while(soundcard != NULL){
 	if(AGS_IS_DEVOUT(soundcard->data) &&
-	   (AGS_DEVOUT_ALSA & (AGS_DEVOUT(soundcard->data)->flags)) != 0){
+	   ags_devout_test_flags(AGS_DEVOUT(soundcard->data), AGS_DEVOUT_ALSA)){
 	  device = ags_soundcard_get_device(AGS_SOUNDCARD(soundcard->data));
 
 	  if(device != NULL){
@@ -521,7 +526,7 @@ ags_export_soundcard_refresh_card(AgsExportSoundcard *export_soundcard)
 				  4)){    
       while(soundcard != NULL){
 	if(AGS_IS_DEVOUT(soundcard->data) &&
-	   (AGS_DEVOUT_OSS & (AGS_DEVOUT(soundcard->data)->flags)) != 0){
+	   ags_devout_test_flags(AGS_DEVOUT(soundcard->data), AGS_DEVOUT_OSS)){
 	  device = ags_soundcard_get_device(AGS_SOUNDCARD(soundcard->data));
 
 	  if(device != NULL){
@@ -560,10 +565,27 @@ ags_export_soundcard_refresh_card(AgsExportSoundcard *export_soundcard)
 	  }
 	}
       
+	soundcard = soundcard->next;
+      }
+    }else if(!g_ascii_strncasecmp(backend,
+				  "core-audio",
+				  6)){
+      while(soundcard != NULL){
+	if(AGS_IS_CORE_AUDIO_DEVOUT(soundcard->data)){
+	  device = ags_soundcard_get_device(AGS_SOUNDCARD(soundcard->data));
+
+	  if(device != NULL){
+	    card_start = g_list_prepend(card_start,
+					device);
+	  }
+	}
+      
 	soundcard = soundcard->next;
       }
     }
   }
+
+  g_list_free(start_soundcard);
   
   model = gtk_combo_box_get_model(GTK_COMBO_BOX(export_soundcard->card));
   gtk_list_store_clear(GTK_LIST_STORE(model));
@@ -581,7 +603,8 @@ ags_export_soundcard_refresh_card(AgsExportSoundcard *export_soundcard)
       card = card->next;
     }
 
-    g_list_free(card_start);
+    g_list_free_full(card_start,
+		     g_free);
   }
 }
 
@@ -592,7 +615,7 @@ ags_export_soundcard_refresh_card(AgsExportSoundcard *export_soundcard)
  * 
  * Set card.
  * 
- * Since: 0.7.119
+ * Since: 2.0.0
  */
 gboolean
 ags_export_soundcard_set_card(AgsExportSoundcard *export_soundcard,
@@ -640,7 +663,7 @@ ags_export_soundcard_set_card(AgsExportSoundcard *export_soundcard,
  * 
  * Set filename.
  * 
- * Since: 0.7.119
+ * Since: 2.0.0
  */
 void
 ags_export_soundcard_set_filename(AgsExportSoundcard *export_soundcard,
@@ -657,7 +680,7 @@ ags_export_soundcard_set_filename(AgsExportSoundcard *export_soundcard,
  * 
  * Set format.
  * 
- * Since: 0.7.119
+ * Since: 2.0.0
  */
 void
 ags_export_soundcard_set_format(AgsExportSoundcard *export_soundcard,
@@ -699,11 +722,11 @@ ags_export_soundcard_set_format(AgsExportSoundcard *export_soundcard,
 /**
  * ags_export_soundcard_new:
  * 
- * Instantiate an #AgsExportSoundcard
+ * Create a new instance of #AgsExportSoundcard
  * 
- * Returns: an #AgsExportSoundcard
+ * Returns: the new #AgsExportSoundcard
  * 
- * Since: 0.7.119
+ * Since: 2.0.0
  */
 AgsExportSoundcard*
 ags_export_soundcard_new()
