@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2017 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -20,23 +20,8 @@
 #include <ags/X/ags_sequencer_editor.h>
 #include <ags/X/ags_sequencer_editor_callbacks.h>
 
-#include <ags/util/ags_list_util.h>
-
-#include <ags/object/ags_config.h>
-#include <ags/object/ags_connectable.h>
-#include <ags/object/ags_applicable.h>
-#include <ags/object/ags_distributed_manager.h>
-#include <ags/object/ags_sequencer.h>
-
-#include <ags/thread/ags_task_thread.h>
-
-#include <ags/audio/ags_sound_provider.h>
-#include <ags/audio/ags_midiin.h>
-
-#include <ags/audio/jack/ags_jack_server.h>
-#include <ags/audio/jack/ags_jack_midiin.h>
-
-#include <ags/audio/thread/ags_sequencer_thread.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
 
 #include <ags/X/ags_xorg_application_context.h>
 #include <ags/X/ags_window.h>
@@ -51,12 +36,13 @@ void ags_sequencer_editor_class_init(AgsSequencerEditorClass *sequencer_editor);
 void ags_sequencer_editor_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_sequencer_editor_applicable_interface_init(AgsApplicableInterface *applicable);
 void ags_sequencer_editor_init(AgsSequencerEditor *sequencer_editor);
+
 void ags_sequencer_editor_connect(AgsConnectable *connectable);
 void ags_sequencer_editor_disconnect(AgsConnectable *connectable);
+
 void ags_sequencer_editor_set_update(AgsApplicable *applicable, gboolean update);
 void ags_sequencer_editor_apply(AgsApplicable *applicable);
 void ags_sequencer_editor_reset(AgsApplicable *applicable);
-static void ags_sequencer_editor_finalize(GObject *gobject);
 
 /**
  * SECTION:ags_sequencer_editor
@@ -119,15 +105,7 @@ ags_sequencer_editor_get_type(void)
 void
 ags_sequencer_editor_class_init(AgsSequencerEditorClass *sequencer_editor)
 {
-  GObjectClass *gobject;
-  GtkWidgetClass *widget;
-
   ags_sequencer_editor_parent_class = g_type_class_peek_parent(sequencer_editor);
-
-  /* GtkObjectClass */
-  gobject = (GObjectClass *) sequencer_editor;
-
-  gobject->finalize = ags_sequencer_editor_finalize;
 }
 
 void
@@ -299,39 +277,31 @@ ags_sequencer_editor_disconnect(AgsConnectable *connectable)
 
   /* backend and card */
   g_object_disconnect(G_OBJECT(sequencer_editor->backend),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_sequencer_editor_backend_changed_callback),
 		      sequencer_editor,
 		      NULL);
 
   g_object_disconnect(G_OBJECT(sequencer_editor->card),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_sequencer_editor_card_changed_callback),
 		      sequencer_editor,
 		      NULL);
 
   /* add / remove jack */
   g_object_disconnect(G_OBJECT(sequencer_editor->add_jack),
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_sequencer_editor_add_jack_callback),
 		      sequencer_editor,
 		      NULL);
 
   g_object_disconnect(G_OBJECT(sequencer_editor->remove_jack),
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_sequencer_editor_remove_jack_callback),
 		      sequencer_editor,
 		      NULL);
 }
 
-static void
-ags_sequencer_editor_finalize(GObject *gobject)
-{
-  //TODO:JK: implement me
-  
-  G_OBJECT_CLASS(ags_sequencer_editor_parent_class)->finalize(gobject);
-}
-
 void
 ags_sequencer_editor_set_update(AgsApplicable *applicable, gboolean update)
 {
@@ -361,8 +331,6 @@ ags_sequencer_editor_apply(AgsApplicable *applicable)
   
   GValue value =  {0,};
 
-  pthread_mutex_t *application_mutex;
-
   sequencer_editor = AGS_SEQUENCER_EDITOR(applicable);
   midi_preferences = (AgsMidiPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 								      AGS_TYPE_MIDI_PREFERENCES);
@@ -582,8 +550,8 @@ ags_sequencer_editor_reset(AgsApplicable *applicable)
 }
 
 void
-ags_sequencer_editor_add_jack(AgsSequencerEditor *sequencer_editor,
-			      gchar *device)
+ags_sequencer_editor_add_source(AgsSequencerEditor *sequencer_editor,
+				gchar *device)
 {
   AgsWindow *window;
   AgsPreferences *preferences;
@@ -591,68 +559,50 @@ ags_sequencer_editor_add_jack(AgsSequencerEditor *sequencer_editor,
   AgsJackServer *jack_server;
   AgsJackMidiin *jack_midiin;
 
-  AgsMutexManager *mutex_manager;
   AgsThread *main_loop;
   AgsThread *sequencer_thread;
 
   AgsApplicationContext *application_context;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *card_name, *card_uri;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
   window = AGS_WINDOW(preferences->window);
 
   application_context = (AgsApplicationContext *) window->application_context;
-  application_mutex = window->application_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
   
   /* create sequencer */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
+  if((sound_server = ags_list_util_find_type(sound_server,
 						    AGS_TYPE_JACK_SERVER)) != NULL){
-    jack_server = AGS_JACK_SERVER(distributed_manager->data);
+    jack_server = AGS_JACK_SERVER(sound_server->data);
   }else{
     g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
     
     return;
   }
   
-  jack_midiin = (AgsJackMidiin *) ags_distributed_manager_register_sequencer(AGS_DISTRIBUTED_MANAGER(jack_server),
-									     FALSE);
-
-  pthread_mutex_unlock(application_mutex);
+  jack_midiin = (AgsJackMidiin *) ags_sound_server_register_sequencer(AGS_SOUND_SERVER(jack_server),
+								      FALSE);
 
   if(jack_midiin == NULL){
     return;
   }
-
-  pthread_mutex_lock(application_mutex);
-
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) jack_midiin);
   
 
   /* add new */
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
   
   sequencer_editor->sequencer = jack_midiin;
 
   ags_sound_provider_set_sequencer(AGS_SOUND_PROVIDER(application_context),
 				   g_list_append(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
 						 jack_midiin));
-
-  pthread_mutex_unlock(application_mutex);  
     
   g_object_ref(jack_midiin);
 
@@ -664,15 +614,11 @@ ags_sequencer_editor_add_jack(AgsSequencerEditor *sequencer_editor,
 				TRUE, TRUE);
 
   /*  */
-  pthread_mutex_lock(mutex);
-
   card_name = NULL;
   card_uri = NULL;
   ags_sequencer_list_cards(AGS_SEQUENCER(jack_midiin),
 			   &card_uri, &card_name);
 
-  pthread_mutex_unlock(mutex);
-
   gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(sequencer_editor->card))));
 
   while(card_uri != NULL){
@@ -686,41 +632,37 @@ ags_sequencer_editor_add_jack(AgsSequencerEditor *sequencer_editor,
 }
 
 void
-ags_sequencer_editor_remove_jack(AgsSequencerEditor *sequencer_editor,
-				 gchar *device)
+ags_sequencer_editor_remove_source(AgsSequencerEditor *sequencer_editor,
+				   gchar *device)
 {
   AgsWindow *window;
   AgsPreferences *preferences;
-
+  GtkDialog *dialog;
+ 
   AgsJackMidiin *jack_midiin;
 
   AgsApplicationContext *application_context;
   AgsThread *main_loop;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *sequencer;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
   window = AGS_WINDOW(preferences->window);
   application_context = (AgsApplicationContext *) window->application_context;
-
-  application_mutex = window->application_mutex;
   
   /* create sequencer */
-  pthread_mutex_lock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
+  
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  main_loop = application_context->main_loop;
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
+  if((sound_server = ags_list_util_find_type(sound_server,
 						    AGS_TYPE_JACK_SERVER)) == NULL){
     g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
     
     return;
   }
@@ -739,8 +681,6 @@ ags_sequencer_editor_remove_jack(AgsSequencerEditor *sequencer_editor,
     sequencer = sequencer->next;
   }
 
-  pthread_mutex_unlock(application_mutex);
-
   if(jack_midiin == NULL){
     return;
   }
@@ -750,24 +690,29 @@ ags_sequencer_editor_remove_jack(AgsSequencerEditor *sequencer_editor,
   gtk_combo_box_set_active(GTK_COMBO_BOX(sequencer_editor->backend),
 			   -1);
 
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
+#if 0
+  if((sound_server = ags_list_util_find_type(sound_server,
 						    AGS_TYPE_JACK_SERVER)) != NULL){
-    ags_distributed_manager_unregister_sequencer(AGS_DISTRIBUTED_MANAGER(distributed_manager->data),
+    ags_sound_server_unregister_sequencer(AGS_SOUND_SERVER(sound_server->data),
 						 jack_midiin);
   }
+#endif
 
   /* remove */
   if(jack_midiin == sequencer_editor->sequencer){
     sequencer_editor->sequencer = NULL;
   }
-  
+
+#if 0
   if(jack_midiin != NULL){
     ags_sound_provider_set_sequencer(AGS_SOUND_PROVIDER(application_context),
 				     g_list_remove(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
 						   jack_midiin));
     g_object_unref(jack_midiin);
   }
+#endif
   
+#if 0
   if(sequencer_editor->sequencer_thread != NULL){
     ags_thread_stop((AgsThread *) sequencer_editor->sequencer_thread);
 
@@ -778,6 +723,17 @@ ags_sequencer_editor_remove_jack(AgsSequencerEditor *sequencer_editor,
     
     sequencer_editor->sequencer_thread = NULL;
   }
+#endif
+  
+  /* notify user about restarting GSequencer */
+  dialog = gtk_message_dialog_new(preferences,
+				  GTK_DIALOG_MODAL,
+				  GTK_MESSAGE_INFO,
+				  GTK_BUTTONS_OK,
+				  "After finished your modifications you should safe your file and restart GSequencer");
+  g_signal_connect(dialog, "response",
+		   G_CALLBACK(gtk_widget_destroy), NULL);
+  gtk_widget_show_all(dialog);
 }
 
 void
@@ -792,8 +748,6 @@ ags_sequencer_editor_add_sequencer(AgsSequencerEditor *sequencer_editor,
 
   AgsApplicationContext *application_context;
 
-  pthread_mutex_t *application_mutex;
-
   if(sequencer == NULL ||
      AGS_IS_JACK_MIDIIN(sequencer)){
     return;
@@ -801,9 +755,10 @@ ags_sequencer_editor_add_sequencer(AgsSequencerEditor *sequencer_editor,
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
-  application_mutex = window->application_mutex;
 
   if(AGS_IS_MIDIIN(sequencer)){
     if((AGS_MIDIIN_ALSA & (AGS_MIDIIN(sequencer)->flags)) != 0){
@@ -820,13 +775,12 @@ ags_sequencer_editor_add_sequencer(AgsSequencerEditor *sequencer_editor,
   }
   
   /*  */
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
   if(g_list_find(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
 		 sequencer) != NULL){
-    pthread_mutex_unlock(application_mutex);  
   
     return;
   }
@@ -836,8 +790,6 @@ ags_sequencer_editor_add_sequencer(AgsSequencerEditor *sequencer_editor,
   ags_sound_provider_set_sequencer(AGS_SOUND_PROVIDER(application_context),
 				   g_list_append(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
 						 sequencer));
-
-  pthread_mutex_unlock(application_mutex);  
     
   g_object_ref(sequencer);
 
@@ -855,40 +807,42 @@ ags_sequencer_editor_remove_sequencer(AgsSequencerEditor *sequencer_editor,
 {
   AgsWindow *window;
   AgsPreferences *preferences;
+  GtkDialog *dialog;
 
   AgsThread *main_loop;
   AgsThread *sequencer_thread;
 
   AgsApplicationContext *application_context;
 
-  pthread_mutex_t *application_mutex;
-
   if(AGS_IS_JACK_MIDIIN(sequencer)){
     return;
   }
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;  
 
-  application_mutex = window->application_mutex;
-  
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
     
   if(sequencer == sequencer_editor->sequencer){
     sequencer_editor->sequencer = NULL;
   }
-  
+
+#if 0
   if(sequencer != NULL){
     ags_sound_provider_set_sequencer(AGS_SOUND_PROVIDER(application_context),
 				     g_list_remove(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
 						   sequencer));
     g_object_unref(sequencer);
   }
+#endif
   
+#if 0
   if(sequencer_editor->sequencer_thread != NULL){
     ags_thread_stop((AgsThread *) sequencer_editor->sequencer_thread);
 
@@ -899,8 +853,17 @@ ags_sequencer_editor_remove_sequencer(AgsSequencerEditor *sequencer_editor,
     
     sequencer_editor->sequencer_thread = NULL;
   }
-
-  pthread_mutex_unlock(application_mutex);
+#endif
+  
+  /* notify user about restarting GSequencer */
+  dialog = gtk_message_dialog_new(preferences,
+				  GTK_DIALOG_MODAL,
+				  GTK_MESSAGE_INFO,
+				  GTK_BUTTONS_OK,
+				  "After finished your modifications you should safe your file and restart GSequencer");
+  g_signal_connect(dialog, "response",
+		   G_CALLBACK(gtk_widget_destroy), NULL);
+  gtk_widget_show_all(dialog);
 }
 
 void
@@ -913,30 +876,24 @@ ags_sequencer_editor_load_jack_card(AgsSequencerEditor *sequencer_editor)
 
   AgsApplicationContext *application_context;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *sequencer;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-
   /* create sequencer */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
+  if((sound_server = ags_list_util_find_type(sound_server,
 						    AGS_TYPE_JACK_SERVER)) == NULL){
     g_warning("distributed manager not found");
 
-    pthread_mutex_unlock(application_mutex);
-
     return;
   }
 
@@ -966,9 +923,6 @@ ags_sequencer_editor_load_jack_card(AgsSequencerEditor *sequencer_editor)
     
     card_id = card_id->next;
   }
-
-  /*  */
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -984,18 +938,12 @@ ags_sequencer_editor_load_alsa_card(AgsSequencerEditor *sequencer_editor)
   GList *list;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
   window = AGS_WINDOW(preferences->window);
   application_context = (AgsApplicationContext *) window->application_context;
-
-  application_mutex = window->application_mutex;
   
   /*  */
-  pthread_mutex_lock(application_mutex);
-
   midiin = g_object_new(AGS_TYPE_MIDIIN,
 			NULL);
   midiin->flags &= (~AGS_MIDIIN_OSS);
@@ -1028,8 +976,6 @@ ags_sequencer_editor_load_alsa_card(AgsSequencerEditor *sequencer_editor)
   /* add new */
   ags_sequencer_editor_add_sequencer(sequencer_editor,
 				     (GObject *) midiin);
-
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -1044,19 +990,15 @@ ags_sequencer_editor_load_oss_card(AgsSequencerEditor *sequencer_editor)
 
   GList *list;
   GList *card_id;
-
-  pthread_mutex_t *application_mutex;
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
-  window = AGS_WINDOW(preferences->window);
-  application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
+  window = AGS_WINDOW(preferences->window);
+
+  application_context = (AgsApplicationContext *) window->application_context;
   
   /*  */  
-  pthread_mutex_lock(application_mutex);
-
   midiin = g_object_new(AGS_TYPE_MIDIIN,
 			NULL);
   midiin->flags &= (~AGS_MIDIIN_ALSA);
@@ -1089,18 +1031,16 @@ ags_sequencer_editor_load_oss_card(AgsSequencerEditor *sequencer_editor)
   /* add new */
   ags_sequencer_editor_add_sequencer(sequencer_editor,
 				     (GObject *) midiin);
-
-  pthread_mutex_unlock(application_mutex);
 }
 
 /**
  * ags_sequencer_editor_new:
  *
- * Creates an #AgsSequencerEditor
+ * Create a new instance of #AgsSequencerEditor
  *
- * Returns: a new #AgsSequencerEditor
+ * Returns: the new #AgsSequencerEditor
  *
- * Since: 0.7.111
+ * Since: 2.0.0
  */
 AgsSequencerEditor*
 ags_sequencer_editor_new()
