@@ -20,28 +20,8 @@
 #include <ags/X/ags_soundcard_editor.h>
 #include <ags/X/ags_soundcard_editor_callbacks.h>
 
-#include <ags/util/ags_list_util.h>
-
-#include <ags/object/ags_config.h>
-#include <ags/object/ags_connectable.h>
-#include <ags/object/ags_applicable.h>
-#include <ags/object/ags_distributed_manager.h>
-#include <ags/object/ags_soundcard.h>
-
-#include <ags/thread/ags_task_thread.h>
-
-#include <ags/audio/ags_sound_provider.h>
-#include <ags/audio/ags_devout.h>
-
-#include <ags/audio/jack/ags_jack_server.h>
-#include <ags/audio/jack/ags_jack_devout.h>
-
-#include <ags/audio/pulse/ags_pulse_server.h>
-#include <ags/audio/pulse/ags_pulse_devout.h>
-
-#include <ags/audio/task/ags_notify_soundcard.h>
-
-#include <ags/audio/thread/ags_sequencer_thread.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
 
 #include <ags/X/ags_xorg_application_context.h>
 #include <ags/X/ags_window.h>
@@ -188,11 +168,20 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
+#ifdef AGS_WITH_CORE_AUDIO
+  gtk_combo_box_text_append_text(soundcard_editor->backend,
+				 "core-audio");
+#endif
+
+#ifdef AGS_WITH_PULSE
   gtk_combo_box_text_append_text(soundcard_editor->backend,
 				 "pulse");
+#endif
 
+#ifdef AGS_WITH_JACK
   gtk_combo_box_text_append_text(soundcard_editor->backend,
 				 "jack");
+#endif
   
 #ifdef AGS_WITH_ALSA
   gtk_combo_box_text_append_text(soundcard_editor->backend,
@@ -419,51 +408,51 @@ ags_soundcard_editor_disconnect(AgsConnectable *connectable)
 
   /* backend and card */
   g_object_disconnect(G_OBJECT(soundcard_editor->backend),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_soundcard_editor_backend_changed_callback),
 		      soundcard_editor,
 		      NULL);
 
   g_object_disconnect(G_OBJECT(soundcard_editor->card),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_soundcard_editor_card_changed_callback),
 		      soundcard_editor,
 		      NULL);
 
   /* add / remove sink */
   g_object_disconnect(G_OBJECT(soundcard_editor->add_sink),
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_soundcard_editor_add_sink_callback),
 		      soundcard_editor,
 		      NULL);
 
   g_object_disconnect(G_OBJECT(soundcard_editor->remove_sink),
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_soundcard_editor_remove_sink_callback),
 		      soundcard_editor,
 		      NULL);
 
   /* presets */
   g_object_disconnect(G_OBJECT(soundcard_editor->audio_channels),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_soundcard_editor_audio_channels_changed_callback),
 		      soundcard_editor,
 		      NULL);
 
   g_object_disconnect(G_OBJECT(soundcard_editor->samplerate),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_soundcard_editor_samplerate_changed_callback),
 		      soundcard_editor,
 		      NULL);
 
   g_object_disconnect(G_OBJECT(soundcard_editor->buffer_size),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_soundcard_editor_buffer_size_changed_callback),
 		      soundcard_editor,
 		      NULL);
 
   g_object_disconnect(G_OBJECT(soundcard_editor->format),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_soundcard_editor_format_changed_callback),
 		      soundcard_editor,
 		      NULL);
@@ -506,12 +495,10 @@ ags_soundcard_editor_apply(AgsApplicable *applicable)
   guint samplerate;
   guint buffer_size;
   guint format;
-  gboolean use_pulse, use_jack, use_alsa, use_oss;
+  gboolean use_core_audio, use_pulse, use_jack, use_alsa, use_oss;
   
   GValue value =  {0,};
 
-  pthread_mutex_t *application_mutex;
-
   soundcard_editor = AGS_SOUNDCARD_EDITOR(applicable);
   audio_preferences = (AgsAudioPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 								      AGS_TYPE_AUDIO_PREFERENCES);
@@ -532,7 +519,12 @@ ags_soundcard_editor_apply(AgsApplicable *applicable)
   g_list_free(list);
   
   /* backend */
+#ifdef AGS_WITH_CORE_AUDIO
+  use_core_audio = TRUE;
+#else
   use_pulse = TRUE;
+#endif
+  
   use_jack = FALSE;
   use_alsa = FALSE;
   use_oss = FALSE;
@@ -545,6 +537,10 @@ ags_soundcard_editor_apply(AgsApplicable *applicable)
 
   if(backend != NULL){
     if(!g_ascii_strncasecmp(backend,
+			    "core-audio",
+			    11)){
+      use_pulse = TRUE;
+    }else if(!g_ascii_strncasecmp(backend,
 			    "pulse",
 			    6)){
       use_pulse = TRUE;
@@ -639,7 +635,12 @@ ags_soundcard_editor_apply(AgsApplicable *applicable)
   /* handle */
   g_message("%s", device);
 
-  if(use_pulse){
+  if(use_core_audio){
+    ags_config_set_value(config,
+			 soundcard_group,
+			 "device",
+			 device);
+  }else if(use_pulse){
     ags_config_set_value(config,
 			 soundcard_group,
 			 "device",
@@ -686,6 +687,13 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
   GError *error;
   
   soundcard_editor = AGS_SOUNDCARD_EDITOR(applicable);
+
+  if((AGS_SOUNDCARD_EDITOR_BLOCK_RESET & (soundcard_editor->flags)) != 0){
+    return;
+  }
+
+  soundcard_editor->flags |= AGS_SOUNDCARD_EDITOR_BLOCK_RESET;
+  
   soundcard = soundcard_editor->soundcard;
   
   /* refresh */
@@ -693,7 +701,9 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
 
   backend = NULL;
   
-  if(AGS_IS_PULSE_DEVOUT(soundcard)){
+  if(AGS_IS_CORE_AUDIO_DEVOUT(soundcard)){
+    backend = "core-audio";
+  }else if(AGS_IS_PULSE_DEVOUT(soundcard)){
     backend = "pulse";
   }else if(AGS_IS_JACK_DEVOUT(soundcard)){
     backend = "jack";
@@ -707,19 +717,26 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
 
   if(backend != NULL){
     if(!g_ascii_strncasecmp(backend,
+			    "core-audio",
+			    11)){
+      gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->backend),
+			       0);
+      
+      //      ags_soundcard_editor_load_core_audio_card(soundcard_editor);
+    }else if(!g_ascii_strncasecmp(backend,
 			    "pulse",
 			    6)){
       gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->backend),
 			       0);
       
-      ags_soundcard_editor_load_pulse_card(soundcard_editor);
+      //      ags_soundcard_editor_load_pulse_card(soundcard_editor);
     }else if(!g_ascii_strncasecmp(backend,
 			    "jack",
 			    5)){
       gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->backend),
 			       1);
       
-      ags_soundcard_editor_load_jack_card(soundcard_editor);
+      //      ags_soundcard_editor_load_jack_card(soundcard_editor);
     }else if(!g_ascii_strncasecmp(backend,
 				  "alsa",
 				  5)){
@@ -730,7 +747,7 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
 			       2);
 #endif
       
-      ags_soundcard_editor_load_alsa_card(soundcard_editor);
+      //      ags_soundcard_editor_load_alsa_card(soundcard_editor);
     }else if(!g_ascii_strncasecmp(backend,
 				  "oss",
 				  4)){
@@ -750,7 +767,7 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
 			       -1);
 #endif
       
-      ags_soundcard_editor_load_oss_card(soundcard_editor);
+      //      ags_soundcard_editor_load_oss_card(soundcard_editor);
     }
   }
 
@@ -774,13 +791,7 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
   while(card_id != NULL){
     //FIXME:JK: work-around for alsa-handle
     tmp = card_id->data;
-    
-    if(card_id->data != NULL &&
-       use_alsa){
-      tmp = g_strdup_printf("%s,0",
-			    card_id->data);
-    }
-    
+        
     if(tmp != NULL &&
        device != NULL &&
        !g_ascii_strcasecmp(tmp,
@@ -792,11 +803,7 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
       gtk_combo_box_text_append_text(soundcard_editor->card,
 				     tmp);
     }
-    
-    if(use_alsa){
-      g_free(tmp);
-    }
-    
+        
     if(!found_card){
       nth++;
     }
@@ -891,6 +898,8 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
 			    samplerate_min, samplerate_max);
   gtk_spin_button_set_range(soundcard_editor->buffer_size,
 			    buffer_size_min, buffer_size_max);
+
+  soundcard_editor->flags &= (~AGS_SOUNDCARD_EDITOR_BLOCK_RESET);
 }
 
 void
@@ -906,9 +915,11 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
   AgsJackServer *jack_server;
   AgsJackDevout *jack_devout;
 
+  AgsCoreAudioServer *core_audio_server;
+  AgsCoreAudioDevout *core_audio_devout;
+
   AgsNotifySoundcard *notify_soundcard;
   
-  AgsMutexManager *mutex_manager;
   AgsThread *main_loop;
   AgsThread *soundcard_thread;
   AgsThread *export_thread;
@@ -919,26 +930,23 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
   
   GType server_type;
   
-  GList *distributed_manager;
+  GList *sound_server;
   GList *card_name, *card_uri;
 
   gchar *backend;
   
-  gboolean use_pulse, use_jack;
+  gboolean use_core_audio, use_pulse, use_jack;
   gboolean initial_soundcard;
-  
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *mutex;
 
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
   window = AGS_WINDOW(preferences->window);
 
   application_context = (AgsApplicationContext *) window->application_context;
-  application_mutex = window->application_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
 
+  core_audio_server = NULL;
+  core_audio_devout = NULL;
+  
   pulse_server = NULL;
   pulse_devout = NULL;
   
@@ -951,6 +959,7 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
   
   backend = NULL;
 
+  use_core_audio = FALSE;
   use_pulse = FALSE;
   use_jack = FALSE;
 
@@ -961,6 +970,12 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
 
   if(backend != NULL){
     if(!g_ascii_strncasecmp(backend,
+			    "core-audio",
+			    11)){
+      server_type = AGS_TYPE_CORE_AUDIO_SERVER;
+      
+      use_core_audio = TRUE;
+    }else if(!g_ascii_strncasecmp(backend,
 			    "pulse",
 			    6)){
       server_type = AGS_TYPE_PULSE_SERVER;
@@ -976,47 +991,44 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
   }
   
   /* create soundcard */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
+  if((sound_server = ags_list_util_find_type(sound_server,
 						    server_type)) != NULL){
-    if(use_pulse){
-      pulse_server = AGS_PULSE_SERVER(distributed_manager->data);
+    if(use_core_audio){
+      core_audio_server = AGS_CORE_AUDIO_SERVER(sound_server->data);
 
       soundcard = 
-	pulse_devout = (AgsPulseDevout *) ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(pulse_server),
+	core_audio_devout = (AgsCoreAudioDevout *) ags_sound_server_register_soundcard(AGS_SOUND_SERVER(core_audio_server),
+											      TRUE);
+    }else if(use_pulse){
+      pulse_server = AGS_PULSE_SERVER(sound_server->data);
+
+      soundcard = 
+	pulse_devout = (AgsPulseDevout *) ags_sound_server_register_soundcard(AGS_SOUND_SERVER(pulse_server),
 										     TRUE);
     }else if(use_jack){
-      jack_server = AGS_JACK_SERVER(distributed_manager->data);
+      jack_server = AGS_JACK_SERVER(sound_server->data);
 
       soundcard = 
-	jack_devout = (AgsJackDevout *) ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(jack_server),
+	jack_devout = (AgsJackDevout *) ags_sound_server_register_soundcard(AGS_SOUND_SERVER(jack_server),
 										   TRUE);
     }
   }else{
     g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
     
     return;
   }
-  
-  pthread_mutex_unlock(application_mutex);
 
-  if(jack_devout == NULL){
+  if(soundcard == NULL){
     return;
   }
-
-  pthread_mutex_lock(application_mutex);
-
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) soundcard);
   
 
   /* add new */
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
   
   if(ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(application_context)) == NULL){
     initial_soundcard = TRUE;
@@ -1039,20 +1051,18 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
     while(machine != NULL){
       g_object_ref(G_OBJECT(AGS_MACHINE(machine->data)->audio));
   
-      list = ags_soundcard_get_audio(AGS_SOUNDCARD(soundcard));
+      list = ags_sound_provider_get_audio(AGS_SOUND_PROVIDER(application_context));
       list = g_list_prepend(list,
 			    AGS_MACHINE(machine->data)->audio);
-      ags_soundcard_set_audio(AGS_SOUNDCARD(soundcard),
-			      list);
+      ags_sound_provider_set_audio(AGS_SOUND_PROVIDER(application_context),
+				   list);
 
       machine = machine->next;
     }
 
     g_list_free(machine_start);
   }
-  
-  pthread_mutex_unlock(application_mutex);  
-    
+      
   g_object_ref(soundcard);
 
   soundcard_thread = (AgsThread *) ags_soundcard_thread_new(soundcard);
@@ -1066,7 +1076,9 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
   notify_soundcard = ags_notify_soundcard_new(soundcard_thread);
   AGS_TASK(notify_soundcard)->task_thread = application_context->task_thread;
 
-  if(use_pulse){
+  if(use_core_audio){
+    core_audio_devout->notify_soundcard = notify_soundcard;
+  }else if(use_pulse){
     pulse_devout->notify_soundcard = notify_soundcard;
   }else if(use_jack){
     jack_devout->notify_soundcard = notify_soundcard;
@@ -1087,18 +1099,12 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
 				(AgsThread *) export_thread,
 				TRUE, TRUE);
 
-  pthread_mutex_unlock(application_mutex);
-
   /*  */
-  pthread_mutex_lock(mutex);
-
   card_name = NULL;
   card_uri = NULL;
   ags_soundcard_list_cards(AGS_SOUNDCARD(soundcard),
 			   &card_uri, &card_name);
 
-  pthread_mutex_unlock(mutex);
-
   gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(soundcard_editor->card))));
 
   while(card_uri != NULL){
@@ -1117,6 +1123,10 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
 {
   AgsWindow *window;
   AgsPreferences *preferences;
+  GtkDialog *dialog;
+  
+  AgsCoreAudioServer *core_audio_server;
+  AgsCoreAudioDevout *core_audio_devout;
 
   AgsPulseServer *pulse_server;
   AgsPulseDevout *pulse_devout;
@@ -1131,24 +1141,26 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
   GObject *soundcard;
 
   GType server_type;
-  
-  GList *distributed_manager;
+
+  GList *machine, *machine_start;
+  GList *sound_server;
   GList *list;
   GList *card_id;
 
   gchar *backend;
   
-  gboolean use_pulse, use_jack;
-
-  pthread_mutex_t *application_mutex;
+  gboolean use_core_audio, use_pulse, use_jack;
 
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-
+  core_audio_server = NULL;
+  core_audio_devout = NULL;
+  
   pulse_server = NULL;
   pulse_devout = NULL;
   
@@ -1162,6 +1174,7 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
   
   backend = NULL;
 
+  use_core_audio = FALSE;
   use_pulse = FALSE;
   use_jack = FALSE;
 
@@ -1170,6 +1183,12 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
 
   if(backend != NULL){
     if(!g_ascii_strncasecmp(backend,
+			    "core-audio",
+			    6)){
+      server_type = AGS_TYPE_CORE_AUDIO_SERVER;
+      
+      use_core_audio = TRUE;
+    }else if(!g_ascii_strncasecmp(backend,
 			    "pulse",
 			    6)){
       server_type = AGS_TYPE_PULSE_SERVER;
@@ -1185,24 +1204,26 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
   }
   
   /* destroy soundcard */
-  pthread_mutex_lock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
-  main_loop = application_context->main_loop;
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
+  if((sound_server = ags_list_util_find_type(sound_server,
 						    server_type)) != NULL){
-    if(use_pulse){
+    if(use_core_audio){
       server = 
-	pulse_server = AGS_PULSE_SERVER(distributed_manager->data);
+	core_audio_server = AGS_CORE_AUDIO_SERVER(sound_server->data);
+    }else if(use_pulse){
+      server = 
+	pulse_server = AGS_PULSE_SERVER(sound_server->data);
     }else if(use_jack){
       server = 
-	jack_server = AGS_JACK_SERVER(distributed_manager->data);
+	jack_server = AGS_JACK_SERVER(sound_server->data);
     }
   }else{
     g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
     
     return;
   }
@@ -1210,7 +1231,15 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
   list = ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(application_context));
   
   while(list != NULL){
-    if(use_pulse){
+    if(use_core_audio){
+      if(AGS_IS_CORE_AUDIO_DEVOUT(list->data) &&
+	 !g_ascii_strcasecmp(ags_soundcard_get_device(AGS_SOUNDCARD(list->data)),
+			     device)){
+	soundcard = 
+	  core_audio_devout = list->data;
+	break;
+      }
+    }else if(use_pulse){
       if(AGS_IS_PULSE_DEVOUT(list->data) &&
 	 !g_ascii_strcasecmp(ags_soundcard_get_device(AGS_SOUNDCARD(list->data)),
 			     device)){
@@ -1231,27 +1260,50 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
     list = list->next;
   }
 
-  pthread_mutex_unlock(application_mutex);
-
   if(soundcard == NULL){
     return;
   }
+
+  /* remove AgsSoundcard from AgsAudio */
+#if 0
+  machine = 
+    machine_start = gtk_container_get_children(window->machines);
+  
+  while(machine != NULL){
+    if(AGS_MACHINE(machine->data)->audio->soundcard == soundcard){
+      g_object_set(AGS_MACHINE(machine->data)->audio,
+		   "soundcard", NULL,
+		   NULL);
+
+      g_object_unref(G_OBJECT(AGS_MACHINE(machine->data)->audio));
+    }
+    
+    machine = machine->next;
+  }
+
+  ags_sound_provider_set_audio(AGS_SOUND_PROVIDER(application_context),
+			       NULL);
+  g_list_free(machine_start);
+#endif
   
   /*  */
   gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(soundcard_editor->card))));
   gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->backend),
 			   -1);
 
+#if 0
   if(server != NULL){
-    ags_distributed_manager_unregister_soundcard(AGS_DISTRIBUTED_MANAGER(server),
+    ags_sound_server_unregister_soundcard(AGS_SOUND_SERVER(server),
 						 soundcard);
   }
-
+#endif
+  
   /* remove */
   if(soundcard == soundcard_editor->soundcard){
     soundcard_editor->soundcard = NULL;
   }
-  
+    
+#if 0  
   if(soundcard != NULL){
     ags_sound_provider_set_soundcard(AGS_SOUND_PROVIDER(application_context),
 				     g_list_remove(ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(application_context)),
@@ -1269,6 +1321,17 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
     
     soundcard_editor->soundcard_thread = NULL;
   }
+#endif
+
+  /* notify user about restarting GSequencer */
+  dialog = gtk_message_dialog_new(preferences,
+				  GTK_DIALOG_MODAL,
+				  GTK_MESSAGE_INFO,
+				  GTK_BUTTONS_OK,
+				  "After finished your modifications you should safe your file and restart GSequencer");
+  g_signal_connect(dialog, "response",
+		   G_CALLBACK(gtk_widget_destroy), NULL);
+  gtk_widget_show_all(dialog);
 }
 
 void
@@ -1287,21 +1350,26 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
   AgsApplicationContext *application_context;
 
   gboolean initial_soundcard;
-  
-  pthread_mutex_t *application_mutex;
 
   if(soundcard == NULL ||
+     AGS_IS_CORE_AUDIO_DEVOUT(soundcard) ||
      AGS_IS_PULSE_DEVOUT(soundcard) ||
      AGS_IS_JACK_DEVOUT(soundcard)){
     return;
   }
-  
+
+  if((AGS_SOUNDCARD_EDITOR_BLOCK_ADD & (soundcard_editor->flags)) != 0){
+    return;
+  }
+
+  soundcard_editor->flags |= AGS_SOUNDCARD_EDITOR_BLOCK_ADD;
+
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
   
   application_context = (AgsApplicationContext *) window->application_context;
-  application_mutex = window->application_mutex;
 
   initial_soundcard = FALSE;
   
@@ -1320,14 +1388,14 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
   }
   
   /*  */
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
   if(g_list_find(ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(application_context)),
 		 soundcard) != NULL){
-    pthread_mutex_unlock(application_mutex);  
-  
+    soundcard_editor->flags &= (~AGS_SOUNDCARD_EDITOR_BLOCK_ADD);
+    
     return;
   }
 
@@ -1352,12 +1420,16 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
     while(machine != NULL){
       g_object_ref(G_OBJECT(AGS_MACHINE(machine->data)->audio));
   
-      list = ags_soundcard_get_audio(AGS_SOUNDCARD(soundcard));
+      list = ags_sound_provider_get_audio(AGS_SOUND_PROVIDER(application_context));
       list = g_list_prepend(list,
 			    AGS_MACHINE(machine->data)->audio);
-      ags_soundcard_set_audio(AGS_SOUNDCARD(soundcard),
-			      list);
+      ags_sound_provider_set_audio(AGS_SOUND_PROVIDER(application_context),
+				   list);
 
+      g_object_set(AGS_MACHINE(machine->data)->audio,
+		   "soundcard", soundcard,
+		   NULL);
+      
       machine = machine->next;
     }
 
@@ -1365,8 +1437,6 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
     g_list_free(machine_start);
   }
   
-  pthread_mutex_unlock(application_mutex);  
-    
   g_object_ref(soundcard);
 
   soundcard_thread = (AgsThread *) ags_soundcard_thread_new(soundcard);
@@ -1398,6 +1468,8 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
   ags_thread_add_child_extended(main_loop,
 				(AgsThread *) export_thread,
 				TRUE, TRUE);
+
+  soundcard_editor->flags &= (~AGS_SOUNDCARD_EDITOR_BLOCK_ADD);
 }
 
 void
@@ -1406,15 +1478,17 @@ ags_soundcard_editor_remove_soundcard(AgsSoundcardEditor *soundcard_editor,
 {
   AgsWindow *window;
   AgsPreferences *preferences;
-
+  GtkDialog *dialog;
+  
   AgsThread *main_loop;
   AgsThread *soundcard_thread;
 
   AgsApplicationContext *application_context;
 
-  pthread_mutex_t *application_mutex;
+  GList *machine, *machine_start;
 
   if(soundcard == NULL ||
+     AGS_IS_CORE_AUDIO_DEVOUT(soundcard) ||
      AGS_IS_PULSE_DEVOUT(soundcard) ||
      AGS_IS_JACK_DEVOUT(soundcard)){
     return;
@@ -1422,26 +1496,52 @@ ags_soundcard_editor_remove_soundcard(AgsSoundcardEditor *soundcard_editor,
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;  
 
-  application_mutex = window->application_mutex;
-  
-  pthread_mutex_lock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
-  main_loop = (AgsThread *) application_context->main_loop;
+  /* remove AgsSoundcard from AgsAudio */
+#if 0
+  machine = 
+    machine_start = gtk_container_get_children(window->machines);
+  
+  while(machine != NULL){
+    if(AGS_MACHINE(machine->data)->audio->soundcard == soundcard){
+      g_object_set(AGS_MACHINE(machine->data)->audio,
+		   "soundcard", NULL,
+		   NULL);
+
+      g_object_unref(G_OBJECT(AGS_MACHINE(machine->data)->audio));
+    }
     
+    machine = machine->next;
+  }
+
+  ags_sound_provider_set_audio(AGS_SOUND_PROVIDER(application_context),
+			       NULL);
+  g_list_free(machine_start);
+#endif
+  
+  /*  */  
   if(soundcard == soundcard_editor->soundcard){
     soundcard_editor->soundcard = NULL;
   }
   
+#if 0
   if(soundcard != NULL){
     ags_sound_provider_set_soundcard(AGS_SOUND_PROVIDER(application_context),
 				     g_list_remove(ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(application_context)),
 						   soundcard));
     g_object_unref(soundcard);
   }
-  
+#endif
+
+#if 0
   if(soundcard_editor->soundcard_thread != NULL){
     ags_thread_stop((AgsThread *) soundcard_editor->soundcard_thread);
 
@@ -1452,8 +1552,82 @@ ags_soundcard_editor_remove_soundcard(AgsSoundcardEditor *soundcard_editor,
     
     soundcard_editor->soundcard_thread = NULL;
   }
+#endif
 
-  pthread_mutex_unlock(application_mutex);
+  /* notify user about restarting GSequencer */
+  dialog = gtk_message_dialog_new(preferences,
+				  GTK_DIALOG_MODAL,
+				  GTK_MESSAGE_INFO,
+				  GTK_BUTTONS_OK,
+				  "After finished your modifications you should safe your file and restart GSequencer");
+  g_signal_connect(dialog, "response",
+		   G_CALLBACK(gtk_widget_destroy), NULL);
+  gtk_widget_show_all(dialog);
+}
+
+void
+ags_soundcard_editor_load_core_audio_card(AgsSoundcardEditor *soundcard_editor)
+{
+  AgsWindow *window;
+  AgsPreferences *preferences;
+
+  AgsCoreAudioDevout *core_audio_devout;
+
+  AgsApplicationContext *application_context;
+
+  GList *sound_server;
+  GList *soundcard;
+  GList *card_id;
+
+  preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
+							   AGS_TYPE_PREFERENCES);
+
+  window = AGS_WINDOW(preferences->window);
+
+  application_context = (AgsApplicationContext *) window->application_context;
+
+  /* create soundcard */
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
+
+  if(sound_server == NULL){
+    g_warning("distributed manager not found");
+
+    return;
+  }
+
+  soundcard = ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(application_context));
+  core_audio_devout = NULL;
+  
+  while(soundcard != NULL){
+    if(AGS_IS_CORE_AUDIO_DEVOUT(soundcard->data)){
+      core_audio_devout = soundcard->data;
+      break;
+    }
+    
+    soundcard = soundcard->next;
+  }
+
+  card_id = NULL;
+  ags_soundcard_list_cards(AGS_SOUNDCARD(core_audio_devout),
+			   &card_id, NULL);
+
+  gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(soundcard_editor->card))));
+
+  while(card_id != NULL){
+    if(card_id->data != NULL){
+      gtk_combo_box_text_append_text(soundcard_editor->card,
+				     card_id->data);
+    }
+    
+    card_id = card_id->next;
+  }
+
+  /*  */
+  gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->buffer_size,
+			   FALSE);
+  
+  gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
+			   FALSE);
 }
 
 void
@@ -1466,29 +1640,23 @@ ags_soundcard_editor_load_pulse_card(AgsSoundcardEditor *soundcard_editor)
 
   AgsApplicationContext *application_context;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *soundcard;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-
   /* create soundcard */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if(distributed_manager == NULL){
+  if(sound_server == NULL){
     g_warning("distributed manager not found");
 
-    pthread_mutex_unlock(application_mutex);
-
     return;
   }
 
@@ -1525,7 +1693,6 @@ ags_soundcard_editor_load_pulse_card(AgsSoundcardEditor *soundcard_editor)
   
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   FALSE);
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -1538,29 +1705,23 @@ ags_soundcard_editor_load_jack_card(AgsSoundcardEditor *soundcard_editor)
 
   AgsApplicationContext *application_context;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *soundcard;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-
   /* create soundcard */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if(distributed_manager == NULL){
+  if(sound_server == NULL){
     g_warning("distributed manager not found");
 
-    pthread_mutex_unlock(application_mutex);
-
     return;
   }
 
@@ -1597,7 +1758,6 @@ ags_soundcard_editor_load_jack_card(AgsSoundcardEditor *soundcard_editor)
   
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   FALSE);
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -1613,18 +1773,20 @@ ags_soundcard_editor_load_alsa_card(AgsSoundcardEditor *soundcard_editor)
   GList *list;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
+  if((AGS_SOUNDCARD_EDITOR_BLOCK_LOAD & (soundcard_editor->flags)) != 0){
+    return;
+  }
+
+  soundcard_editor->flags |= AGS_SOUNDCARD_EDITOR_BLOCK_LOAD;
 
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-  
   /*  */
-  pthread_mutex_lock(application_mutex);
-
   devout = g_object_new(AGS_TYPE_DEVOUT,
 			NULL);
   devout->flags &= (~AGS_DEVOUT_OSS);
@@ -1671,7 +1833,7 @@ ags_soundcard_editor_load_alsa_card(AgsSoundcardEditor *soundcard_editor)
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   TRUE);
 
-  pthread_mutex_unlock(application_mutex);
+  soundcard_editor->flags &= (~AGS_SOUNDCARD_EDITOR_BLOCK_LOAD);
 }
 
 void
@@ -1686,19 +1848,15 @@ ags_soundcard_editor_load_oss_card(AgsSoundcardEditor *soundcard_editor)
 
   GList *list;
   GList *card_id;
-
-  pthread_mutex_t *application_mutex;
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
-
-  application_mutex = window->application_mutex;
   
-  /*  */  
-  pthread_mutex_lock(application_mutex);
-
+  /*  */
   devout = g_object_new(AGS_TYPE_DEVOUT,
 			NULL);
   devout->flags &= (~AGS_DEVOUT_ALSA);
@@ -1744,18 +1902,16 @@ ags_soundcard_editor_load_oss_card(AgsSoundcardEditor *soundcard_editor)
 
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   TRUE);
-
-  pthread_mutex_unlock(application_mutex);
 }
 
 /**
  * ags_soundcard_editor_new:
  *
- * Creates an #AgsSoundcardEditor
+ * Create a new instance of #AgsSoundcardEditor
  *
- * Returns: a new #AgsSoundcardEditor
+ * Returns: the new #AgsSoundcardEditor
  *
- * Since: 0.7.65
+ * Since: 2.0.0
  */
 AgsSoundcardEditor*
 ags_soundcard_editor_new()
@@ -1767,4 +1923,3 @@ ags_soundcard_editor_new()
   
   return(soundcard_editor);
 }
-
