@@ -20,18 +20,34 @@
 #include <ags/X/ags_window.h>
 #include <ags/X/ags_window_callbacks.h>
 
-#include <ags/object/ags_connectable.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
+
+#include <ags/X/ags_ui_provider.h>
 
 #include <ags/X/machine/ags_panel.h>
 #include <ags/X/machine/ags_mixer.h>
+#include <ags/X/machine/ags_spectrometer.h>
+#include <ags/X/machine/ags_equalizer10.h>
 #include <ags/X/machine/ags_drum.h>
 #include <ags/X/machine/ags_matrix.h>
 #include <ags/X/machine/ags_synth.h>
 #include <ags/X/machine/ags_syncsynth.h>
 #include <ags/X/machine/ags_ffplayer.h>
+#include <ags/X/machine/ags_audiorec.h>
 #include <ags/X/machine/ags_ladspa_bridge.h>
 #include <ags/X/machine/ags_dssi_bridge.h>
 #include <ags/X/machine/ags_lv2_bridge.h>
+#include <ags/X/machine/ags_live_dssi_bridge.h>
+#include <ags/X/machine/ags_live_lv2_bridge.h>
+
+#include <ags/X/file/ags_simple_file.h>
+
+#include <ags/X/thread/ags_gui_thread.h>
+
+#ifdef AGS_WITH_QUARTZ
+#include <gtkmacintegration-gtk2/gtkosxapplication.h>
+#endif
 
 #include <stdlib.h>
 
@@ -72,6 +88,7 @@ enum{
 };
 
 static gpointer ags_window_parent_class = NULL;
+GHashTable *ags_window_load_file = NULL;
 
 GType
 ags_window_get_type()
@@ -132,7 +149,7 @@ ags_window_class_init(AgsWindowClass *window)
    *
    * The assigned main AgsSoundcard.
    * 
-   * Since: 0.7.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("soundcard",
 				   i18n_pspec("assigned soundcard"),
@@ -148,7 +165,7 @@ ags_window_class_init(AgsWindowClass *window)
    *
    * The assigned application context.
    * 
-   * Since: 0.7.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("application-context",
 				   i18n_pspec("assigned application context"),
@@ -180,6 +197,7 @@ void
 ags_window_init(AgsWindow *window)
 {
   GtkVBox *vbox;
+  GtkViewport *viewport;
   GtkWidget *scrolled_window;
 
   gchar *str;
@@ -202,10 +220,15 @@ ags_window_init(AgsWindow *window)
   window->soundcard = NULL;
 
   /* window name and title */
+  window->filename = NULL;
+
   window->name = g_strdup("unnamed");
 
-  str = g_strconcat("GSequencer - ", window->name, NULL);
-  gtk_window_set_title((GtkWindow *) window, str);
+  str = g_strconcat("GSequencer - ",
+		    window->name,
+		    NULL);
+  gtk_window_set_title((GtkWindow *) window,
+		       str);
 
   g_free(str);
 
@@ -230,12 +253,23 @@ ags_window_init(AgsWindow *window)
 
   /* vpaned and scrolled window */
   window->paned = (GtkVPaned *) gtk_vpaned_new();
-  gtk_box_pack_start((GtkBox*) vbox, (GtkWidget*) window->paned, TRUE, TRUE, 0);
+  gtk_box_pack_start((GtkBox*) vbox,
+		     (GtkWidget*) window->paned,
+		     TRUE, TRUE,
+		     0);
 
-  scrolled_window = (GtkWidget *) gtk_scrolled_window_new(NULL, NULL);
+  viewport = gtk_viewport_new(NULL,
+			      NULL);
+  g_object_set(viewport,
+	       "shadow-type", GTK_SHADOW_NONE,
+	       NULL);
   gtk_paned_pack1((GtkPaned *) window->paned,
-		  scrolled_window,
+		  viewport,
 		  TRUE, TRUE);
+  
+  scrolled_window = (GtkWidget *) gtk_scrolled_window_new(NULL, NULL);
+  gtk_container_add(viewport,
+		    scrolled_window);
 
   /* machines rack */
   window->machines = (GtkVBox *) gtk_vbox_new(FALSE, 0);
@@ -246,14 +280,22 @@ ags_window_init(AgsWindow *window)
   window->selected = NULL;
 
   /* editor */
-  window->editor = g_object_new(AGS_TYPE_EDITOR,
-				"homogeneous", FALSE,
-				"spacing", 0,
-				NULL);
+  viewport = gtk_viewport_new(NULL,
+			      NULL);
+  g_object_set(viewport,
+	       "shadow-type", GTK_SHADOW_NONE,
+	       NULL);
   gtk_paned_pack2((GtkPaned *) window->paned,
-		  (GtkWidget *) window->editor,
+		  (GtkWidget *) viewport,
 		  TRUE, TRUE);
 
+  window->notation_editor = g_object_new(AGS_TYPE_NOTATION_EDITOR,
+					 "homogeneous", FALSE,
+					 "spacing", 0,
+					 NULL);
+  gtk_container_add(viewport,
+		    window->notation_editor);
+
   /* navigation */
   window->navigation = g_object_new(AGS_TYPE_NAVIGATION,
 				    "homogeneous", FALSE,
@@ -268,6 +310,8 @@ ags_window_init(AgsWindow *window)
 
   window->automation_window = ags_automation_window_new((GtkWidget *) window);
 
+  window->wave_window = ags_wave_window_new((GtkWidget *) window);
+  
   window->export_window = (AgsExportWindow *) g_object_new(AGS_TYPE_EXPORT_WINDOW,
 							   "main-window", window,
 							   NULL);
@@ -278,6 +322,18 @@ ags_window_init(AgsWindow *window)
   
   window->preferences = NULL;
   window->history_browser = NULL;
+
+  /* load file */
+  if(ags_window_load_file == NULL){
+    ags_window_load_file = g_hash_table_new_full(g_direct_hash, g_direct_equal,
+						 NULL,
+						 NULL);
+  }
+
+  g_hash_table_insert(ags_window_load_file,
+		      window, ags_window_load_file_timeout);
+
+  g_timeout_add(1000, (GSourceFunc) ags_window_load_file_timeout, (gpointer) window);
 }
 
 void
@@ -307,11 +363,15 @@ ags_window_set_property(GObject *gobject,
       
       window->soundcard = soundcard;
 
+      g_object_set(G_OBJECT(window->notation_editor),
+		   "soundcard", soundcard,
+		   NULL);
+
       g_object_set(G_OBJECT(window->automation_window),
 		   "soundcard", soundcard,
 		   NULL);
 
-      g_object_set(G_OBJECT(window->editor),
+      g_object_set(G_OBJECT(window->wave_window),
 		   "soundcard", soundcard,
 		   NULL);
 
@@ -336,7 +396,7 @@ ags_window_set_property(GObject *gobject,
       }
 
       if(application_context != NULL){
-	window->application_mutex = application_context->mutex;
+	window->application_mutex = application_context->obj_mutex;
 	
 	g_object_ref(application_context);
       }
@@ -382,6 +442,7 @@ void
 ags_window_connect(AgsConnectable *connectable)
 {
   AgsWindow *window;
+
   GList *list, *list_start;
 
   window = AGS_WINDOW(connectable);
@@ -409,19 +470,59 @@ ags_window_connect(AgsConnectable *connectable)
 
   g_list_free(list_start);
   
-  ags_connectable_connect(AGS_CONNECTABLE(window->editor));
+  ags_connectable_connect(AGS_CONNECTABLE(window->notation_editor));
   ags_connectable_connect(AGS_CONNECTABLE(window->navigation));
 
   ags_connectable_connect(AGS_CONNECTABLE(window->automation_window));
 
+  ags_connectable_connect(AGS_CONNECTABLE(window->wave_window));
+
   ags_connectable_connect(AGS_CONNECTABLE(window->export_window));
 }
 
 void
 ags_window_disconnect(AgsConnectable *connectable)
 {
-  //TODO:JK:
-  /* implement me */
+  AgsWindow *window;
+
+  GList *list, *list_start;
+
+  window = AGS_WINDOW(connectable);
+
+  if((AGS_WINDOW_CONNECTED & (window->flags)) == 0){
+    return;
+  }
+
+  window->flags &= (~AGS_WINDOW_CONNECTED);
+  
+  g_object_disconnect(window,
+		      "any_signal::delete_event",
+		      G_CALLBACK(ags_window_delete_event_callback),
+		      NULL,
+		      NULL);
+
+  ags_connectable_disconnect(AGS_CONNECTABLE(window->context_menu));
+  ags_connectable_disconnect(AGS_CONNECTABLE(window->menu_bar));
+
+  list_start = 
+    list = gtk_container_get_children((GtkContainer *) window->machines);
+
+  while(list != NULL){
+    ags_connectable_disconnect(AGS_CONNECTABLE(list->data));
+
+    list = list->next;
+  }
+
+  g_list_free(list_start);
+  
+  ags_connectable_disconnect(AGS_CONNECTABLE(window->notation_editor));
+  ags_connectable_disconnect(AGS_CONNECTABLE(window->navigation));
+
+  ags_connectable_disconnect(AGS_CONNECTABLE(window->automation_window));
+
+  ags_connectable_disconnect(AGS_CONNECTABLE(window->wave_window));
+
+  ags_connectable_disconnect(AGS_CONNECTABLE(window->export_window));
 }
 
 void
@@ -431,13 +532,17 @@ ags_window_finalize(GObject *gobject)
 
   window = (AgsWindow *) gobject;
 
+  /* remove message monitor */
+  g_hash_table_remove(ags_window_load_file,
+		      window);
+
   g_object_unref(G_OBJECT(window->soundcard));
   g_object_unref(G_OBJECT(window->export_window));
 
   if(window->name != NULL){
     free(window->name);
   }
-
+  
   /* call parent */
   G_OBJECT_CLASS(ags_window_parent_class)->finalize(gobject);
 }
@@ -447,11 +552,36 @@ ags_window_show(GtkWidget *widget)
 {
   AgsWindow *window;
 
+#ifdef AGS_WITH_QUARTZ
+  GtkosxApplication *app;
+
+  GList *list;
+#endif
+
   window = (AgsWindow *) widget;
 
-  gtk_widget_show((GtkWidget *) window->menu_bar);
-
   GTK_WIDGET_CLASS(ags_window_parent_class)->show(widget);
+
+#ifndef AGS_WITH_QUARTZ
+  gtk_widget_show((GtkWidget *) window->menu_bar);
+#else
+  app = gtkosx_application_get();
+
+  gtk_widget_hide((GtkWidget *) window->menu_bar);
+  gtkosx_application_set_menu_bar(app,
+				  window->menu_bar);
+
+  list = gtk_container_get_children(window->menu_bar);
+
+  while(list != NULL){
+    gtk_widget_show(list->data);
+
+    list = list->next;
+  }
+
+  gtkosx_application_sync_menubar(app);
+  gtkosx_application_ready(app);
+#endif
 }
 
 gboolean
@@ -472,7 +602,7 @@ ags_window_delete_event(GtkWidget *widget, GdkEventAny *event)
  * 
  * Returns: a new #GList containing #AgsMachineCounter for know machines
  * 
- * Since: 0.5.0
+ * Since: 2.0.0
  */
 GList*
 ags_window_standard_machine_counter_alloc()
@@ -487,6 +617,12 @@ ags_window_standard_machine_counter_alloc()
   machine_counter = g_list_prepend(machine_counter,
 				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
 							     AGS_TYPE_MIXER, 0));
+  machine_counter = g_list_prepend(machine_counter,
+				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
+							     AGS_TYPE_SPECTROMETER, 0));
+  machine_counter = g_list_prepend(machine_counter,
+				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
+							     AGS_TYPE_EQUALIZER10, 0));
   machine_counter = g_list_prepend(machine_counter,
 				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
 							     AGS_TYPE_DRUM, 0));
@@ -502,6 +638,9 @@ ags_window_standard_machine_counter_alloc()
   machine_counter = g_list_prepend(machine_counter,
 				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
 							     AGS_TYPE_FFPLAYER, 0));
+  machine_counter = g_list_prepend(machine_counter,
+				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
+							     AGS_TYPE_AUDIOREC, 0));
   machine_counter = g_list_prepend(machine_counter,
 				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
 							     AGS_TYPE_LADSPA_BRIDGE, 0));
@@ -511,6 +650,12 @@ ags_window_standard_machine_counter_alloc()
   machine_counter = g_list_prepend(machine_counter,
 				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
 							     AGS_TYPE_LV2_BRIDGE, 0));
+  machine_counter = g_list_prepend(machine_counter,
+				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
+							     AGS_TYPE_LIVE_DSSI_BRIDGE, 0));
+  machine_counter = g_list_prepend(machine_counter,
+				   ags_machine_counter_alloc(AGS_RECALL_DEFAULT_VERSION, AGS_RECALL_DEFAULT_BUILD_ID,
+							     AGS_TYPE_LIVE_LV2_BRIDGE, 0));
   
   return(machine_counter);
 }
@@ -524,7 +669,7 @@ ags_window_standard_machine_counter_alloc()
  * 
  * Returns: an #AgsMachineCounter
  * 
- * Since: 0.5.0
+ * Since: 2.0.0
  */
 AgsMachineCounter*
 ags_window_find_machine_counter(AgsWindow *window,
@@ -552,7 +697,7 @@ ags_window_find_machine_counter(AgsWindow *window,
  *
  * Keep track of count of machines. Increment window's counter.
  * 
- * Since: 0.5.0
+ * Since: 2.0.0
  */
 void
 ags_window_increment_machine_counter(AgsWindow *window,
@@ -575,7 +720,7 @@ ags_window_increment_machine_counter(AgsWindow *window,
  *
  * Keep track of count of machines. Decrement window's counter.
  * 
- * Since: 0.5.0
+ * Since: 2.0.0
  */
 void
 ags_window_decrement_machine_counter(AgsWindow *window,
@@ -602,7 +747,7 @@ ags_window_decrement_machine_counter(AgsWindow *window,
  * 
  * Returns: an #AgsMachineCounter
  * 
- * Since: 0.5.0
+ * Since: 2.0.0
  */
 AgsMachineCounter*
 ags_machine_counter_alloc(gchar *version, gchar *build_id,
@@ -637,6 +782,78 @@ ags_window_show_error(AgsWindow *window,
   gtk_widget_show_all((GtkWidget *) dialog);
 }
 
+/**
+ * ags_window_load_file_timeout:
+ * @window: the #AgsWindow
+ * 
+ * Load file.
+ *
+ * Returns: %TRUE if proceed with redraw, otherwise %FALSE
+ * 
+ * Since: 2.0.0
+ */
+gboolean
+ags_window_load_file_timeout(AgsWindow *window)
+{
+  AgsGuiThread *gui_thread;
+
+  gui_thread = NULL;
+  
+  if(window->application_context != NULL){
+    gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(window->application_context));
+  }
+  
+  if(gui_thread != NULL &&
+     gui_thread->gtk_thread == NULL){
+      gui_thread->gtk_thread = g_thread_self();
+  }
+
+  if((AGS_WINDOW_TERMINATING & (window->flags)) != 0){
+    ags_application_context_quit(window->application_context);
+
+    return(FALSE);
+  }
+
+  if(g_hash_table_lookup(ags_window_load_file,
+			 window) != NULL){
+    if(window->filename != NULL){
+      AgsSimpleFile *simple_file;
+      
+      gchar *str;
+
+      GError *error;
+
+      simple_file = (AgsSimpleFile *) g_object_new(AGS_TYPE_SIMPLE_FILE,
+						   "application-context", window->application_context,
+						   "filename", window->filename,
+						   NULL);
+      
+      error = NULL;
+      ags_simple_file_open(simple_file,
+			   &error);
+
+      ags_simple_file_read(simple_file);
+      ags_simple_file_close(simple_file);
+      
+      /* set name */
+      window->name = g_strdup(window->filename);
+
+      str = g_strconcat("GSequencer - ",
+			window->name,
+			NULL);
+      
+      gtk_window_set_title((GtkWindow *) window,
+			   str);
+      
+      window->filename = NULL;
+    }
+
+    return(TRUE);
+  }else{
+    return(FALSE);
+  }
+}
+
 /**
  * ags_window_new:
  * @application_context: the application object.
@@ -645,7 +862,7 @@ ags_window_show_error(AgsWindow *window,
  *
  * Returns: a new #AgsWindow
  *
- * Since: 0.3
+ * Since: 2.0.0
  */
 AgsWindow*
 ags_window_new(GObject *application_context)
