@@ -20,47 +20,9 @@
 #include <ags/X/machine/ags_dssi_bridge.h>
 #include <ags/X/machine/ags_dssi_bridge_callbacks.h>
 
-#include <ags/util/ags_id_generator.h>
-
-#include <ags/object/ags_application_context.h>
-#include <ags/object/ags_marshal.h>
-#include <ags/object/ags_connectable.h>
-#include <ags/object/ags_config.h>
-#include <ags/object/ags_soundcard.h>
-#include <ags/object/ags_plugin.h>
-#include <ags/object/ags_seekable.h>
-
-#include <ags/file/ags_file.h>
-#include <ags/file/ags_file_stock.h>
-#include <ags/file/ags_file_id_ref.h>
-#include <ags/file/ags_file_launch.h>
-
-#include <ags/thread/ags_mutex_manager.h>
-#include <ags/thread/ags_thread-posix.h>
-
-#include <ags/plugin/ags_dssi_manager.h>
-#include <ags/plugin/ags_dssi_plugin.h>
-
-#include <ags/audio/ags_input.h>
-#include <ags/audio/ags_recall_factory.h>
-#include <ags/audio/ags_recall.h>
-#include <ags/audio/ags_recall_dssi.h>
-#include <ags/audio/ags_recall_container.h>
-
-#include <ags/audio/recall/ags_delay_audio.h>
-#include <ags/audio/recall/ags_delay_audio_run.h>
-#include <ags/audio/recall/ags_count_beats_audio.h>
-#include <ags/audio/recall/ags_count_beats_audio_run.h>
-#include <ags/audio/recall/ags_envelope_channel.h>
-#include <ags/audio/recall/ags_envelope_channel_run.h>
-#include <ags/audio/recall/ags_play_notation_audio.h>
-#include <ags/audio/recall/ags_play_notation_audio_run.h>
-#include <ags/audio/recall/ags_record_midi_audio.h>
-#include <ags/audio/recall/ags_record_midi_audio_run.h>
-#include <ags/audio/recall/ags_route_dssi_audio.h>
-#include <ags/audio/recall/ags_route_dssi_audio_run.h>
-
-#include <ags/widget/ags_dial.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
+#include <ags/libags-gui.h>
 
 #include <ags/X/ags_window.h>
 #include <ags/X/ags_effect_bridge.h>
@@ -88,8 +50,11 @@ void ags_dssi_bridge_get_property(GObject *gobject,
 				  guint prop_id,
 				  GValue *value,
 				  GParamSpec *param_spec);
+void ags_dssi_bridge_finalize(GObject *gobject);
+
 void ags_dssi_bridge_connect(AgsConnectable *connectable);
 void ags_dssi_bridge_disconnect(AgsConnectable *connectable);
+
 gchar* ags_dssi_bridge_get_version(AgsPlugin *plugin);
 void ags_dssi_bridge_set_version(AgsPlugin *plugin, gchar *version);
 gchar* ags_dssi_bridge_get_build_id(AgsPlugin *plugin);
@@ -99,14 +64,13 @@ void ags_dssi_bridge_set_xml_type(AgsPlugin *plugin, gchar *xml_type);
 void ags_dssi_bridge_read(AgsFile *file, xmlNode *node, AgsPlugin *plugin);
 void ags_dssi_bridge_launch_task(AgsFileLaunch *file_launch, AgsDssiBridge *dssi_bridge);
 xmlNode* ags_dssi_bridge_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin);
-void ags_dssi_bridge_finalize(GObject *gobject);
 
-void ags_dssi_bridge_set_audio_channels(AgsAudio *audio,
-					guint audio_channels, guint audio_channels_old,
-					gpointer data);
-void ags_dssi_bridge_set_pads(AgsAudio *audio, GType type,
-			      guint pads, guint pads_old,
-			      gpointer data);
+void ags_dssi_bridge_resize_audio_channels(AgsMachine *machine,
+					   guint audio_channels, guint audio_channels_old,
+					   gpointer data);
+void ags_dssi_bridge_resize_pads(AgsMachine *machine, GType type,
+				 guint pads, guint pads_old,
+				 gpointer data);
 
 void ags_dssi_bridge_map_recall(AgsMachine *machine);
 
@@ -183,6 +147,7 @@ ags_dssi_bridge_class_init(AgsDssiBridgeClass *dssi_bridge)
 {
   AgsMachineClass *machine;
   GObjectClass *gobject;
+
   GParamSpec *param_spec;
 
   ags_dssi_bridge_parent_class = g_type_class_peek_parent(dssi_bridge);
@@ -201,7 +166,7 @@ ags_dssi_bridge_class_init(AgsDssiBridgeClass *dssi_bridge)
    *
    * The plugins filename.
    * 
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_string("filename",
 				    i18n_pspec("the object file"),
@@ -217,7 +182,7 @@ ags_dssi_bridge_class_init(AgsDssiBridgeClass *dssi_bridge)
    *
    * The effect's name.
    * 
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_string("effect",
 				    i18n_pspec("the effect"),
@@ -233,7 +198,7 @@ ags_dssi_bridge_class_init(AgsDssiBridgeClass *dssi_bridge)
    *
    * The effect's index.
    * 
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_uint("index",
 				  i18n_pspec("index of effect"),
@@ -294,13 +259,13 @@ ags_dssi_bridge_init(AgsDssiBridge *dssi_bridge)
 			 G_CALLBACK(ags_dssi_bridge_parent_set_callback), (gpointer) dssi_bridge);
 
   audio = AGS_MACHINE(dssi_bridge)->audio;
-  audio->flags |= (AGS_AUDIO_OUTPUT_HAS_RECYCLING |
-		   AGS_AUDIO_INPUT_HAS_RECYCLING |
-		   AGS_AUDIO_SYNC |
-		   AGS_AUDIO_ASYNC |
-		   AGS_AUDIO_HAS_NOTATION | 
-		   AGS_AUDIO_NOTATION_DEFAULT |
-		   AGS_AUDIO_REVERSE_MAPPING);
+  ags_audio_set_flags(audio, (AGS_AUDIO_SYNC |
+			      AGS_AUDIO_ASYNC |
+			      AGS_AUDIO_OUTPUT_HAS_RECYCLING |
+			      AGS_AUDIO_INPUT_HAS_RECYCLING));
+  ags_audio_set_ability_flags(audio, (AGS_SOUND_ABILITY_NOTATION));
+  ags_audio_set_behaviour_flags(audio, (AGS_SOUND_BEHAVIOUR_REVERSE_MAPPING |
+					AGS_SOUND_BEHAVIOUR_DEFAULTS_TO_INPUT));
   g_object_set(audio,
 	       "audio-start-mapping", 0,
 	       "audio-end-mapping", 128,
@@ -314,11 +279,11 @@ ags_dssi_bridge_init(AgsDssiBridge *dssi_bridge)
   ags_machine_popup_add_connection_options((AgsMachine *) dssi_bridge,
 					   (AGS_MACHINE_POPUP_MIDI_DIALOG));
 
-  g_signal_connect_after(G_OBJECT(audio), "set-audio-channels",
-			 G_CALLBACK(ags_dssi_bridge_set_audio_channels), NULL);
+  g_signal_connect_after(G_OBJECT(dssi_bridge), "resize-audio-channels",
+			 G_CALLBACK(ags_dssi_bridge_resize_audio_channels), NULL);
 
-  g_signal_connect_after(G_OBJECT(audio), "set-pads",
-			 G_CALLBACK(ags_dssi_bridge_set_pads), NULL);
+  g_signal_connect_after(G_OBJECT(dssi_bridge), "resize-pads",
+			 G_CALLBACK(ags_dssi_bridge_resize_pads), NULL);
     
   dssi_bridge->flags = 0;
 
@@ -505,6 +470,29 @@ ags_dssi_bridge_get_property(GObject *gobject,
   }
 }
 
+void
+ags_dssi_bridge_finalize(GObject *gobject)
+{
+  AgsDssiBridge *dssi_bridge;
+
+  dssi_bridge = (AgsDssiBridge *) gobject;
+  
+  g_object_disconnect(G_OBJECT(dssi_bridge),
+		      "any_signal::resize-audio-channels",
+		      G_CALLBACK(ags_dssi_bridge_resize_audio_channels),
+		      NULL,
+		      "any_signal::resize-pads",
+		      G_CALLBACK(ags_dssi_bridge_resize_pads),
+		      NULL,
+		      NULL);
+
+  g_free(dssi_bridge->filename);
+  g_free(dssi_bridge->effect);
+  
+  /* call parent */
+  G_OBJECT_CLASS(ags_dssi_bridge_parent_class)->finalize(gobject);
+}
+
 void
 ags_dssi_bridge_connect(AgsConnectable *connectable)
 {
@@ -536,7 +524,7 @@ ags_dssi_bridge_disconnect(AgsConnectable *connectable)
   dssi_bridge = AGS_DSSI_BRIDGE(connectable);
 
   g_object_disconnect(G_OBJECT(dssi_bridge->program),
-		      "changed",
+		      "any_signal::changed",
 		      G_CALLBACK(ags_dssi_bridge_program_changed_callback),
 		      dssi_bridge,
 		      NULL);
@@ -669,9 +657,9 @@ ags_dssi_bridge_launch_task(AgsFileLaunch *file_launch, AgsDssiBridge *dssi_brid
     recall = AGS_MACHINE(dssi_bridge)->audio->input->recall;
     
     while((recall = ags_recall_template_find_type(recall, AGS_TYPE_RECALL_DSSI)) != NULL){
-      if(!g_strcmp0(AGS_RECALL_DSSI(recall->data)->filename,
+      if(!g_strcmp0(AGS_RECALL(recall->data)->filename,
 		  dssi_bridge->filename) &&
-	 !g_strcmp0(AGS_RECALL_DSSI(recall->data)->effect,
+	 !g_strcmp0(AGS_RECALL(recall->data)->effect,
 		    dssi_bridge->effect)){
 	break;
       }
@@ -770,91 +758,113 @@ ags_dssi_bridge_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin)
 }
 
 void
-ags_dssi_bridge_finalize(GObject *gobject)
+ags_dssi_bridge_resize_audio_channels(AgsMachine *machine,
+				      guint audio_channels, guint audio_channels_old,
+				      gpointer data)
 {
   AgsDssiBridge *dssi_bridge;
 
-  dssi_bridge = (AgsDssiBridge *) gobject;
-  
-  g_object_disconnect(G_OBJECT(AGS_MACHINE(dssi_bridge)->audio),
-		      "set-audio-channels",
-		      G_CALLBACK(ags_dssi_bridge_set_audio_channels),
-		      NULL,
-		      "set-pads",
-		      G_CALLBACK(ags_dssi_bridge_set_pads),
-		      NULL,
-		      NULL);
-
-  g_free(dssi_bridge->filename);
-  g_free(dssi_bridge->effect);
-  
-  /* call parent */
-  G_OBJECT_CLASS(ags_dssi_bridge_parent_class)->finalize(gobject);
-}
-
-void
-ags_dssi_bridge_set_audio_channels(AgsAudio *audio,
-				   guint audio_channels, guint audio_channels_old,
-				   gpointer data)
-{
-  AgsMachine *machine;
-  AgsDssiBridge *dssi_bridge;
-
+  AgsAudio *audio;
+  AgsChannel *output, *input;
   AgsChannel *channel, *next_pad;
+  AgsRecycling *first_recycling;
   AgsAudioSignal *audio_signal;  
 
-  /* get machine */
-  dssi_bridge = (AgsDssiBridge *) audio->machine;
+  GObject *output_soundcard;
+  
+  guint output_pads, input_pads;
 
-  if(audio->input_pads == 0 &&
-     audio->output_pads == 0){
+  dssi_bridge = (AgsDssiBridge *) machine;  
+
+  audio = machine->audio;
+
+  /* get some fields */
+  g_object_get(audio,
+	       "output", &output,
+	       "input", &input,
+	       "output-pads", &output_pads,
+	       "input-pads", &input_pads,
+	       NULL);
+
+  /* check available */
+  if(input_pads == 0 &&
+     output_pads == 0){
     return;
   }
-  
-  machine = AGS_MACHINE(dssi_bridge);
 
   if(audio_channels > audio_channels_old){
     /* AgsInput */
-    channel = audio->input;
-    
+    channel = input;
+
+    /* resize audio signal */
     while(channel != NULL){
-      next_pad = channel->next_pad;
+      /* get next pad */
+      g_object_get(channel,
+		   "next-pad", &next_pad,
+		   NULL);
+
       channel = ags_channel_nth(channel,
 				audio_channels_old);
       
       while(channel != next_pad){
-	audio_signal = ags_audio_signal_new(audio->soundcard,
-					    (GObject *) channel->first_recycling,
+	/* get some fields */
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
+
+	/* audio signal */
+	audio_signal = ags_audio_signal_new(output_soundcard,
+					    (GObject *) first_recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
 	ags_audio_signal_stream_resize(audio_signal,
 				       1);
-	ags_recycling_add_audio_signal(channel->first_recycling,
+	ags_recycling_add_audio_signal(first_recycling,
 				       audio_signal);
 
-	channel = channel->next;
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
     }
 
     /* AgsOutput */
-    channel = audio->output;
+    channel = output;
 
+    /* resize audio signal */
     while(channel != NULL){
-      next_pad = channel->next_pad;
+      /* get some fields */
+      g_object_get(channel,
+		   "next-pad", &next_pad,
+		   NULL);
+      
       channel = ags_channel_pad_nth(channel,
 				    audio_channels_old);
 
       while(channel != next_pad){
-	audio_signal = ags_audio_signal_new(audio->soundcard,
-					    (GObject *) channel->first_recycling,
+	/* get some fields */
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
+
+	/* audio signal */
+	audio_signal = ags_audio_signal_new(output_soundcard,
+					    (GObject *) first_recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
 	ags_audio_signal_stream_resize(audio_signal,
 				       3);
-	ags_recycling_add_audio_signal(channel->first_recycling,
+	ags_recycling_add_audio_signal(first_recycling,
 				       audio_signal);
 	
-	channel = channel->next;
+
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
     }
 
@@ -872,27 +882,40 @@ ags_dssi_bridge_set_audio_channels(AgsAudio *audio,
 }
 
 void
-ags_dssi_bridge_set_pads(AgsAudio *audio, GType type,
-			 guint pads, guint pads_old,
-			 gpointer data)
+ags_dssi_bridge_resize_pads(AgsMachine *machine, GType type,
+			    guint pads, guint pads_old,
+			    gpointer data)
 {
-  AgsMachine *machine;
   AgsDssiBridge *dssi_bridge;
-
-  AgsChannel *channel;
-  AgsAudioSignal *audio_signal;
   
+  AgsAudio *audio;
+  AgsChannel *output, *input;
+  AgsChannel *channel;
+  AgsRecycling *first_recycling;
+  AgsAudioSignal *audio_signal;
+
+  GObject *output_soundcard;
+  
+  guint audio_channels;
   gboolean grow;
 
+  dssi_bridge = (AgsDssiBridge *) machine;
+
+  audio = machine->audio;
+
+  /* get some fields */
+  g_object_get(audio,
+	       "output", &output,
+	       "input", &input,
+	       "audio-channels", &audio_channels,
+	       NULL);
+  
+  /* check available */
   if(pads == pads_old ||
-     audio->audio_channels == 0){
+     audio_channels == 0){
     return;
   }
-
-  /* get machine */
-  dssi_bridge = (AgsDssiBridge *) audio->machine;
-  machine = AGS_MACHINE(dssi_bridge);
-
+  
   if(pads_old < pads){
     grow = TRUE;
   }else{
@@ -902,20 +925,30 @@ ags_dssi_bridge_set_pads(AgsAudio *audio, GType type,
   if(g_type_is_a(type, AGS_TYPE_INPUT)){
     if(grow){
       /* AgsInput */
-      channel = ags_channel_pad_nth(audio->input,
+      channel = ags_channel_pad_nth(input,
 				    pads_old);
 
       while(channel != NULL){
-	audio_signal = ags_audio_signal_new(audio->soundcard,
-					    (GObject *) channel->first_recycling,
+	/* get some fields */
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
+
+	/* audio signal */
+	audio_signal = ags_audio_signal_new(output_soundcard,
+					    (GObject *) first_recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
 	ags_audio_signal_stream_resize(audio_signal,
 				       1);
-	ags_recycling_add_audio_signal(channel->first_recycling,
+	ags_recycling_add_audio_signal(first_recycling,
 				       audio_signal);
 	
-	channel = channel->next;
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
 
       /* recall */
@@ -930,20 +963,30 @@ ags_dssi_bridge_set_pads(AgsAudio *audio, GType type,
   }else{
     if(grow){
       /* AgsOutput */
-      channel = ags_channel_pad_nth(audio->output,
+      channel = ags_channel_pad_nth(output,
 				    pads_old);
 
       while(channel != NULL){
-	audio_signal = ags_audio_signal_new(audio->soundcard,
-					    (GObject *) channel->first_recycling,
+	/* get some fields */
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
+
+	/* audio signal */
+	audio_signal = ags_audio_signal_new(output_soundcard,
+					    (GObject *) first_recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
 	ags_audio_signal_stream_resize(audio_signal,
 				       3);
-	ags_recycling_add_audio_signal(channel->first_recycling,
+	ags_recycling_add_audio_signal(first_recycling,
 				       audio_signal);
-	
-	channel = channel->next;
+
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
 
       /* recall */
@@ -977,7 +1020,8 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
   AgsRouteDssiAudio *recall_route_dssi_audio;
   AgsRouteDssiAudioRun *recall_route_dssi_audio_run;
 
-  GList *list;
+  GList *start_play, *play;
+  GList *start_recall, *recall;
   
   if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0 ||
      (AGS_MACHINE_PREMAPPED_RECALL & (machine->flags)) != 0){
@@ -988,8 +1032,13 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
 						 AGS_TYPE_WINDOW);
 
   dssi_bridge = (AgsDssiBridge *) machine;
-  audio = machine->audio;
 
+  audio = machine->audio;
+  g_object_get(audio,
+	       "play", &start_play,
+	       "recall", &start_recall,
+	       NULL);
+  
   /* ags-delay */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -998,15 +1047,14 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
 			    0, 0,
 			    (AGS_RECALL_FACTORY_OUTPUT |
 			     AGS_RECALL_FACTORY_ADD |
-			     AGS_RECALL_FACTORY_PLAY |
-			     AGS_RECALL_FACTORY_RECALL),
+			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->play,
+  play = ags_recall_find_type(start_play,
 			      AGS_TYPE_DELAY_AUDIO_RUN);
 
-  if(list != NULL){
-    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(play->data);
     //    AGS_RECALL(play_delay_audio_run)->flags |= AGS_RECALL_PERSISTENT;
   }else{
     play_delay_audio_run = NULL;
@@ -1020,15 +1068,14 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
 			    0, 0,
 			    (AGS_RECALL_FACTORY_OUTPUT |
 			     AGS_RECALL_FACTORY_ADD |
-			     AGS_RECALL_FACTORY_PLAY |
-			     AGS_RECALL_FACTORY_RECALL),
+			     AGS_RECALL_FACTORY_PLAY),
 			    0);
-  
-  list = ags_recall_find_type(audio->play,
+
+  play = ags_recall_find_type(start_play,
 			      AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
 
-  if(list != NULL){
-    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(play->data);
 
     /* set dependency */  
     g_object_set(G_OBJECT(play_count_beats_audio_run),
@@ -1052,11 +1099,11 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_RECALL),
 			    0);
 
-  list = ags_recall_find_type(audio->recall,
-			      AGS_TYPE_ROUTE_DSSI_AUDIO_RUN);
+  recall = ags_recall_find_type(start_recall,
+				AGS_TYPE_ROUTE_DSSI_AUDIO_RUN);
 
-  if(list != NULL){
-    recall_route_dssi_audio_run = AGS_ROUTE_DSSI_AUDIO_RUN(list->data);
+  if(recall != NULL){
+    recall_route_dssi_audio_run = AGS_ROUTE_DSSI_AUDIO_RUN(recall->data);
 
     /* set dependency */
     g_object_set(G_OBJECT(recall_route_dssi_audio_run),
@@ -1082,10 +1129,11 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_RECALL),
 			    0);
 
-  list = ags_recall_find_type(audio->recall, AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
+  recall = ags_recall_find_type(start_recall,
+			      AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
 
-  if(list != NULL){
-    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(list->data);
+  if(recall != NULL){
+    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(recall->data);
     
     /* set dependency */
     g_object_set(G_OBJECT(recall_record_midi_audio_run),
@@ -1109,11 +1157,11 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_RECALL),
 			    0);
 
-  list = ags_recall_find_type(audio->recall,
-			      AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+  recall = ags_recall_find_type(start_recall,
+				AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
 
-  if(list != NULL){
-    recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(list->data);
+  if(recall != NULL){
+    recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(recall->data);
 
     /* set dependency */
     g_object_set(G_OBJECT(recall_notation_audio_run),
@@ -1127,7 +1175,9 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
   }
 
   /* depending on destination */
-  ags_dssi_bridge_input_map_recall(dssi_bridge, 0, 0);
+  ags_dssi_bridge_input_map_recall(dssi_bridge,
+				   0,
+				   0);
 
   /* add to effect bridge */
   ags_effect_bulk_add_effect((AgsEffectBulk *) AGS_EFFECT_BRIDGE(AGS_MACHINE(dssi_bridge)->bridge)->bulk_input,
@@ -1136,268 +1186,242 @@ ags_dssi_bridge_map_recall(AgsMachine *machine)
 			     dssi_bridge->effect);
 
   /* depending on destination */
-  ags_dssi_bridge_output_map_recall(dssi_bridge, 0, 0);
+  ags_dssi_bridge_output_map_recall(dssi_bridge,
+				    0,
+				    0);
 
+  g_list_free(start_play);
+  g_list_free(start_recall);
+  
   /* call parent */
   AGS_MACHINE_CLASS(ags_dssi_bridge_parent_class)->map_recall(machine);
 }
 
 void
-ags_dssi_bridge_input_map_recall(AgsDssiBridge *dssi_bridge, guint audio_channel_start, guint input_pad_start)
+ags_dssi_bridge_input_map_recall(AgsDssiBridge *dssi_bridge,
+				 guint audio_channel_start,
+				 guint input_pad_start)
 {
   AgsAudio *audio;
+  AgsChannel *input;
   AgsChannel *source, *current;
-  
-  AgsConfig *config;
 
-  GList *list;
-
-  gchar *str;
-
-  gboolean performance_mode;
-
-  audio = AGS_MACHINE(dssi_bridge)->audio;
+  guint input_pads;
+  guint audio_channels;
 
   if(dssi_bridge->mapped_input_pad > input_pad_start){
     return;
   }
 
-  source = ags_channel_nth(audio->input,
-			   audio_channel_start + input_pad_start * audio->audio_channels);
+  audio = AGS_MACHINE(dssi_bridge)->audio;
 
-  config = ags_config_get_instance();
+  /* get some fields */
+  g_object_get(audio,
+	       "input", &input,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
+
+  /* source */
+  source = ags_channel_nth(input,
+			   audio_channel_start + input_pad_start * audio_channels);
+
+  /* ags-envelope */
+  ags_recall_factory_create(audio,
+			    NULL, NULL,
+			    "ags-envelope",
+			    audio_channel_start, audio_channels, 
+			    input_pad_start, input_pads,
+			    (AGS_RECALL_FACTORY_INPUT |
+			     AGS_RECALL_FACTORY_PLAY |
+			     AGS_RECALL_FACTORY_RECALL | 
+			     AGS_RECALL_FACTORY_ADD),
+			    0);
 
-  /*  */
   current = source;
 
   while(current != NULL){
-    /* ags-envelope */
-    ags_recall_factory_create(audio,
-			      NULL, NULL,
-			      "ags-envelope",
-			      audio_channel_start, audio->audio_channels, 
-			      current->pad, current->pad + 1,
-			      (AGS_RECALL_FACTORY_INPUT |
-			       AGS_RECALL_FACTORY_PLAY |
-			       AGS_RECALL_FACTORY_RECALL | 
-			       AGS_RECALL_FACTORY_ADD),
-			      0);
+    GList *start_play, *play;
+    GList *start_recall, *recall;
 
-    /* play - use note length */
-    list = ags_recall_template_find_type(current->play, AGS_TYPE_ENVELOPE_CHANNEL);
+    g_object_get(current,
+		 "play", &start_play,
+		 "recall", &start_recall,
+		 NULL);
+    
+    /* play - use note length */    
+    play = ags_recall_template_find_type(start_play,
+					 AGS_TYPE_ENVELOPE_CHANNEL);
 
-    if(list != NULL){
+    if(play != NULL){
       GValue use_note_length_value = {0,};
 
       g_value_init(&use_note_length_value, G_TYPE_BOOLEAN);
       g_value_set_boolean(&use_note_length_value,
 			  TRUE);
 
-      ags_port_safe_write(AGS_ENVELOPE_CHANNEL(list->data)->use_note_length,
+      ags_port_safe_write(AGS_ENVELOPE_CHANNEL(play->data)->use_note_length,
 			  &use_note_length_value);
     }
 
     /* recall - use note length */
-    list = ags_recall_template_find_type(current->recall, AGS_TYPE_ENVELOPE_CHANNEL);
+    recall = ags_recall_template_find_type(start_recall,
+					 AGS_TYPE_ENVELOPE_CHANNEL);
 
-    if(list != NULL){
+    if(recall != NULL){
       GValue use_note_length_value = {0,};
 
       g_value_init(&use_note_length_value, G_TYPE_BOOLEAN);
       g_value_set_boolean(&use_note_length_value,
 			  TRUE);
 
-      ags_port_safe_write(AGS_ENVELOPE_CHANNEL(list->data)->use_note_length,
+      ags_port_safe_write(AGS_ENVELOPE_CHANNEL(recall->data)->use_note_length,
 			  &use_note_length_value);
     }
       
-    current = current->next_pad;
+    g_list_free(start_play);
+    g_list_free(start_recall);
+
+    /* iterate */
+    g_object_get(current,
+		 "next-pad", &current,
+		 NULL);
   }
 
   /* map dependending on output */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "engine-mode");
-  performance_mode = FALSE;
-  
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "performance",
-			  12)){
-    current = source;
-
-    while(current != NULL){
-      /* ags-copy */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-copy",
-				audio_channel_start, audio->audio_channels, 
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      current = current->next_pad;
-    }
-
-    /* set performance mode */
-    performance_mode = TRUE;
-  }else{    
-    current = source;
-
-    while(current != NULL){
-      /* ags-buffer */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-buffer",
-				audio_channel_start, audio->audio_channels, 
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-    
-      current = current->next_pad;
-    }
-  }
-  
-  /*  */
-  current = source;
-
-  while(current != NULL){
-    /* ags-play */
+  if(ags_recall_global_get_rt_safe() ||
+     ags_recall_global_get_performance_mode()){
+    /* ags-copy */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
-			      "ags-play",
-			      audio_channel_start, audio->audio_channels, 
-			      current->pad, current->pad + 1,
+			      "ags-copy",
+			      audio_channel_start, audio_channels, 
+			      input_pad_start, input_pads,
 			      (AGS_RECALL_FACTORY_INPUT |
-			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
+  }else{    
+      /* ags-buffer */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-buffer",
+			      audio_channel_start, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
-
-    current = current->next_pad;
   }
+  
+  /* ags-play */
+  ags_recall_factory_create(audio,
+			    NULL, NULL,
+			    "ags-play",
+			    audio_channel_start, audio_channels, 
+			    input_pad_start, input_pads,
+			    (AGS_RECALL_FACTORY_INPUT |
+			     AGS_RECALL_FACTORY_PLAY |
+			     AGS_RECALL_FACTORY_ADD),
+			    0);
 
-  /*  */
-  current = source;
-
-  while(current != NULL){
-    /* ags-stream */
+  /* ags-stream */
+  if(!(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode())){
     ags_recall_factory_create(audio,
 			      NULL, NULL,
 			      "ags-stream",
-			      audio_channel_start, audio->audio_channels, 
-			      current->pad, current->pad + 1,
+			      audio_channel_start, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_RECALL | 
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
+  }else{
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-rt-stream",
+			      audio_channel_start, audio_channels, 
+			      input_pad_start, input_pads,
 			      (AGS_RECALL_FACTORY_INPUT |
 			       AGS_RECALL_FACTORY_PLAY |
 			       AGS_RECALL_FACTORY_RECALL | 
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
-
-    current = current->next_pad;
   }
   
-  dssi_bridge->mapped_input_pad = audio->input_pads;
+  dssi_bridge->mapped_input_pad = input_pads;
 }
 
 void
-ags_dssi_bridge_output_map_recall(AgsDssiBridge *dssi_bridge, guint audio_channel_start, guint output_pad_start)
+ags_dssi_bridge_output_map_recall(AgsDssiBridge *dssi_bridge,
+				  guint audio_channel_start,
+				  guint output_pad_start)
 {
   AgsAudio *audio;
-  AgsChannel *input, *current;
 
   AgsDelayAudio *recall_delay_audio;
   AgsCountBeatsAudioRun *recall_count_beats_audio_run;
 
-  AgsConfig *config;
-
-  GList *list;
-
-  gchar *str;
-
-  gboolean performance_mode;
-
-  audio = AGS_MACHINE(dssi_bridge)->audio;
+  guint output_pads, input_pads;
+  guint audio_channels;
 
   if(dssi_bridge->mapped_output_pad > output_pad_start){
     return;
   }
 
-  config = ags_config_get_instance();
+  audio = AGS_MACHINE(dssi_bridge)->audio;
 
-  /* map dependending on output */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "engine-mode");
-  performance_mode = FALSE;
+  /* get some fields */
+  g_object_get(audio,
+	       "input-pads", &input_pads,
+	       "output-pads", &output_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
 
   /* remap for input */
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "performance",
-			  12)){
-    input = audio->input;
-
-    while(input != NULL){
-      /* ags-copy */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-copy",
-				audio_channel_start, audio->audio_channels, 
-				input->pad, input->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_REMAP),
-				0);
-    
-      input = input->next_pad;
-    }
-
-    /* set performance mode */
-    performance_mode = TRUE;
+  if(ags_recall_global_get_rt_safe() ||
+     ags_recall_global_get_performance_mode()){
+    /* ags-copy */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-copy",
+			      audio_channel_start, audio_channels, 
+			      0, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_REMAP),
+			      0);
   }else{      
-    input = audio->input;
-
-    while(input != NULL){
-      /* ags-buffer */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-buffer",
-				audio_channel_start, audio->audio_channels, 
-				input->pad, input->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_REMAP),
-				0);
-    
-      input = input->next_pad;
-    }
+    /* ags-buffer */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-buffer",
+			      audio_channel_start, audio_channels, 
+			      0, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_REMAP),
+			      0);    
   }
     
-  if(!performance_mode){
-    current = ags_channel_nth(audio->output,
-			      audio_channel_start + output_pad_start * audio->audio_channels);
-
-    while(current != NULL){
-      /* ags-stream */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-stream",
-				audio_channel_start, audio->audio_channels,
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_OUTPUT |
-				 AGS_RECALL_FACTORY_PLAY |
-				 AGS_RECALL_FACTORY_RECALL | 
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      current = current->next_pad;
-    }
+  if(!(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode())){
+    /* ags-stream */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-stream",
+			      audio_channel_start, audio_channels,
+			      output_pad_start, output_pads,
+			      (AGS_RECALL_FACTORY_OUTPUT |
+			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_RECALL | 
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }
   
-  dssi_bridge->mapped_output_pad = audio->output_pads;
+  dssi_bridge->mapped_output_pad = output_pads;
 }
 
 void
@@ -1409,9 +1433,7 @@ ags_dssi_bridge_load(AgsDssiBridge *dssi_bridge)
 
   AgsDssiPlugin *dssi_plugin;
 
-  AgsConfig *config;
-
-  gchar *str;
+  GList *start_plugin_port, *plugin_port;
 
   void *plugin_so;
   DSSI_Descriptor_Function dssi_descriptor;
@@ -1425,28 +1447,8 @@ ags_dssi_bridge_load(AgsDssiBridge *dssi_bridge)
   unsigned long port_count;
   unsigned long i;
 
-  config = ags_config_get_instance();
-
-  /* samplerate */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_SOUNDCARD,
-			     "samplerate");
-
-  if(str == NULL){
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_SOUNDCARD_0,
-			       "samplerate");
-  }
+  samplerate = ags_soundcard_helper_config_get_samplerate(ags_config_get_instance());
   
-  if(str != NULL){
-    samplerate = g_ascii_strtoull(str,
-				  NULL,
-				  10);
-    free(str);
-  }else{
-    samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
-  }
-
   g_message("ags_dssi_bridge.c - load %s %s",dssi_bridge->filename, dssi_bridge->effect);
  
   /* load plugin */
@@ -1482,6 +1484,10 @@ ags_dssi_bridge_load(AgsDssiBridge *dssi_bridge)
       port_count = plugin_descriptor->LADSPA_Plugin->PortCount;
       port_descriptor = plugin_descriptor->LADSPA_Plugin->PortDescriptors;
 
+      g_object_get(dssi_plugin,
+		   "plugin-port", &start_plugin_port,
+		   NULL);
+
       dssi_bridge->port_values = (LADSPA_Data *) malloc(plugin_descriptor->LADSPA_Plugin->PortCount * sizeof(LADSPA_Data));
       
       for(i = 0; i < port_count; i++){
@@ -1490,24 +1496,23 @@ ags_dssi_bridge_load(AgsDssiBridge *dssi_bridge)
 	     LADSPA_IS_PORT_OUTPUT(port_descriptor[i])){
 	    AgsDssiPlugin *dssi_plugin;
 
-	    GList *list;
-
 	    gchar *specifier;
 	    
 	    dssi_plugin = ags_dssi_manager_find_dssi_plugin(ags_dssi_manager_get_instance(),
 							    dssi_bridge->filename, dssi_bridge->effect);
 
-	    list = AGS_BASE_PLUGIN(dssi_plugin)->port;
+	    plugin_port = start_plugin_port;
  	    specifier = plugin_descriptor->LADSPA_Plugin->PortNames[i];
 
-	    while(list != NULL){
+	    while(plugin_port != NULL){
 	      if(!g_strcmp0(specifier,
-			    AGS_PORT_DESCRIPTOR(list->data)->port_name)){
-		dssi_bridge->port_values[i] = g_value_get_float(AGS_PORT_DESCRIPTOR(list->data)->default_value);
+			    AGS_PLUGIN_PORT(plugin_port->data)->port_name)){
+		dssi_bridge->port_values[i] = g_value_get_float(AGS_PLUGIN_PORT(plugin_port->data)->default_value);
+
 		break;
 	      }
 
-	      list = list->next;
+	      plugin_port = plugin_port->next;
 	    }
 	    
 	    plugin_descriptor->LADSPA_Plugin->connect_port(dssi_bridge->ladspa_handle,
@@ -1527,6 +1532,8 @@ ags_dssi_bridge_load(AgsDssiBridge *dssi_bridge)
 			     -1);
 	}
       }
+
+      g_list_free(start_plugin_port);
     }
   }
   
@@ -1540,11 +1547,11 @@ ags_dssi_bridge_load(AgsDssiBridge *dssi_bridge)
  * @filename: the plugin.so
  * @effect: the effect
  *
- * Creates an #AgsDssiBridge
+ * Create a new instance of #AgsDssiBridge
  *
- * Returns: a new #AgsDssiBridge
+ * Returns: the new #AgsDssiBridge
  *
- * Since: 0.7.0
+ * Since: 2.0.0
  */
 AgsDssiBridge*
 ags_dssi_bridge_new(GObject *soundcard,
@@ -1558,7 +1565,7 @@ ags_dssi_bridge_new(GObject *soundcard,
 
   if(soundcard != NULL){
     g_object_set(G_OBJECT(AGS_MACHINE(dssi_bridge)->audio),
-		 "soundcard", soundcard,
+		 "output-soundcard", soundcard,
 		 NULL);
   }
 
