@@ -20,55 +20,12 @@
 #include <ags/X/machine/ags_live_lv2_bridge.h>
 #include <ags/X/machine/ags_live_lv2_bridge_callbacks.h>
 
-#include <ags/util/ags_id_generator.h>
-
-#include <ags/lib/ags_string_util.h>
-
-#include <ags/object/ags_application_context.h>
-#include <ags/object/ags_marshal.h>
-#include <ags/object/ags_connectable.h>
-#include <ags/object/ags_config.h>
-#include <ags/object/ags_soundcard.h>
-#include <ags/object/ags_plugin.h>
-#include <ags/object/ags_seekable.h>
-
-#include <ags/file/ags_file.h>
-#include <ags/file/ags_file_stock.h>
-#include <ags/file/ags_file_id_ref.h>
-#include <ags/file/ags_file_launch.h>
-
-#include <ags/thread/ags_mutex_manager.h>
-#include <ags/thread/ags_thread-posix.h>
-
-#include <ags/plugin/ags_lv2_manager.h>
-#include <ags/plugin/ags_lv2ui_manager.h>
-#include <ags/plugin/ags_base_plugin.h>
-#include <ags/plugin/ags_lv2_plugin.h>
-#include <ags/plugin/ags_lv2_preset.h>
-#include <ags/plugin/ags_lv2ui_plugin.h>
-#include <ags/plugin/ags_lv2_conversion.h>
-
-#include <ags/audio/ags_output.h>
-#include <ags/audio/ags_input.h>
-#include <ags/audio/ags_recall_factory.h>
-#include <ags/audio/ags_recall.h>
-#include <ags/audio/ags_recall_lv2.h>
-#include <ags/audio/ags_recall_container.h>
-
-#include <ags/audio/recall/ags_delay_audio.h>
-#include <ags/audio/recall/ags_delay_audio_run.h>
-#include <ags/audio/recall/ags_count_beats_audio.h>
-#include <ags/audio/recall/ags_count_beats_audio_run.h>
-#include <ags/audio/recall/ags_play_lv2_audio.h>
-#include <ags/audio/recall/ags_play_lv2_audio_run.h>
-#include <ags/audio/recall/ags_record_midi_audio.h>
-#include <ags/audio/recall/ags_record_midi_audio_run.h>
-
-#include <ags/widget/ags_led.h>
-#include <ags/widget/ags_hindicator.h>
-#include <ags/widget/ags_dial.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
+#include <ags/libags-gui.h>
 
 #include <ags/X/ags_window.h>
+#include <ags/X/ags_machine.h>
 #include <ags/X/ags_effect_bridge.h>
 #include <ags/X/ags_effect_bulk.h>
 #include <ags/X/ags_bulk_member.h>
@@ -88,8 +45,10 @@ void ags_live_lv2_bridge_get_property(GObject *gobject,
 				      GValue *value,
 				      GParamSpec *param_spec);
 void ags_live_lv2_bridge_finalize(GObject *gobject);
+
 void ags_live_lv2_bridge_connect(AgsConnectable *connectable);
 void ags_live_lv2_bridge_disconnect(AgsConnectable *connectable);
+
 gchar* ags_live_lv2_bridge_get_version(AgsPlugin *plugin);
 void ags_live_lv2_bridge_set_version(AgsPlugin *plugin, gchar *version);
 gchar* ags_live_lv2_bridge_get_build_id(AgsPlugin *plugin);
@@ -100,12 +59,12 @@ void ags_live_lv2_bridge_read(AgsFile *file, xmlNode *node, AgsPlugin *plugin);
 void ags_live_lv2_bridge_launch_task(AgsFileLaunch *file_launch, AgsLiveLv2Bridge *live_lv2_bridge);
 xmlNode* ags_live_lv2_bridge_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin);
 
-void ags_live_lv2_bridge_set_audio_channels(AgsAudio *audio,
-					    guint audio_channels, guint audio_channels_old,
-					    gpointer data);
-void ags_live_lv2_bridge_set_pads(AgsAudio *audio, GType type,
-				  guint pads, guint pads_old,
-				  gpointer data);
+void ags_live_lv2_bridge_resize_audio_channels(AgsMachine *machine,
+					       guint audio_channels, guint audio_channels_old,
+					       gpointer data);
+void ags_live_lv2_bridge_resize_pads(AgsMachine *machine, GType channel_type,
+				     guint pads, guint pads_old,
+				     gpointer data);
 
 void ags_live_lv2_bridge_map_recall(AgsMachine *machine);
 
@@ -210,7 +169,7 @@ ags_live_lv2_bridge_class_init(AgsLiveLv2BridgeClass *live_lv2_bridge)
    *
    * The plugin's filename.
    * 
-   * Since: 0.7.134
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_string("filename",
 				    "the object file",
@@ -226,7 +185,7 @@ ags_live_lv2_bridge_class_init(AgsLiveLv2BridgeClass *live_lv2_bridge)
    *
    * The effect's name.
    * 
-   * Since: 0.7.134
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_string("effect",
 				    "the effect",
@@ -242,7 +201,7 @@ ags_live_lv2_bridge_class_init(AgsLiveLv2BridgeClass *live_lv2_bridge)
    *
    * The uri's name.
    * 
-   * Since: 0.7.134
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_string("uri",
 				    "the uri",
@@ -258,7 +217,7 @@ ags_live_lv2_bridge_class_init(AgsLiveLv2BridgeClass *live_lv2_bridge)
    *
    * The uri's index.
    * 
-   * Since: 0.7.134
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_ulong("index",
 				   "index of uri",
@@ -277,7 +236,7 @@ ags_live_lv2_bridge_class_init(AgsLiveLv2BridgeClass *live_lv2_bridge)
    * If has-midi is set to %TRUE appropriate flag is set
    * to audio in order to become a sequencer.
    * 
-   * Since: 0.7.134
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_boolean("has-midi",
 				     "has-midi",
@@ -294,7 +253,7 @@ ags_live_lv2_bridge_class_init(AgsLiveLv2BridgeClass *live_lv2_bridge)
    * If has-gui is set to %TRUE 128 inputs are allocated and appropriate flag is set
    * to audio in order to become a sequencer.
    * 
-   * Since: 0.7.134
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_boolean("has-gui",
 				     "has-gui",
@@ -310,7 +269,7 @@ ags_live_lv2_bridge_class_init(AgsLiveLv2BridgeClass *live_lv2_bridge)
    *
    * The plugin's GUI filename.
    * 
-   * Since: 0.7.134
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_string("gui-filename",
 				    "the GUI object file",
@@ -326,7 +285,7 @@ ags_live_lv2_bridge_class_init(AgsLiveLv2BridgeClass *live_lv2_bridge)
    *
    * The GUI's uri name.
    * 
-   * Since: 0.7.134
+   * Since: 1.0.0
    */
   param_spec =  g_param_spec_string("gui-uri",
 				    "the gui-uri",
@@ -379,6 +338,9 @@ ags_live_lv2_bridge_init(AgsLiveLv2Bridge *live_lv2_bridge)
 
   AgsAudio *audio;
 
+  g_signal_connect_after((GObject *) live_lv2_bridge, "parent-set",
+			 G_CALLBACK(ags_live_lv2_bridge_parent_set_callback), (gpointer) live_lv2_bridge);
+
   if(ags_live_lv2_bridge_lv2ui_handle == NULL){
     ags_live_lv2_bridge_lv2ui_handle = g_hash_table_new_full(g_direct_hash, g_direct_equal,
 							   NULL,
@@ -392,14 +354,15 @@ ags_live_lv2_bridge_init(AgsLiveLv2Bridge *live_lv2_bridge)
   }
 
   audio = AGS_MACHINE(live_lv2_bridge)->audio;
-  audio->flags |= (AGS_AUDIO_OUTPUT_HAS_RECYCLING |
-		   AGS_AUDIO_INPUT_HAS_RECYCLING |
-		   AGS_AUDIO_SYNC |
-		   AGS_AUDIO_ASYNC |
-		   AGS_AUDIO_HAS_NOTATION |  
-		   AGS_AUDIO_SKIP_INPUT |
-		   AGS_AUDIO_REVERSE_MAPPING);
-  audio->flags &= (~AGS_AUDIO_NOTATION_DEFAULT);
+  ags_audio_set_flags(audio, (AGS_AUDIO_SYNC |
+			      AGS_AUDIO_ASYNC |
+			      AGS_AUDIO_OUTPUT_HAS_RECYCLING |
+			      AGS_AUDIO_INPUT_HAS_RECYCLING |
+			      AGS_AUDIO_SKIP_INPUT));
+  ags_audio_set_ability_flags(audio, (AGS_SOUND_ABILITY_NOTATION));
+  ags_audio_set_behaviour_flags(audio, (AGS_SOUND_BEHAVIOUR_REVERSE_MAPPING));
+  //  audio->flags &= (~AGS_AUDIO_NOTATION_DEFAULT);
+  
   g_object_set(audio,
 	       "audio-start-mapping", 0,
 	       "audio-end-mapping", 128,
@@ -413,11 +376,11 @@ ags_live_lv2_bridge_init(AgsLiveLv2Bridge *live_lv2_bridge)
   AGS_MACHINE(live_lv2_bridge)->flags |= (AGS_MACHINE_IS_SYNTHESIZER |
 					  AGS_MACHINE_REVERSE_NOTATION);
 
-  g_signal_connect_after(G_OBJECT(audio), "set_audio_channels",
-			 G_CALLBACK(ags_live_lv2_bridge_set_audio_channels), NULL);
+  g_signal_connect_after(G_OBJECT(live_lv2_bridge), "resize-audio-channels",
+			 G_CALLBACK(ags_live_lv2_bridge_resize_audio_channels), NULL);
   
-  g_signal_connect_after(G_OBJECT(audio), "set_pads",
-			 G_CALLBACK(ags_live_lv2_bridge_set_pads), NULL);
+  g_signal_connect_after(G_OBJECT(live_lv2_bridge), "resize-pads",
+			 G_CALLBACK(ags_live_lv2_bridge_resize_pads), NULL);
   
   live_lv2_bridge->flags = 0;
 
@@ -862,9 +825,9 @@ ags_live_lv2_bridge_launch_task(AgsFileLaunch *file_launch, AgsLiveLv2Bridge *li
     recall = AGS_MACHINE(live_lv2_bridge)->audio->input->recall;
     
     while((recall = ags_recall_template_find_type(recall, AGS_TYPE_PLAY_LV2_AUDIO)) != NULL){
-      if(!g_strcmp0(AGS_PLAY_LV2_AUDIO(recall->data)->filename,
+      if(!g_strcmp0(AGS_RECALL(recall->data)->filename,
 		    live_lv2_bridge->filename) &&
-	 !g_strcmp0(AGS_PLAY_LV2_AUDIO(recall->data)->effect,
+	 !g_strcmp0(AGS_RECALL(recall->data)->effect,
 		    live_lv2_bridge->effect)){
 	break;
       }
@@ -955,69 +918,109 @@ ags_live_lv2_bridge_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin)
 }
 
 void
-ags_live_lv2_bridge_set_audio_channels(AgsAudio *audio,
-				       guint audio_channels, guint audio_channels_old,
-				       gpointer data)
+ags_live_lv2_bridge_resize_audio_channels(AgsMachine *machine,
+					  guint audio_channels, guint audio_channels_old,
+					  gpointer data)
 {
-  AgsMachine *machine;
   AgsLiveLv2Bridge *live_lv2_bridge;
 
+  AgsAudio *audio;
+  AgsChannel *output, *input;
   AgsChannel *channel, *next_pad;
+  AgsRecycling *first_recycling;
   AgsAudioSignal *audio_signal;  
 
-  /* get machine */
-  live_lv2_bridge = (AgsLiveLv2Bridge *) audio->machine;
-  machine = AGS_MACHINE(live_lv2_bridge);
+  GObject *output_soundcard;
   
-  if(audio->input_pads == 0 &&
-     audio->output_pads == 0){
+  guint output_pads, input_pads;
+
+  live_lv2_bridge = (AgsLiveLv2Bridge *) machine;
+
+  audio = machine->audio;
+
+  /* get some fields */
+  g_object_get(audio,
+	       "output", &output,
+	       "input", &input,
+	       "output-pads", &output_pads,
+	       "input-pads", &input_pads,
+	       NULL);
+  
+  if(input_pads == 0 &&
+     output_pads == 0){
     return;
   }
 
   if(audio_channels > audio_channels_old){
     /* AgsInput */
-    channel = audio->input;
+    channel = input;
 
     while(channel != NULL){
-      next_pad = channel->next_pad;
+      /* get next pad */
+      g_object_get(channel,
+		   "next-pad", &next_pad,
+		   NULL);
+
       channel = ags_channel_nth(channel,
 				audio_channels_old);
 
       while(channel != next_pad){
-	audio_signal = ags_audio_signal_new(audio->soundcard,
-					    (GObject *) channel->first_recycling,
+	/* get some fields */
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
+
+	/* audio signal */
+	audio_signal = ags_audio_signal_new(output_soundcard,
+					    (GObject *) first_recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
-	//	audio_signal->loop_start = 0;
-	//	audio_signal->loop_end = audio_signal->buffer_size;
 	ags_audio_signal_stream_resize(audio_signal,
 				       1);
-	ags_recycling_add_audio_signal(channel->first_recycling,
+	ags_recycling_add_audio_signal(first_recycling,
 				       audio_signal);
 	
-	channel = channel->next;
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
     }
 
     /* AgsOutput */
-    channel = audio->output;
+    channel = output;
 
     while(channel != NULL){
-      next_pad = channel->next_pad;
+      /* get next pad */
+      g_object_get(channel,
+		   "next-pad", &next_pad,
+		   NULL);
+
       channel = ags_channel_pad_nth(channel,
 				    audio_channels_old);
 
       while(channel != next_pad){
-	audio_signal = ags_audio_signal_new(audio->soundcard,
-					    (GObject *) channel->first_recycling,
+	/* get some fields */
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
+
+	/* audio signal */
+	audio_signal = ags_audio_signal_new(output_soundcard,
+					    (GObject *) first_recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
 	ags_audio_signal_stream_resize(audio_signal,
 				       3);
-	ags_recycling_add_audio_signal(channel->first_recycling,
+	ags_recycling_add_audio_signal(first_recycling,
 				       audio_signal);
 	
-	channel = channel->next;
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
     }
 
@@ -1035,52 +1038,72 @@ ags_live_lv2_bridge_set_audio_channels(AgsAudio *audio,
 }
 
 void
-ags_live_lv2_bridge_set_pads(AgsAudio *audio, GType type,
-			     guint pads, guint pads_old,
-			     gpointer data)
+ags_live_lv2_bridge_resize_pads(AgsMachine *machine, GType channel_type,
+				guint pads, guint pads_old,
+				gpointer data)
 {
-  AgsMachine *machine;
   AgsLiveLv2Bridge *live_lv2_bridge;
 
+  AgsAudio *audio;
+  AgsChannel *output, *input;
   AgsChannel *channel;
+  AgsRecycling *first_recycling;
   AgsAudioSignal *audio_signal;
   
+  GObject *output_soundcard;
+  
+  guint audio_channels;
   gboolean grow;
 
+  live_lv2_bridge = (AgsLiveLv2Bridge *) machine;
+
+  audio = machine->audio;
+
+  /* get some fields */
+  g_object_get(audio,
+	       "output", &output,
+	       "input", &input,
+	       "audio-channels", &audio_channels,
+	       NULL);
+  
   if(pads == pads_old ||
-     audio->audio_channels == 0){
+     audio_channels == 0){
     return;
   }
 
-  /* get machine */
-  live_lv2_bridge = (AgsLiveLv2Bridge *) audio->machine;
-  machine = AGS_MACHINE(live_lv2_bridge);
-
   if(pads_old < pads){
     grow = TRUE;
   }else{
     grow = FALSE;
   }
   
-  if(g_type_is_a(type, AGS_TYPE_INPUT)){
+  if(g_type_is_a(channel_type, AGS_TYPE_INPUT)){
     if(grow){
       /* AgsInput */
-      channel = ags_channel_pad_nth(audio->input,
+      channel = ags_channel_pad_nth(input,
 				    pads_old);
 
       while(channel != NULL){
-	audio_signal = ags_audio_signal_new(audio->soundcard,
-					    (GObject *) channel->first_recycling,
+	/* get some fields */
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
+
+	/* audio signal */
+	audio_signal = ags_audio_signal_new(output_soundcard,
+					    (GObject *) first_recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
-	//	audio_signal->loop_start = 0;
-	//	audio_signal->loop_end = audio_signal->buffer_size;
 	ags_audio_signal_stream_resize(audio_signal,
 				       1);
-	ags_recycling_add_audio_signal(channel->first_recycling,
+	ags_recycling_add_audio_signal(first_recycling,
 				       audio_signal);
 	
-	channel = channel->next;
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
 
       /* recall */
@@ -1095,20 +1118,30 @@ ags_live_lv2_bridge_set_pads(AgsAudio *audio, GType type,
   }else{
     if(grow){
       /* AgsOutput */
-      channel = ags_channel_pad_nth(audio->output,
+      channel = ags_channel_pad_nth(output,
 				    pads_old);
 
       while(channel != NULL){
-	audio_signal = ags_audio_signal_new(audio->soundcard,
-					    (GObject *) channel->first_recycling,
+	/* get some fields */
+	g_object_get(channel,
+		     "output-soundcard", &output_soundcard,
+		     "first-recycling", &first_recycling,
+		     NULL);
+
+	/* audio signal */
+	audio_signal = ags_audio_signal_new(output_soundcard,
+					    (GObject *) first_recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
 	ags_audio_signal_stream_resize(audio_signal,
 				       3);
-	ags_recycling_add_audio_signal(channel->first_recycling,
+	ags_recycling_add_audio_signal(first_recycling,
 				       audio_signal);
 	
-	channel = channel->next;
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
       }
 
       /* recall */
@@ -1140,7 +1173,8 @@ ags_live_lv2_bridge_map_recall(AgsMachine *machine)
   AgsPlayLv2Audio *play_lv2_audio;
   AgsPlayLv2AudioRun *play_lv2_audio_run;
 
-  GList *list;
+  GList *start_play, *play;
+  GList *start_recall, *recall;
   
   if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0 ||
      (AGS_MACHINE_PREMAPPED_RECALL & (machine->flags)) != 0){
@@ -1151,7 +1185,12 @@ ags_live_lv2_bridge_map_recall(AgsMachine *machine)
 						 AGS_TYPE_WINDOW);
 
   live_lv2_bridge = (AgsLiveLv2Bridge *) machine;
+
   audio = machine->audio;
+  g_object_get(audio,
+	       "play", &start_play,
+	       "recall", &start_recall,
+	       NULL);
 
   /* ags-delay */
   ags_recall_factory_create(audio,
@@ -1165,16 +1204,16 @@ ags_live_lv2_bridge_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_RECALL),
 			    0);
 
-  list = ags_recall_find_type(audio->play,
+  play = ags_recall_find_type(start_play,
 			      AGS_TYPE_DELAY_AUDIO_RUN);
 
-  if(list != NULL){
-    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(play->data);
     //    AGS_RECALL(play_delay_audio_run)->flags |= AGS_RECALL_PERSISTENT;
   }else{
     play_delay_audio_run = NULL;
   }
-  
+
   /* ags-count-beats */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -1187,11 +1226,11 @@ ags_live_lv2_bridge_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_RECALL),
 			    0);
   
-  list = ags_recall_find_type(audio->play,
+  play = ags_recall_find_type(start_play,
 			      AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
 
-  if(list != NULL){
-    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(play->data);
 
     /* set dependency */  
     g_object_set(G_OBJECT(play_count_beats_audio_run),
@@ -1215,10 +1254,11 @@ ags_live_lv2_bridge_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_RECALL),
 			    0);
 
-  list = ags_recall_find_type(audio->recall, AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
+  recall = ags_recall_find_type(start_recall,
+				AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
 
-  if(list != NULL){
-    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(list->data);
+  if(recall != NULL){
+    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(recall->data);
     
     /* set dependency */
     g_object_set(G_OBJECT(recall_record_midi_audio_run),
@@ -1243,11 +1283,11 @@ ags_live_lv2_bridge_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_BULK),
 			    0);
 
-  list = ags_recall_find_type(audio->play,
+  play = ags_recall_find_type(start_play,
 			      AGS_TYPE_PLAY_LV2_AUDIO);
   
-  if(list != NULL){
-    play_lv2_audio = AGS_PLAY_LV2_AUDIO(list->data);
+  if(play != NULL){
+    play_lv2_audio = AGS_PLAY_LV2_AUDIO(play->data);
     
     g_object_set(play_lv2_audio,
 		 "filename", live_lv2_bridge->filename,
@@ -1258,11 +1298,11 @@ ags_live_lv2_bridge_map_recall(AgsMachine *machine)
     ags_play_lv2_audio_load_ports(play_lv2_audio);
   }
 
-  list = ags_recall_find_type(audio->play,
+  play = ags_recall_find_type(start_play,
 			      AGS_TYPE_PLAY_LV2_AUDIO_RUN);
 
-  if(list != NULL){
-    play_lv2_audio_run = AGS_PLAY_LV2_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_lv2_audio_run = AGS_PLAY_LV2_AUDIO_RUN(play->data);
 
     /* set dependency */
     g_object_set(G_OBJECT(play_lv2_audio_run),
@@ -1276,10 +1316,14 @@ ags_live_lv2_bridge_map_recall(AgsMachine *machine)
   }
 
   /* depending on destination */
-  ags_live_lv2_bridge_input_map_recall(live_lv2_bridge, 0, 0);
+  ags_live_lv2_bridge_input_map_recall(live_lv2_bridge,
+				       0,
+				       0);
 
   /* depending on destination */
-  ags_live_lv2_bridge_output_map_recall(live_lv2_bridge, 0, 0);
+  ags_live_lv2_bridge_output_map_recall(live_lv2_bridge,
+					0,
+					0);
 
   /* call parent */
   AGS_MACHINE_CLASS(ags_live_lv2_bridge_parent_class)->map_recall(machine);
@@ -1339,35 +1383,49 @@ ags_live_lv2_bridge_load_program(AgsLiveLv2Bridge *live_lv2_bridge)
     uint32_t i;
 
     if(live_lv2_bridge->lv2_handle == NULL){
+      guint samplerate;
+      guint buffer_size;
+
+      g_object_get(AGS_MACHINE(live_lv2_bridge)->audio,
+		   "samplerate", &samplerate,
+		   "buffer-size", &buffer_size,
+		   NULL);
+      
       live_lv2_bridge->lv2_handle = (LV2_Handle *) ags_base_plugin_instantiate(lv2_plugin,
-									       AGS_MACHINE(live_lv2_bridge)->audio->samplerate);
+									       samplerate, buffer_size);
     }
 
     if(live_lv2_bridge->port_value == NULL){
-      GList *port_descriptor;
+      GList *start_plugin_port, *plugin_port;
       
       guint port_count;
 
-      port_count = g_list_length(AGS_BASE_PLUGIN(lv2_plugin)->port);
+      port_count = g_list_length(AGS_BASE_PLUGIN(lv2_plugin)->plugin_port);
 
       if(port_count > 0){
 	live_lv2_bridge->port_value = (float *) malloc(port_count * sizeof(float));
       }
 
-      port_descriptor = AGS_BASE_PLUGIN(lv2_plugin)->port;
+      g_object_get(lv2_plugin,
+		   "plugin-port", &start_plugin_port,
+		   NULL);
       
-      for(i = 0; port_descriptor != NULL;){
-	if((AGS_PORT_DESCRIPTOR_CONTROL & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
-	  if((AGS_PORT_DESCRIPTOR_INPUT & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
+      plugin_port = start_plugin_port;
+      
+      for(i = 0; plugin_port != NULL;){
+	if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_CONTROL)){
+	  if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_INPUT)){
 	    plugin_descriptor->connect_port(live_lv2_bridge->lv2_handle[0],
-					    AGS_PORT_DESCRIPTOR(port_descriptor->data)->port_index,
+					    AGS_PLUGIN_PORT(plugin_port->data)->port_index,
 					    &(live_lv2_bridge->port_value[i]));
 	    i++;
 	  }
 	}
 
-	port_descriptor = port_descriptor->next;
+	plugin_port = plugin_port->next;
       }
+
+      g_list_free(start_plugin_port);
     }
     
     if(live_lv2_bridge->program == NULL){
@@ -1537,17 +1595,14 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
 
   AgsLv2Plugin *lv2_plugin;
 
-  AgsConfig *config;
-
   void *plugin_so;
 
-  GList *port;
   GList *list;
-
-  gchar *str;
+  GList *start_plugin_port, *plugin_port;
   
-  unsigned long samplerate;
-  unsigned long effect_index;
+  guint samplerate;
+  guint buffer_size;
+  guint effect_index;
   gdouble step;
   unsigned long port_count;
   gboolean has_output_port;
@@ -1569,27 +1624,9 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
 	       "uri", lv2_plugin->uri,
 	       NULL);
 
-  config = ags_config_get_instance();
-
-  /* samplerate */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_SOUNDCARD,
-			     "samplerate");
-
-  if(str == NULL){
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_SOUNDCARD_0,
-			       "samplerate");
-  }
-  
-  if(str != NULL){
-    samplerate = g_ascii_strtoull(str,
-				  NULL,
-				  10);
-    free(str);
-  }else{
-    samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
-  }
+  /* samplerate and buffer size */
+  samplerate = ags_soundcard_helper_config_get_samplerate(ags_config_get_instance());
+  buffer_size = ags_soundcard_helper_config_get_buffer_size(ags_config_get_instance());
 
   g_message("ags_live_lv2_bridge.c - load %s %s", live_lv2_bridge->filename, live_lv2_bridge->effect);
  
@@ -1616,13 +1653,15 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
   }
 
   /* load ports */
-  port = AGS_BASE_PLUGIN(lv2_plugin)->port;
+  g_object_get(lv2_plugin,
+	       "plugin-port", &start_plugin_port,
+	       NULL);
 
-  port_count = g_list_length(port);
+  port_count = g_list_length(start_plugin_port);
   k = 0;
 
-  while(port != NULL){
-    if((AGS_PORT_DESCRIPTOR_CONTROL & (AGS_PORT_DESCRIPTOR(port->data)->flags)) != 0){
+  while(plugin_port != NULL){
+    if(ags_plugin_port_test_flags(AGS_PLUGIN_PORT(plugin_port->data), AGS_PLUGIN_PORT_CONTROL)){
       GtkWidget *child_widget;
 
       AgsLv2Conversion *lv2_conversion;
@@ -1644,14 +1683,14 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
 			 y + 1, AGS_EFFECT_BULK_COLUMNS_COUNT);
       }
 
-      if((AGS_PORT_DESCRIPTOR_TOGGLED & (AGS_PORT_DESCRIPTOR(port->data)->flags)) != 0){
-	if((AGS_PORT_DESCRIPTOR_OUTPUT & (AGS_PORT_DESCRIPTOR(port->data)->flags)) != 0){
+      if(ags_plugin_port_test_flags(AGS_PLUGIN_PORT(plugin_port->data), AGS_PLUGIN_PORT_TOGGLED)){
+	if(ags_plugin_port_test_flags(AGS_PLUGIN_PORT(plugin_port->data), AGS_PLUGIN_PORT_OUTPUT)){
 	  widget_type = AGS_TYPE_LED;
 	}else{
 	  widget_type = GTK_TYPE_TOGGLE_BUTTON;
 	}
       }else{
-	if((AGS_PORT_DESCRIPTOR_OUTPUT & (AGS_PORT_DESCRIPTOR(port->data)->flags)) != 0){
+	if(ags_plugin_port_test_flags(AGS_PLUGIN_PORT(plugin_port->data), AGS_PLUGIN_PORT_OUTPUT)){
 	  widget_type = AGS_TYPE_HINDICATOR;
 	}else{
 	  widget_type = AGS_TYPE_DIAL;
@@ -1660,8 +1699,8 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
 
       step_count = AGS_DIAL_DEFAULT_PRECISION;
 
-      if((AGS_PORT_DESCRIPTOR_INTEGER & (AGS_PORT_DESCRIPTOR(port->data)->flags)) != 0){
-	step_count = AGS_PORT_DESCRIPTOR(port->data)->scale_steps;
+      if(ags_plugin_port_test_flags(AGS_PLUGIN_PORT(plugin_port->data), AGS_PLUGIN_PORT_INTEGER)){
+	step_count = AGS_PLUGIN_PORT(plugin_port->data)->scale_steps;
 
 	disable_seemless = TRUE;	
       }
@@ -1674,11 +1713,11 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
 				     port_count);
       bulk_member = (AgsBulkMember *) g_object_new(AGS_TYPE_BULK_MEMBER,
 						   "widget-type", widget_type,
-						   "widget-label", AGS_PORT_DESCRIPTOR(port->data)->port_name,
+						   "widget-label", AGS_PLUGIN_PORT(plugin_port->data)->port_name,
 						   "plugin-name", plugin_name,
 						   "filename", live_lv2_bridge->filename,
 						   "effect", live_lv2_bridge->effect,
-						   "specifier", AGS_PORT_DESCRIPTOR(port->data)->port_name,
+						   "specifier", AGS_PLUGIN_PORT(plugin_port->data)->port_name,
 						   "control-port", control_port,
 						   "steps", step_count,
 						   NULL);
@@ -1690,7 +1729,7 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
       /* lv2 conversion */
       lv2_conversion = NULL;
 
-      if((AGS_PORT_DESCRIPTOR_LOGARITHMIC & (AGS_PORT_DESCRIPTOR(port->data)->flags)) != 0){
+      if(ags_plugin_port_test_flags(AGS_PLUGIN_PORT(plugin_port->data), AGS_PLUGIN_PORT_LOGARITHMIC)){
 	if(lv2_conversion == NULL ||
 	   !AGS_IS_LV2_CONVERSION(lv2_conversion)){
 	  lv2_conversion = ags_lv2_conversion_new();
@@ -1702,11 +1741,11 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
       bulk_member->conversion = (AgsConversion *) lv2_conversion;
 
       /* child widget */
-      if((AGS_PORT_DESCRIPTOR_TOGGLED & (AGS_PORT_DESCRIPTOR(port->data)->flags)) != 0){
+      if(ags_plugin_port_test_flags(AGS_PLUGIN_PORT(plugin_port->data), AGS_PLUGIN_PORT_TOGGLED)){
 	bulk_member->port_flags = AGS_BULK_MEMBER_PORT_BOOLEAN;
       }
       
-      if((AGS_PORT_DESCRIPTOR_INTEGER & (AGS_PORT_DESCRIPTOR(port->data)->flags)) != 0){
+      if(ags_plugin_port_test_flags(AGS_PLUGIN_PORT(plugin_port->data), AGS_PLUGIN_PORT_INTEGER)){
 	bulk_member->port_flags = AGS_BULK_MEMBER_PORT_INTEGER;
       }
 
@@ -1723,8 +1762,8 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
 	}
 
 	/* add controls of ports and apply range  */
-	lower_bound = g_value_get_float(AGS_PORT_DESCRIPTOR(port->data)->lower_value);
-	upper_bound = g_value_get_float(AGS_PORT_DESCRIPTOR(port->data)->upper_value);
+	lower_bound = g_value_get_float(AGS_PLUGIN_PORT(plugin_port->data)->lower_value);
+	upper_bound = g_value_get_float(AGS_PLUGIN_PORT(plugin_port->data)->upper_value);
 
 	adjustment = (GtkAdjustment *) gtk_adjustment_new(0.0, 0.0, 1.0, 0.1, 0.1, 0.0);
 	g_object_set(dial,
@@ -1746,7 +1785,7 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
 	gtk_adjustment_set_upper(adjustment,
 				 upper_bound);
 	gtk_adjustment_set_value(adjustment,
-				 g_value_get_float(AGS_PORT_DESCRIPTOR(port->data)->default_value));
+				 g_value_get_float(AGS_PLUGIN_PORT(plugin_port->data)->default_value));
       }else if(AGS_IS_INDICATOR(child_widget) ||
 	       AGS_IS_LED(child_widget)){
 	g_hash_table_insert(ags_effect_bulk_indicator_queue_draw,
@@ -1772,13 +1811,13 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
       x++;
     }
 
-    port = port->next;    
+    plugin_port = plugin_port->next;    
     k++;
   }
 
   /* program */
   live_lv2_bridge->lv2_handle = ags_base_plugin_instantiate(lv2_plugin,
-							    AGS_MACHINE(live_lv2_bridge)->audio->samplerate);
+							    samplerate, buffer_size);
   
   if((AGS_LV2_PLUGIN_HAS_PROGRAM_INTERFACE & (lv2_plugin->flags)) != 0){
     ags_live_lv2_bridge_load_program(live_lv2_bridge);
@@ -1801,7 +1840,7 @@ ags_live_lv2_bridge_load(AgsLiveLv2Bridge *live_lv2_bridge)
  *
  * Returns: %TRUE if proceed with redraw, otherwise %FALSE
  *
- * Since: 0.7.134
+ * Since: 1.0.0
  */
 gboolean
 ags_live_lv2_bridge_lv2ui_idle_timeout(GtkWidget *widget)
@@ -1827,11 +1866,11 @@ ags_live_lv2_bridge_lv2ui_idle_timeout(GtkWidget *widget)
  * @filename: the plugin.so
  * @effect: the effect
  *
- * Creates an #AgsLiveLv2Bridge
+ * Create a new instance of #AgsLiveLv2Bridge
  *
- * Returns: a new #AgsLiveLv2Bridge
+ * Returns: the new #AgsLiveLv2Bridge
  *
- * Since: 0.7.134
+ * Since: 2.0.0
  */
 AgsLiveLv2Bridge*
 ags_live_lv2_bridge_new(GObject *soundcard,
@@ -1839,18 +1878,13 @@ ags_live_lv2_bridge_new(GObject *soundcard,
 			gchar *effect)
 {
   AgsLiveLv2Bridge *live_lv2_bridge;
-  GValue value = {0,};
 
   live_lv2_bridge = (AgsLiveLv2Bridge *) g_object_new(AGS_TYPE_LIVE_LV2_BRIDGE,
 						      NULL);
 
-  if(soundcard != NULL){
-    g_value_init(&value, G_TYPE_OBJECT);
-    g_value_set_object(&value, soundcard);
-    g_object_set_property(G_OBJECT(AGS_MACHINE(live_lv2_bridge)->audio),
-			  "soundcard", &value);
-    g_value_unset(&value);
-  }
+  g_object_set(AGS_MACHINE(live_lv2_bridge)->audio,
+	       "output-soundcard", soundcard,
+	       NULL);
 
   g_object_set(live_lv2_bridge,
 	       "filename", filename,
