@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -20,57 +20,9 @@
 #include <ags/X/machine/ags_matrix.h>
 #include <ags/X/machine/ags_matrix_callbacks.h>
 
-#include <ags/util/ags_id_generator.h>
-
-#include <ags/object/ags_application_context.h>
-#include <ags/object/ags_config.h>
-#include <ags/object/ags_connectable.h>
-#include <ags/object/ags_portlet.h>
-#include <ags/object/ags_plugin.h>
-#include <ags/object/ags_seekable.h>
-
-#include <ags/file/ags_file.h>
-#include <ags/file/ags_file_stock.h>
-#include <ags/file/ags_file_id_ref.h>
-#include <ags/file/ags_file_lookup.h>
-#include <ags/file/ags_file_launch.h>
-
-#include <ags/thread/ags_mutex_manager.h>
-#include <ags/thread/ags_thread-posix.h>
-
-#include <ags/audio/ags_audio.h>
-#include <ags/audio/ags_channel.h>
-#include <ags/audio/ags_input.h>
-#include <ags/audio/ags_output.h>
-#include <ags/audio/ags_pattern.h>
-#include <ags/audio/ags_recall_factory.h>
-#include <ags/audio/ags_recall.h>
-#include <ags/audio/ags_recall_container.h>
-
-#include <ags/audio/thread/ags_audio_loop.h>
-
-#include <ags/audio/recall/ags_delay_audio.h>
-#include <ags/audio/recall/ags_delay_audio_run.h>
-#include <ags/audio/recall/ags_count_beats_audio.h>
-#include <ags/audio/recall/ags_count_beats_audio_run.h>
-#include <ags/audio/recall/ags_loop_channel.h>
-#include <ags/audio/recall/ags_loop_channel_run.h>
-#include <ags/audio/recall/ags_envelope_channel.h>
-#include <ags/audio/recall/ags_envelope_channel_run.h>
-#include <ags/audio/recall/ags_stream_channel.h>
-#include <ags/audio/recall/ags_stream_channel_run.h>
-#include <ags/audio/recall/ags_copy_pattern_audio.h>
-#include <ags/audio/recall/ags_copy_pattern_audio_run.h>
-#include <ags/audio/recall/ags_copy_pattern_channel.h>
-#include <ags/audio/recall/ags_copy_pattern_channel_run.h>
-#include <ags/audio/recall/ags_play_notation_audio.h>
-#include <ags/audio/recall/ags_play_notation_audio_run.h>
-#include <ags/audio/recall/ags_buffer_channel.h>
-#include <ags/audio/recall/ags_buffer_channel_run.h>
-#include <ags/audio/recall/ags_record_midi_audio.h>
-#include <ags/audio/recall/ags_record_midi_audio_run.h>
-#include <ags/audio/recall/ags_play_notation_audio.h>
-#include <ags/audio/recall/ags_play_notation_audio_run.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
+#include <ags/libags-gui.h>
 
 #include <ags/X/ags_menu_bar.h>
 
@@ -80,16 +32,15 @@
 
 #include <math.h>
 
-#define AGS_MATRIX_INPUT_LINE_MAPPED_KEY "AGS_MATRIX_INPUT_LINE_MAPPED_KEY"
-#define AGS_MATRIX_INPUT_LINE_MAPPED_DATA "AGS_MATRIX_INPUT_LINE_MAPPED_DATA"
-
 void ags_matrix_class_init(AgsMatrixClass *matrix);
 void ags_matrix_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_matrix_plugin_interface_init(AgsPluginInterface *plugin);
 void ags_matrix_init(AgsMatrix *matrix);
 void ags_matrix_finalize(GObject *gobject);
+
 void ags_matrix_connect(AgsConnectable *connectable);
 void ags_matrix_disconnect(AgsConnectable *connectable);
+
 void ags_matrix_map_recall(AgsMachine *machine);
 gchar* ags_matrix_get_name(AgsPlugin *plugin);
 void ags_matrix_set_name(AgsPlugin *plugin, gchar *name);
@@ -101,12 +52,15 @@ void ags_matrix_read_resolve_audio(AgsFileLookup *file_lookup,
 void ags_matrix_launch_task(AgsFileLaunch *file_launch, AgsMatrix *matrix);
 xmlNode* ags_matrix_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin);
 
-void ags_matrix_set_audio_channels(AgsAudio *audio,
-				   guint audio_channels, guint audio_channels_old,
-				   gpointer data);
-void ags_matrix_set_pads(AgsAudio *audio, GType type,
-			 guint pads, guint pads_old,
-			 gpointer data);
+void ags_matrix_resize_audio_channels(AgsMachine *machine,
+				      guint audio_channels, guint audio_channels_old,
+				      gpointer data);
+void ags_matrix_resize_pads(AgsMachine *machine, GType type,
+			    guint pads, guint pads_old,
+			    gpointer data);
+
+#define AGS_MATRIX_INPUT_LINE_MAPPED_KEY "AGS_MATRIX_INPUT_LINE_MAPPED_KEY"
+#define AGS_MATRIX_INPUT_LINE_MAPPED_DATA "AGS_MATRIX_INPUT_LINE_MAPPED_DATA"
 
 /**
  * SECTION:ags_matrix
@@ -230,13 +184,15 @@ ags_matrix_init(AgsMatrix *matrix)
 			 G_CALLBACK(ags_matrix_parent_set_callback), (gpointer) matrix);
 
   audio = AGS_MACHINE(matrix)->audio;
-  audio->flags |= (AGS_AUDIO_OUTPUT_HAS_RECYCLING |
-		   AGS_AUDIO_INPUT_HAS_RECYCLING |
-		   AGS_AUDIO_SYNC |
-		   AGS_AUDIO_ASYNC |
-		   AGS_AUDIO_NOTATION_DEFAULT |
-		   AGS_AUDIO_HAS_NOTATION |
-		   AGS_AUDIO_PATTERN_MODE);
+  ags_audio_set_flags(audio, (AGS_AUDIO_SYNC |
+			      AGS_AUDIO_ASYNC |
+			      AGS_AUDIO_OUTPUT_HAS_RECYCLING |
+			      AGS_AUDIO_INPUT_HAS_RECYCLING));
+  ags_audio_set_ability_flags(audio, (AGS_SOUND_ABILITY_SEQUENCER |
+				      AGS_SOUND_ABILITY_NOTATION));
+  ags_audio_set_behaviour_flags(audio, (AGS_SOUND_BEHAVIOUR_PATTERN_MODE |
+					AGS_SOUND_BEHAVIOUR_REVERSE_MAPPING |
+					AGS_SOUND_BEHAVIOUR_DEFAULTS_TO_INPUT));
   g_object_set(audio,
 	       "audio-start-mapping", 0,
 	       "audio-end-mapping", 128,
@@ -244,6 +200,10 @@ ags_matrix_init(AgsMatrix *matrix)
 	       "midi-end-mapping", 128,
 	       NULL);
 
+  audio->bank_dim[0] = 1;
+  audio->bank_dim[1] = 9;
+  audio->bank_dim[2] = 32;
+
   AGS_MACHINE(matrix)->flags |= (AGS_MACHINE_REVERSE_NOTATION);
 
   ags_machine_popup_add_connection_options((AgsMachine *) matrix,
@@ -254,11 +214,11 @@ ags_matrix_init(AgsMatrix *matrix)
   AGS_MACHINE(matrix)->output_pad_type = G_TYPE_NONE;
   AGS_MACHINE(matrix)->output_line_type = G_TYPE_NONE;
 
-  g_signal_connect_after(G_OBJECT(audio), "set_audio_channels",
-			 G_CALLBACK(ags_matrix_set_audio_channels), matrix);
+  g_signal_connect_after(G_OBJECT(matrix), "resize-audio-channels",
+			 G_CALLBACK(ags_matrix_resize_audio_channels), NULL);
 
-  g_signal_connect_after(G_OBJECT(audio), "set_pads",
-			 G_CALLBACK(ags_matrix_set_pads), matrix);
+  g_signal_connect_after(G_OBJECT(matrix), "resize-pads",
+			 G_CALLBACK(ags_matrix_resize_pads), NULL);
 
   /*  */
   AGS_MACHINE(matrix)->flags |= (AGS_MACHINE_IS_SEQUENCER |
@@ -395,11 +355,8 @@ ags_matrix_connect(AgsConnectable *connectable)
   g_signal_connect((GObject *) matrix->loop_button, "clicked",
 		   G_CALLBACK(ags_matrix_loop_button_callback), (gpointer) matrix);
 
-  g_signal_connect_after(G_OBJECT(AGS_MACHINE(matrix)->audio), "tact",
-			 G_CALLBACK(ags_matrix_tact_callback), matrix);
-
-  g_signal_connect_after(G_OBJECT(AGS_MACHINE(matrix)->audio), "done",
-			 G_CALLBACK(ags_matrix_done_callback), matrix);
+  g_signal_connect_after(G_OBJECT(matrix), "stop",
+			 G_CALLBACK(ags_matrix_stop_callback), NULL);
 }
 
 void
@@ -418,7 +375,7 @@ ags_matrix_disconnect(AgsConnectable *connectable)
 
   for(i  = 0; i < 9; i++){
     g_object_disconnect(G_OBJECT (matrix->index[i]),
-			"clicked",
+			"any_signal::clicked",
 			G_CALLBACK (ags_matrix_index_callback),
 			(gpointer) matrix,
 			NULL);
@@ -427,27 +384,21 @@ ags_matrix_disconnect(AgsConnectable *connectable)
   ags_connectable_disconnect(AGS_CONNECTABLE(matrix->cell_pattern));
   
   g_object_disconnect((GObject *) matrix->length_spin,
-		      "value-changed",
+		      "any_signal::value-changed",
 		      G_CALLBACK(ags_matrix_length_spin_callback),
 		      (gpointer) matrix,
 		      NULL);
 
   g_object_disconnect((GObject *) matrix->loop_button,
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_matrix_loop_button_callback),
 		      (gpointer) matrix,
 		      NULL);
 
-  g_object_disconnect(G_OBJECT(AGS_MACHINE(matrix)->audio),
-		      "tact",
-		      G_CALLBACK(ags_matrix_tact_callback),
-		      matrix,
-		      NULL);
-
-  g_object_disconnect(G_OBJECT(AGS_MACHINE(matrix)->audio),
-		      "done",
-		      G_CALLBACK(ags_matrix_done_callback),
-		      matrix,
+  g_object_disconnect(G_OBJECT(matrix),
+		      "any_signal::stop",
+		      G_CALLBACK(ags_matrix_stop_callback),
+		      NULL,
 		      NULL);
 
   /* call parent */
@@ -455,65 +406,38 @@ ags_matrix_disconnect(AgsConnectable *connectable)
 }
 
 void
-ags_matrix_set_audio_channels(AgsAudio *audio,
-			      guint audio_channels, guint audio_channels_old,
-			      gpointer data)
+ags_matrix_resize_audio_channels(AgsMachine *machine,
+				 guint audio_channels, guint audio_channels_old,
+				 gpointer data)
 {
   g_message("AgsMatrix only pads can be adjusted");
-  //  _ags_audio_set_audio_channels(audio, audio_channels);
 }
 
 void
-ags_matrix_set_pads(AgsAudio *audio, GType type,
-		    guint pads, guint pads_old,
-		    gpointer data)
+ags_matrix_resize_pads(AgsMachine *machine, GType type,
+		       guint pads, guint pads_old,
+		       gpointer data)
 {
   AgsWindow *window;
-  AgsMachine *machine;
   AgsMatrix *matrix;
 
+  AgsAudio *audio;
   AgsChannel *channel, *source;
   AgsAudioSignal *audio_signal;
-
-  AgsMutexManager *mutex_manager;
-
-  AgsApplicationContext *application_context;
   
   guint i, j;
   gboolean grow;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *source_mutex;
+  matrix = (AgsMatrix *) machine;
 
+  audio = machine->audio;
+  
   if(pads == pads_old){
     return;
   }
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* lookup audio mutex */
-  pthread_mutex_lock(application_mutex);
-    
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* get machine */
-  pthread_mutex_lock(audio_mutex);
-  
-  matrix = (AgsMatrix *) audio->machine;
-
-  pthread_mutex_unlock(audio_mutex);
-
-  gdk_threads_enter();
-  
-  machine = AGS_MACHINE(matrix);
   window = (AgsWindow *) gtk_widget_get_toplevel((GtkWidget *) machine);
-
-  application_context = window->application_context;
   
   /* set size request if needed */
   if(g_type_is_a(type, AGS_TYPE_INPUT)){
@@ -532,8 +456,6 @@ ags_matrix_set_pads(AgsAudio *audio, GType type,
     }
   }
 
-  gdk_threads_leave();
-  
   if(pads_old < pads){
     grow = TRUE;
   }else{
@@ -541,39 +463,11 @@ ags_matrix_set_pads(AgsAudio *audio, GType type,
   }
   
   if(g_type_is_a(type, AGS_TYPE_INPUT)){
-    pthread_mutex_lock(audio_mutex);
-
-    source = audio->input;
-
-    pthread_mutex_unlock(audio_mutex);
-
+    g_object_get(audio,
+		 "input", &source,
+		 NULL);
+    
     if(grow){
-      /* create pattern */
-      source = ags_channel_nth(source, pads_old);
-      
-      while(source != NULL){
-	/* lookup source mutex */
-	pthread_mutex_lock(application_mutex);
-
-	source_mutex = ags_mutex_manager_lookup(mutex_manager,
-						(GObject *) source);
-  
-	pthread_mutex_unlock(application_mutex);
-
-	/* instantiate pattern */
-	pthread_mutex_lock(source_mutex);
-
-	if(source->pattern == NULL){
-	  source->pattern = g_list_alloc();
-	  source->pattern->data = (gpointer) ags_pattern_new();
-	  ags_pattern_set_dim((AgsPattern *) source->pattern->data, 1, 9, 32);
-	}
-	
-	source = source->next;
-
-	pthread_mutex_unlock(source_mutex);
-      }
-
       if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0){
 	ags_matrix_input_map_recall(matrix,
 				    pads_old);
@@ -583,11 +477,9 @@ ags_matrix_set_pads(AgsAudio *audio, GType type,
     }
   }else{
     if(grow){
-      pthread_mutex_lock(audio_mutex);
-
-      source = audio->output;
-
-      pthread_mutex_unlock(audio_mutex);
+      g_object_get(audio,
+		   "output", &source,
+		   NULL);
 
       source = ags_channel_nth(source,
 			       pads_old);
@@ -596,31 +488,19 @@ ags_matrix_set_pads(AgsAudio *audio, GType type,
 	AgsRecycling *recycling;
 	AgsAudioSignal *audio_signal;
 
-	GObject *soundcard;
+	GObject *output_soundcard;
 
-	pthread_mutex_lock(audio_mutex);
-	
-	soundcard = audio->soundcard;
-
-	pthread_mutex_unlock(audio_mutex);
-
-	/* lookup source mutex */
-	pthread_mutex_lock(application_mutex);
-
-	source_mutex = ags_mutex_manager_lookup(mutex_manager,
-						(GObject *) source);
-  
-	pthread_mutex_unlock(application_mutex);
+	g_object_get(audio,
+		     "output-soundcard", &output_soundcard,
+		     NULL);
 
 	/* get recycling */
-	pthread_mutex_lock(source_mutex);
-
-	recycling = source->first_recycling;
-
-	pthread_mutex_unlock(source_mutex);
+	g_object_get(source,
+		     "first-recycling", &recycling,
+		     NULL);
 
 	/* instantiate template audio signal */
-	audio_signal = ags_audio_signal_new(soundcard,
+	audio_signal = ags_audio_signal_new(output_soundcard,
 					    (GObject *) recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
@@ -645,8 +525,8 @@ ags_matrix_map_recall(AgsMachine *machine)
   AgsMatrix *matrix;
 
   AgsAudio *audio;
+  AgsChannel *input;
   AgsChannel *channel;
-  AgsPattern *pattern;
 
   AgsDelayAudio *play_delay_audio;
   AgsDelayAudioRun *play_delay_audio_run;
@@ -661,9 +541,11 @@ ags_matrix_map_recall(AgsMachine *machine)
   AgsPlayNotationAudio *recall_notation_audio;
   AgsPlayNotationAudioRun *recall_notation_audio_run;
 
-  GList *notation;
-  GList *list;
+  GList *start_play, *play;
+  GList *start_recall, *recall;
 
+  guint input_pads;
+  guint audio_channels;
   guint i, j;
 
   GValue value = {0,};
@@ -677,8 +559,16 @@ ags_matrix_map_recall(AgsMachine *machine)
 						 AGS_TYPE_WINDOW);
 
   matrix = AGS_MATRIX(machine);
+
   audio = machine->audio;
 
+  /* get some fields */
+  g_object_get(audio,
+	       "input", &input,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
+
   /* ags-delay */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -689,16 +579,22 @@ ags_matrix_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_ADD |
 			     AGS_RECALL_FACTORY_PLAY),
 			    0);
+  
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
 
-  list = ags_recall_find_type(audio->play,
+  play = ags_recall_find_type(start_play,
 			      AGS_TYPE_DELAY_AUDIO_RUN);
 
-  if(list != NULL){
-    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(play->data);
     //    AGS_RECALL(play_delay_audio_run)->flags |= AGS_RECALL_PERSISTENT;
   }else{
     play_delay_audio_run = NULL;
   }
+
+  g_list_free(start_play);
   
   /* ags-count-beats */
   ags_recall_factory_create(audio,
@@ -710,12 +606,16 @@ ags_matrix_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_ADD |
 			     AGS_RECALL_FACTORY_PLAY),
 			    0);
-  
-  list = ags_recall_find_type(audio->play,
+
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
+
+  play = ags_recall_find_type(start_play,
 			      AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
 
-  if(list != NULL){
-    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(play->data);
 
     /* set dependency */  
     g_object_set(G_OBJECT(play_count_beats_audio_run),
@@ -733,22 +633,28 @@ ags_matrix_map_recall(AgsMachine *machine)
     play_count_beats_audio_run = NULL;
   }
 
+  g_list_free(start_play);
+  
   /* ags-copy-pattern */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
 			    "ags-copy-pattern",
-			    0, audio->audio_channels,
-			    0, audio->input_lines,
+			    0, 0,
+			    0, 0,
 			    (AGS_RECALL_FACTORY_INPUT |
 			     AGS_RECALL_FACTORY_ADD |
 			     AGS_RECALL_FACTORY_RECALL),
 			    0);
 
-  list = ags_recall_find_type(audio->recall,
-			      AGS_TYPE_COPY_PATTERN_AUDIO_RUN);
+  g_object_get(audio,
+	       "recall", &start_recall,
+	       NULL);
 
-  if(list != NULL){
-    recall_copy_pattern_audio_run = AGS_COPY_PATTERN_AUDIO_RUN(list->data);
+  recall = ags_recall_find_type(start_recall,
+				AGS_TYPE_COPY_PATTERN_AUDIO_RUN);
+
+  if(recall != NULL){
+    recall_copy_pattern_audio_run = AGS_COPY_PATTERN_AUDIO_RUN(recall->data);
 
     /* set dependency */
     g_object_set(G_OBJECT(recall_copy_pattern_audio_run),
@@ -758,27 +664,57 @@ ags_matrix_map_recall(AgsMachine *machine)
 
   }
 
+  g_list_free(start_recall);
+  
   /* set pattern object on port */
-  channel = ags_channel_pad_nth(audio->input, 0);
+  channel = ags_channel_pad_nth(input,
+				0);
       
-  for(i = 0; i < audio->input_pads; i++){
-    for(j = 0; j < audio->audio_channels; j++){
-      list = ags_recall_template_find_type(channel->recall,
+  for(i = 0; i < input_pads; i++){
+    for(j = 0; j < audio_channels; j++){
+      AgsPort *port;
+      
+      GList *start_list, *list;
+      GList *pattern;
+      
+      GValue pattern_value = {0,};
+
+      g_object_get(channel,
+		   "recall", &start_list,
+		   NULL);
+      
+      /* ags-copy-pattern-channel */
+      list = ags_recall_template_find_type(start_list,
 					   AGS_TYPE_COPY_PATTERN_CHANNEL);
 
       if(list != NULL){
 	copy_pattern_channel = AGS_COPY_PATTERN_CHANNEL(list->data);
+	g_object_get(copy_pattern_channel,
+		     "pattern", &port,
+		     NULL);
 
-	list = channel->pattern;
-	pattern = AGS_PATTERN(list->data);
+	g_object_get(channel,
+		     "pattern", &pattern,
+		     NULL);
 
-	copy_pattern_channel->pattern->port_value.ags_port_object = (GObject *) pattern;
-	  
-	ags_portlet_set_port(AGS_PORTLET(pattern),
-			     (GObject *) copy_pattern_channel->pattern);
+	g_value_init(&pattern_value,
+		     G_TYPE_OBJECT);
+    
+	g_value_set_object(&pattern_value,
+			   pattern->data);
+
+	ags_port_safe_write(port,
+			    &pattern_value);
+
+	g_list_free(pattern);
       }
+
+      g_list_free(start_list);
       
-      channel = channel->next;
+      /* iterate */
+      g_object_get(channel,
+		   "next", &channel,
+		   NULL);
     }
   }
 
@@ -790,13 +726,18 @@ ags_matrix_map_recall(AgsMachine *machine)
 			    0, 0,
 			    (AGS_RECALL_FACTORY_INPUT |
 			     AGS_RECALL_FACTORY_ADD |
-			     AGS_RECALL_FACTORY_RECALL),
+			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->recall, AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
 
-  if(list != NULL){
-    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(list->data);
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
+
+  if(play != NULL){
+    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(play->data);
     
     /* set dependency */
     g_object_set(G_OBJECT(recall_record_midi_audio_run),
@@ -809,6 +750,8 @@ ags_matrix_map_recall(AgsMachine *machine)
 		 NULL);
   }  
 
+  g_list_free(start_play);
+  
   /* ags-play-notation */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -817,13 +760,18 @@ ags_matrix_map_recall(AgsMachine *machine)
 			    0, 0,
 			    (AGS_RECALL_FACTORY_INPUT |
 			     AGS_RECALL_FACTORY_ADD |
-			     AGS_RECALL_FACTORY_RECALL),
+			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->recall, AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
 
-  if(list != NULL){
-    recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(list->data);
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+
+  if(play != NULL){
+    recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(play->data);
 
     /* set dependency */
     g_object_set(G_OBJECT(recall_notation_audio_run),
@@ -836,238 +784,262 @@ ags_matrix_map_recall(AgsMachine *machine)
 		 NULL);
   }
 
+  g_list_free(start_play);
+  
   /* depending on destination */
-  ags_matrix_input_map_recall(matrix, 0);
+  ags_matrix_input_map_recall(matrix,
+			      0);
 
   /* depending on destination */
-  ags_matrix_output_map_recall(matrix, 0);
+  ags_matrix_output_map_recall(matrix,
+			       0);
 
   AGS_MACHINE_CLASS(ags_matrix_parent_class)->map_recall(machine);
 }
 
 void
-ags_matrix_input_map_recall(AgsMatrix *matrix, guint input_pad_start)
+ags_matrix_input_map_recall(AgsMatrix *matrix,
+			    guint input_pad_start)
 {
   AgsAudio *audio;
-  AgsChannel *source, *current;
-
-  AgsConfig *config;
+  AgsChannel *input;
+  AgsChannel *channel;
   
-  GList *list;
+  AgsCopyPatternChannel *copy_pattern_channel;
 
-  gchar *str;
+  GList *start_play, *play;
 
-  gboolean performance_mode;
-
-  audio = AGS_MACHINE(matrix)->audio;
+  guint input_pads;
+  guint audio_channels;
+  guint i, j;
 
   if(matrix->mapped_input_pad > input_pad_start){
     return;
   }
 
-  config = ags_config_get_instance();
+  audio = AGS_MACHINE(matrix)->audio;
 
-  source = ags_channel_nth(audio->input,
-			   input_pad_start * audio->audio_channels);
+  /* get some fields */
+  g_object_get(audio,
+	       "input", &input,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
 
-  /* map dependending on output */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "engine-mode");
-  performance_mode = FALSE;
+  if(audio_channels == 0){
+    return;
+  }
 
-  /* remap for input */
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "performance",
-			  12)){
-    current = source;
+  /* ags-copy-pattern */
+  ags_recall_factory_create(audio,
+			    NULL, NULL,
+			    "ags-copy-pattern",
+			    0, audio_channels, 
+			    input_pad_start, input_pads,
+			    (AGS_RECALL_FACTORY_INPUT |
+			     AGS_RECALL_FACTORY_REMAP |
+			     AGS_RECALL_FACTORY_RECALL),
+			    0);
 
-    while(current != NULL){
-      /* ags-copy */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-copy",
-				0, audio->audio_channels, 
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
+  /* set pattern object on port */
+  channel = ags_channel_pad_nth(input,
+				input_pad_start);
+      
+  for(i = 0; i < input_pads; i++){
+    for(j = 0; j < audio_channels; j++){
+      AgsPort *port;
+      
+      GList *start_list, *list;
+      GList *pattern;
+      
+      GValue pattern_value = {0,};
 
-      current = current->next_pad;
-    }
+      g_object_get(channel,
+		   "recall", &start_list,
+		   NULL);
+      
+      /* ags-copy-pattern-channel */
+      list = ags_recall_template_find_type(start_list,
+					   AGS_TYPE_COPY_PATTERN_CHANNEL);
+
+      if(list != NULL){
+	copy_pattern_channel = AGS_COPY_PATTERN_CHANNEL(list->data);
+	g_object_get(copy_pattern_channel,
+		     "pattern", &port,
+		     NULL);
+
+	g_object_get(channel,
+		     "pattern", &pattern,
+		     NULL);
+
+	g_value_init(&pattern_value,
+		     G_TYPE_OBJECT);
     
-    /* set performance mode */
-    performance_mode = TRUE;
-  }else{    
-    current = source;
+	g_value_set_object(&pattern_value,
+			   pattern->data);
 
-    while(current != NULL){
-      /* ags-buffer */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-buffer",
-				0, audio->audio_channels, 
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
+	ags_port_safe_write(port,
+			    &pattern_value);
 
-      current = current->next_pad;
+	g_list_free(pattern);
+      }
+
+      g_list_free(start_list);
+      
+      /* iterate */
+      g_object_get(channel,
+		   "next", &channel,
+		   NULL);
     }
   }
   
-  /*  */
-  current = source;
-
-  while(current != NULL){
-    /* ags-play */
+  /* remap for input */
+  if(ags_recall_global_get_rt_safe() ||
+     ags_recall_global_get_performance_mode()){
+    /* ags-copy */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
-			      "ags-play",
-			      current->audio_channel, current->audio_channel + 1, 
-			      current->pad, current->pad + 1,
+			      "ags-copy",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
 			      (AGS_RECALL_FACTORY_INPUT |
-			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_RECALL |
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
-
-    current = current->next_pad;
-  }
-
-  /*  */
-  current = source;
-
-  while(current != NULL){
-    /* ags-envelope */
+  }else{    
+    /* ags-buffer */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
-			      "ags-envelope",
-			      current->audio_channel, current->audio_channel + 1, 
-			      current->pad, current->pad + 1,
+			      "ags-buffer",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
 			      (AGS_RECALL_FACTORY_INPUT |
-			       AGS_RECALL_FACTORY_PLAY |
-			       AGS_RECALL_FACTORY_RECALL | 
+			       AGS_RECALL_FACTORY_RECALL |
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
-
-    current = current->next_pad;
   }
+  
+  /* ags-play */
+  ags_recall_factory_create(audio,
+			    NULL, NULL,
+			    "ags-play",
+			    0, audio_channels, 
+			    input_pad_start, input_pads,
+			    (AGS_RECALL_FACTORY_INPUT |
+			     AGS_RECALL_FACTORY_PLAY |
+			     AGS_RECALL_FACTORY_ADD),
+			    0);
 
-  /*  */
-  current = source;
+  /* ags-envelope */
+  ags_recall_factory_create(audio,
+			    NULL, NULL,
+			    "ags-envelope",
+			    0, audio_channels, 
+			    input_pad_start, input_pads,
+			    (AGS_RECALL_FACTORY_INPUT |
+			     AGS_RECALL_FACTORY_PLAY |
+			     AGS_RECALL_FACTORY_RECALL | 
+			     AGS_RECALL_FACTORY_ADD),
+			    0);
 
-  while(current != NULL){
-    /* ags-stream */
+  /* ags-stream */
+  if(!ags_recall_global_get_rt_safe()){
     ags_recall_factory_create(audio,
 			      NULL, NULL,
 			      "ags-stream",
-			      current->audio_channel, current->audio_channel + 1, 
-			      current->pad, current->pad + 1,
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_RECALL | 
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
+  }else{
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-rt-stream",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
 			      (AGS_RECALL_FACTORY_INPUT |
 			       AGS_RECALL_FACTORY_PLAY |
 			       AGS_RECALL_FACTORY_RECALL | 
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
-
-    current = current->next_pad;
   }
   
-  matrix->mapped_input_pad = audio->input_pads;
+  matrix->mapped_input_pad = input_pads;
 }
 
 void
-ags_matrix_output_map_recall(AgsMatrix *matrix, guint output_pad_start)
+ags_matrix_output_map_recall(AgsMatrix *matrix,
+			     guint output_pad_start)
 {
   AgsAudio *audio;
-  AgsChannel *input, *current;
 
-  AgsConfig *config;
-
-  gchar *str;
-
-  gboolean performance_mode;
-
-  audio = AGS_MACHINE(matrix)->audio;
+  guint input_pad_start;
+  guint output_pads, input_pads;
+  guint audio_channels;
 
   if(matrix->mapped_output_pad > output_pad_start){
     return;
   }
 
-  config = ags_config_get_instance();
+  audio = AGS_MACHINE(matrix)->audio;
 
-  /* map dependending on output */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "engine-mode");
-  performance_mode = FALSE;
+  /* get some fields */
+  g_object_get(audio,
+	       "output-pads", &output_pads,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
+  
+  input_pad_start = 0;
+
+  if(audio_channels == 0){
+    return;
+  }  
 
   /* remap for input */
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "performance",
-			  12)){
-    input = audio->input;
-
-    while(input != NULL){
-      /* ags-copy */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-copy",
-				0, audio->audio_channels, 
-				input->pad, input->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      input = input->next_pad;
-    }
-    
-    /* set performance mode */
-    performance_mode = TRUE;
+  if(ags_recall_global_get_rt_safe() ||
+     ags_recall_global_get_performance_mode()){
+    /* ags-copy */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-copy",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }else{    
-    input = audio->input;
-
-    while(input != NULL){
-      /* ags-buffer */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-buffer",
-				0, audio->audio_channels, 
-				input->pad, input->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      input = input->next_pad;
-    }
+    /* ags-buffer */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-buffer",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }
 
-  if(!performance_mode){
-    current = ags_channel_nth(audio->output,
-			      output_pad_start * audio->audio_channels);
-
-    while(current != NULL){
-      /* ags-stream */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-stream",
-				current->audio_channel, current->audio_channel + 1,
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_OUTPUT |
-				 AGS_RECALL_FACTORY_PLAY |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      current = current->next;
-    }
+  if(!(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode())){
+    /* ags-stream */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-stream",
+			      0, audio_channels,
+			      output_pad_start, output_pads,
+			      (AGS_RECALL_FACTORY_OUTPUT |
+			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }
   
-  matrix->mapped_output_pad = audio->output_pads;
+  matrix->mapped_output_pad = output_pads;
 }
 
 gchar*
@@ -1155,11 +1127,11 @@ ags_matrix_read_resolve_audio(AgsFileLookup *file_lookup,
 
   matrix = AGS_MATRIX(machine);
 
-  g_signal_connect_after(G_OBJECT(machine->audio), "set_audio_channels",
-			 G_CALLBACK(ags_matrix_set_audio_channels), matrix);
+  g_signal_connect_after(G_OBJECT(machine), "resize-audio-channels",
+			 G_CALLBACK(ags_matrix_resize_audio_channels), matrix);
 
-  g_signal_connect_after(G_OBJECT(machine->audio), "set_pads",
-			 G_CALLBACK(ags_matrix_set_pads), matrix);
+  g_signal_connect_after(G_OBJECT(machine), "resize-pads",
+			 G_CALLBACK(ags_matrix_resize_pads), matrix);
 
   if((AGS_MACHINE_PREMAPPED_RECALL & (machine->flags)) == 0){
     ags_matrix_output_map_recall(matrix, 0);
@@ -1261,11 +1233,11 @@ ags_matrix_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin)
  * ags_matrix_new:
  * @soundcard: the assigned soundcard.
  *
- * Creates an #AgsMatrix
+ * Create a new instance of #AgsMatrix
  *
- * Returns: a new #AgsMatrix
+ * Returns: the new #AgsMatrix
  *
- * Since: 0.3
+ * Since: 2.0.0
  */
 AgsMatrix*
 ags_matrix_new(GObject *soundcard)
@@ -1276,7 +1248,7 @@ ags_matrix_new(GObject *soundcard)
 				      NULL);
   
   g_object_set(AGS_MACHINE(matrix)->audio,
-	       "soundcard", soundcard,
+	       "output-soundcard", soundcard,
 	       NULL);
   
   return(matrix);
