@@ -20,64 +20,20 @@
 #include <ags/X/machine/ags_syncsynth.h>
 #include <ags/X/machine/ags_syncsynth_callbacks.h>
 
-#include <ags/util/ags_id_generator.h>
-
-#include <ags/object/ags_application_context.h>
-#include <ags/object/ags_connectable.h>
-#include <ags/object/ags_soundcard.h>
-#include <ags/object/ags_plugin.h>
-#include <ags/object/ags_seekable.h>
-
-#include <ags/file/ags_file.h>
-#include <ags/file/ags_file_stock.h>
-#include <ags/file/ags_file_id_ref.h>
-#include <ags/file/ags_file_lookup.h>
-#include <ags/file/ags_file_launch.h>
-
-#include <ags/thread/ags_mutex_manager.h>
-#include <ags/thread/ags_task_thread.h>
-
-#include <ags/audio/ags_audio.h>
-#include <ags/audio/ags_channel.h>
-#include <ags/audio/ags_input.h>
-#include <ags/audio/ags_output.h>
-#include <ags/audio/ags_audio_signal.h>
-#include <ags/audio/ags_recall_factory.h>
-#include <ags/audio/ags_recall.h>
-#include <ags/audio/ags_recall_container.h>
-
-#include <ags/audio/thread/ags_audio_loop.h>
-
-#include <ags/audio/recall/ags_delay_audio.h>
-#include <ags/audio/recall/ags_delay_audio_run.h>
-#include <ags/audio/recall/ags_count_beats_audio.h>
-#include <ags/audio/recall/ags_count_beats_audio_run.h>
-#include <ags/audio/recall/ags_envelope_channel.h>
-#include <ags/audio/recall/ags_envelope_channel_run.h>
-#include <ags/audio/recall/ags_stream_channel.h>
-#include <ags/audio/recall/ags_stream_channel_run.h>
-#include <ags/audio/recall/ags_buffer_channel.h>
-#include <ags/audio/recall/ags_buffer_channel_run.h>
-#include <ags/audio/recall/ags_record_midi_audio.h>
-#include <ags/audio/recall/ags_record_midi_audio_run.h>
-#include <ags/audio/recall/ags_play_notation_audio.h>
-#include <ags/audio/recall/ags_play_notation_audio_run.h>
-
-#include <ags/audio/task/ags_clear_audio_signal.h>
-#include <ags/audio/task/ags_apply_synth.h>
-
-#include <ags/widget/ags_expander_set.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
+#include <ags/libags-gui.h>
 
+#include <ags/X/ags_ui_provider.h>
+#include <ags/X/ags_window.h>
 #include <ags/X/ags_machine.h>
 #include <ags/X/ags_pad.h>
 #include <ags/X/ags_line.h>
 
-#include <ags/X/file/ags_gui_file_xml.h>
-
-#include <ags/X/ags_window.h>
-
 #include <ags/X/thread/ags_gui_thread.h>
 
+#include <ags/X/file/ags_gui_file_xml.h>
+
 #include <math.h>
 
 #include <ags/i18n.h>
@@ -87,8 +43,10 @@ void ags_syncsynth_connectable_interface_init(AgsConnectableInterface *connectab
 void ags_syncsynth_plugin_interface_init(AgsPluginInterface *plugin);
 void ags_syncsynth_init(AgsSyncsynth *syncsynth);
 void ags_syncsynth_finalize(GObject *gobject);
+
 void ags_syncsynth_connect(AgsConnectable *connectable);
 void ags_syncsynth_disconnect(AgsConnectable *connectable);
+
 void ags_syncsynth_show(GtkWidget *widget);
 void ags_syncsynth_map_recall(AgsMachine *machine);
 gchar* ags_syncsynth_get_name(AgsPlugin *plugin);
@@ -100,14 +58,17 @@ void ags_syncsynth_read_resolve_audio(AgsFileLookup *file_lookup,
 				      AgsMachine *machine);
 xmlNode* ags_syncsynth_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin);
 
-void ags_syncsynth_set_audio_channels(AgsAudio *audio,
-				      guint audio_channels, guint audio_channels_old,
-				      AgsSyncsynth *syncsynth);
-void ags_syncsynth_set_pads(AgsAudio *audio, GType type,
-			    guint pads, guint pads_old,
-			    AgsSyncsynth *syncsynth);
+void ags_syncsynth_resize_audio_channels(AgsMachine *machine,
+					 guint audio_channels, guint audio_channels_old,
+					 gpointer data);
+void ags_syncsynth_resize_pads(AgsMachine *machine, GType channel_type,
+			       guint pads, guint pads_old,
+			       gpointer data);
 
-void ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth, guint output_pad_start);
+void ags_syncsynth_input_map_recall(AgsSyncsynth *syncsynth,
+				    guint input_pad_start);
+void ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth,
+				     guint output_pad_start);
 
 /**
  * SECTION:ags_syncsynth
@@ -224,14 +185,14 @@ ags_syncsynth_init(AgsSyncsynth *syncsynth)
 			 G_CALLBACK(ags_syncsynth_parent_set_callback), (gpointer) syncsynth);
 
   audio = AGS_MACHINE(syncsynth)->audio;
-  audio->flags |= (AGS_AUDIO_SYNC |
-		   AGS_AUDIO_ASYNC |
-		   AGS_AUDIO_OUTPUT_HAS_RECYCLING |
-		   AGS_AUDIO_INPUT_HAS_RECYCLING |
-		   AGS_AUDIO_INPUT_TAKES_SYNTH |
-		   AGS_AUDIO_HAS_NOTATION | 
-		   AGS_AUDIO_NOTATION_DEFAULT |
-		   AGS_AUDIO_REVERSE_MAPPING);
+  ags_audio_set_flags(audio, (AGS_AUDIO_SYNC |
+			      AGS_AUDIO_ASYNC |
+			      AGS_AUDIO_OUTPUT_HAS_RECYCLING |
+			      AGS_AUDIO_INPUT_HAS_RECYCLING |
+			      AGS_AUDIO_INPUT_HAS_SYNTH));
+  ags_audio_set_ability_flags(audio, (AGS_SOUND_ABILITY_NOTATION));
+  ags_audio_set_behaviour_flags(audio, (AGS_SOUND_BEHAVIOUR_REVERSE_MAPPING |
+					AGS_SOUND_BEHAVIOUR_DEFAULTS_TO_INPUT));
   g_object_set(audio,
 	       "audio-start-mapping", 0,
 	       "audio-end-mapping", 128,
@@ -253,11 +214,11 @@ ags_syncsynth_init(AgsSyncsynth *syncsynth)
   					   (AGS_MACHINE_POPUP_MIDI_DIALOG));
 
   /* audio resize */
-  g_signal_connect_after(G_OBJECT(AGS_MACHINE(syncsynth)->audio), "set_audio_channels",
-			 G_CALLBACK(ags_syncsynth_set_audio_channels), syncsynth);
+  g_signal_connect_after(G_OBJECT(syncsynth), "resize-audio-channels",
+			 G_CALLBACK(ags_syncsynth_resize_audio_channels), NULL);
 
-  g_signal_connect_after(G_OBJECT(AGS_MACHINE(syncsynth)->audio), "set_pads",
-			 G_CALLBACK(ags_syncsynth_set_pads), syncsynth);
+  g_signal_connect_after(G_OBJECT(syncsynth), "resize-pads",
+			 G_CALLBACK(ags_syncsynth_resize_pads), NULL);
   
   /* create widgets */
   syncsynth->flags = 0;
@@ -460,7 +421,7 @@ ags_syncsynth_disconnect(AgsConnectable *connectable)
     ags_connectable_disconnect(AGS_CONNECTABLE(child_start->next->data));
     
     g_object_disconnect((GObject *) child_start->next->data,
-			"control-changed",
+			"any_signal::control-changed",
 			G_CALLBACK(ags_syncsynth_oscillator_control_changed_callback),
 			(gpointer) syncsynth,
 			NULL);
@@ -473,25 +434,25 @@ ags_syncsynth_disconnect(AgsConnectable *connectable)
   g_list_free(list_start);
 
   g_object_disconnect((GObject *) syncsynth->add,
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_syncsynth_add_callback),
 		      (gpointer) syncsynth,
 		      NULL);
   
   g_object_disconnect((GObject *) syncsynth->remove,
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_syncsynth_remove_callback),
 		      (gpointer) syncsynth,
 		      NULL);
 
   g_object_disconnect((GObject *) syncsynth->auto_update,
-		      "toggled",
+		      "any_signal::toggled",
 		      G_CALLBACK(ags_syncsynth_auto_update_callback),
 		      syncsynth,
 		      NULL);
   
   g_object_disconnect((GObject *) syncsynth->update,
-		      "clicked",
+		      "any_signal::clicked",
 		      G_CALLBACK(ags_syncsynth_update_callback),
 		      (gpointer) syncsynth,
 		      NULL);
@@ -514,7 +475,7 @@ ags_syncsynth_map_recall(AgsMachine *machine)
   AgsPlayNotationAudio *recall_notation_audio;
   AgsPlayNotationAudioRun *recall_notation_audio_run;
 
-  GList *list;
+  GList *start_play, *play;
 
   if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0 ||
      (AGS_MACHINE_PREMAPPED_RECALL & (machine->flags)) != 0){
@@ -539,15 +500,22 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->play, AGS_TYPE_DELAY_AUDIO_RUN);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
+  
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_DELAY_AUDIO_RUN);
 
-  if(list != NULL){
-    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(play->data);
     //    AGS_RECALL(play_delay_audio_run)->flags |= AGS_RECALL_PERSISTENT;
   }else{
     play_delay_audio_run = NULL;
   }
   
+  g_list_free(start_play);
+  
   /* ags-count-beats */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -558,13 +526,18 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_ADD |
 			     AGS_RECALL_FACTORY_PLAY),
 			    0);
-  
-  list = ags_recall_find_type(audio->play, AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
 
-  if(list != NULL){
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
+  
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
+
+  if(play != NULL){
     GValue value = {0,};
 
-    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(list->data);
+    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(play->data);
 
     /* set dependency */  
     g_object_set(G_OBJECT(play_count_beats_audio_run),
@@ -582,6 +555,8 @@ ags_syncsynth_map_recall(AgsMachine *machine)
     play_count_beats_audio_run = NULL;
   }
 
+  g_list_free(start_play);
+  
   /* ags-record-midi */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -590,13 +565,18 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 			    0, 0,
 			    (AGS_RECALL_FACTORY_INPUT |
 			     AGS_RECALL_FACTORY_ADD |
-			     AGS_RECALL_FACTORY_RECALL),
+			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->recall, AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
 
-  if(list != NULL){
-    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(list->data);
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+
+  if(play != NULL){
+    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(play->data);
     
     /* set dependency */
     g_object_set(G_OBJECT(recall_record_midi_audio_run),
@@ -609,6 +589,8 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 		 NULL);
   }  
 
+  g_list_free(start_play);
+  
   /* ags-play-notation */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -617,14 +599,19 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 			    0, 0,
 			    (AGS_RECALL_FACTORY_INPUT |
 			     AGS_RECALL_FACTORY_ADD |
-			     AGS_RECALL_FACTORY_RECALL),
+			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->recall, AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
+
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+
+  if(play != NULL){
+    recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(play->data);
 
-  if(list != NULL){
-    recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(list->data);
-    
     /* set dependency */
     g_object_set(G_OBJECT(recall_notation_audio_run),
 		 "delay-audio-run", play_delay_audio_run,
@@ -636,6 +623,8 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 		 NULL);
   }
 
+  g_list_free(start_play);
+
   /* depending on destination */
   ags_syncsynth_input_map_recall(syncsynth, 0);
 
@@ -720,11 +709,11 @@ ags_syncsynth_read_resolve_audio(AgsFileLookup *file_lookup,
 
   syncsynth = AGS_SYNCSYNTH(machine);
 
-  g_signal_connect_after(G_OBJECT(machine->audio), "set_audio_channels",
-			 G_CALLBACK(ags_syncsynth_set_audio_channels), syncsynth);
+  g_signal_connect_after(G_OBJECT(machine), "resize-audio-channels",
+			 G_CALLBACK(ags_syncsynth_resize_audio_channels), NULL);
 
-  g_signal_connect_after(G_OBJECT(machine->audio), "set_pads",
-			 G_CALLBACK(ags_syncsynth_set_pads), syncsynth);
+  g_signal_connect_after(G_OBJECT(machine), "resize-pads",
+			 G_CALLBACK(ags_syncsynth_resize_pads), NULL);
 
   if((AGS_MACHINE_PREMAPPED_RECALL & (machine->flags)) == 0){
     /* ags-play-notation */
@@ -779,49 +768,46 @@ ags_syncsynth_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin)
 }
 
 void
-ags_syncsynth_set_audio_channels(AgsAudio *audio,
-				 guint audio_channels, guint audio_channels_old,
-				 AgsSyncsynth *syncsynth)
+ags_syncsynth_resize_audio_channels(AgsMachine *machine,
+				    guint audio_channels, guint audio_channels_old,
+				    gpointer data)
 {
-  AgsConfig *config;
+  AgsSyncsynth *syncsynth;
 
-  gchar *str;
+  AgsAudio *audio;
 
-  gboolean performance_mode;
+  guint output_pads, input_pads;
 
-  config = ags_config_get_instance();
+  syncsynth = (AgsSyncsynth *) machine;
+  
+  audio = machine->audio;
+
+  /* get some fields */
+  g_object_get(audio,
+	       "output-pads", &output_pads,
+	       "input-pads", &input_pads,
+	       NULL);
 
   if(audio_channels > audio_channels_old){
-    /* map dependending on output */
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_GENERIC,
-			       "engine-mode");
-    performance_mode = FALSE;
-  
-    if(str != NULL &&
-       !g_ascii_strncasecmp(str,
-			    "performance",
-			    12)){
+    if(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode()){
       /* ags-copy */
       ags_recall_factory_create(audio,
 				NULL, NULL,
 				"ags-copy",
 				audio_channels_old, audio_channels,
-				0, audio->input_pads,
+				0, input_pads,
 				(AGS_RECALL_FACTORY_INPUT |
 				 AGS_RECALL_FACTORY_RECALL |
 				 AGS_RECALL_FACTORY_ADD),
 				0);
-
-      /* set performance mode */
-      performance_mode = TRUE;
     }else{    
       /* ags-buffer */
       ags_recall_factory_create(audio,
 				NULL, NULL,
 				"ags-buffer",
 				audio_channels_old, audio_channels,
-				0, audio->input_pads,
+				0, input_pads,
 				(AGS_RECALL_FACTORY_INPUT |
 				 AGS_RECALL_FACTORY_RECALL |
 				 AGS_RECALL_FACTORY_ADD),
@@ -833,7 +819,7 @@ ags_syncsynth_set_audio_channels(AgsAudio *audio,
 			      NULL, NULL,
 			      "ags-play",
 			      audio_channels_old, audio_channels, 
-			      0, audio->input_pads,
+			      0, input_pads,
 			      (AGS_RECALL_FACTORY_INPUT |
 			       AGS_RECALL_FACTORY_PLAY |
 			       AGS_RECALL_FACTORY_ADD),
@@ -844,7 +830,7 @@ ags_syncsynth_set_audio_channels(AgsAudio *audio,
 			      NULL, NULL,
 			      "ags-envelope",
 			      audio_channels_old, audio_channels, 
-			      0, audio->input_pads,
+			      0, input_pads,
 			      (AGS_RECALL_FACTORY_INPUT |
 			       AGS_RECALL_FACTORY_PLAY |
 			       AGS_RECALL_FACTORY_RECALL | 
@@ -852,25 +838,39 @@ ags_syncsynth_set_audio_channels(AgsAudio *audio,
 			      0);
 
     /* ags-stream */
-    ags_recall_factory_create(audio,
-			      NULL, NULL,
-			      "ags-stream",
-			      audio_channels_old, audio_channels, 
-			      0, audio->input_pads,
-			      (AGS_RECALL_FACTORY_INPUT |
-			       AGS_RECALL_FACTORY_PLAY |
-			       AGS_RECALL_FACTORY_RECALL | 
-			       AGS_RECALL_FACTORY_ADD),
-			      0);
-
+    if(!ags_recall_global_get_rt_safe()){
+      ags_recall_factory_create(audio,
+				NULL, NULL,
+				"ags-stream",
+				audio_channels_old, audio_channels, 
+				0, input_pads,
+				(AGS_RECALL_FACTORY_INPUT |
+				 AGS_RECALL_FACTORY_PLAY |
+				 AGS_RECALL_FACTORY_RECALL | 
+				 AGS_RECALL_FACTORY_ADD),
+				0);
+    }else{
+      ags_recall_factory_create(audio,
+				NULL, NULL,
+				"ags-rt-stream",
+				audio_channels_old, audio_channels, 
+				0, input_pads,
+				(AGS_RECALL_FACTORY_INPUT |
+				 AGS_RECALL_FACTORY_PLAY |
+				 AGS_RECALL_FACTORY_RECALL | 
+				 AGS_RECALL_FACTORY_ADD),
+				0);
+    }
+    
     /* AgsOutput */
     /* ags-stream */
-    if(!performance_mode){
+    if(!(ags_recall_global_get_rt_safe() ||
+	 ags_recall_global_get_performance_mode())){
       ags_recall_factory_create(audio,
 				NULL, NULL,
 				"ags-stream",
 				audio_channels_old, audio_channels,
-				0, audio->output_pads,
+				0, output_pads,
 				(AGS_RECALL_FACTORY_OUTPUT |
 				 AGS_RECALL_FACTORY_PLAY |
 				 AGS_RECALL_FACTORY_ADD),
@@ -880,50 +880,29 @@ ags_syncsynth_set_audio_channels(AgsAudio *audio,
 }
 
 void
-ags_syncsynth_set_pads(AgsAudio *audio, GType type,
-		       guint pads, guint pads_old,
-		       AgsSyncsynth *syncsynth)
+ags_syncsynth_resize_pads(AgsMachine *machine, GType type,
+			  guint pads, guint pads_old,
+			  gpointer data)
 {
   AgsWindow *window;
-  AgsMachine *machine;
+  AgsSyncsynth *syncsynth;
 
+  AgsAudio *audio;
   AgsChannel *channel, *source;
   AgsAudioSignal *audio_signal;
 
-  AgsMutexManager *mutex_manager;
-
-  AgsApplicationContext *application_context;
-  
   guint i, j;
   gboolean grow;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *source_mutex;
-
   if(pads == pads_old){
     return;
   }
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* lookup audio mutex */
-  pthread_mutex_lock(application_mutex);
-    
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
-  gdk_threads_enter();
-  
-  machine = AGS_MACHINE(syncsynth);
   window = (AgsWindow *) gtk_widget_get_toplevel((GtkWidget *) machine);
 
-  application_context = window->application_context;
+  syncsynth = (AgsSyncsynth *) machine;
 
-  gdk_threads_leave();
+  audio = machine->audio;
   
   if(pads_old < pads){
     grow = TRUE;
@@ -933,32 +912,6 @@ ags_syncsynth_set_pads(AgsAudio *audio, GType type,
   
   if(type == AGS_TYPE_INPUT){
     if(grow){
-      pthread_mutex_lock(audio_mutex);
-
-      source = audio->input;
-
-      pthread_mutex_unlock(audio_mutex);
-      
-      /* create pattern */
-      source = ags_channel_nth(source, pads_old);
-      
-      while(source != NULL){
-	/* lookup source mutex */
-	pthread_mutex_lock(application_mutex);
-
-	source_mutex = ags_mutex_manager_lookup(mutex_manager,
-						(GObject *) source);
-  
-	pthread_mutex_unlock(application_mutex);
-
-	/* iterate pattern */
-	pthread_mutex_lock(source_mutex);
-	
-	source = source->next;
-
-	pthread_mutex_unlock(source_mutex);
-      }
-
       if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0){
 	/* depending on destination */
 	ags_syncsynth_input_map_recall(syncsynth,
@@ -972,11 +925,9 @@ ags_syncsynth_set_pads(AgsAudio *audio, GType type,
     if(grow){
       AgsChannel *current, *output;
 
-      pthread_mutex_lock(audio_mutex);
-
-      source = audio->output;
-
-      pthread_mutex_unlock(audio_mutex);
+      g_object_get(audio,
+		   "output", &source,
+		   NULL);
 
       source = ags_channel_nth(source,
 			       pads_old);
@@ -985,31 +936,19 @@ ags_syncsynth_set_pads(AgsAudio *audio, GType type,
 	AgsRecycling *recycling;
 	AgsAudioSignal *audio_signal;
 
-	GObject *soundcard;
-
-	pthread_mutex_lock(audio_mutex);
+	GObject *output_soundcard;
 	
-	soundcard = audio->soundcard;
-
-	pthread_mutex_unlock(audio_mutex);
-
-	/* lookup source mutex */
-	pthread_mutex_lock(application_mutex);
-
-	source_mutex = ags_mutex_manager_lookup(mutex_manager,
-						(GObject *) source);
-  
-	pthread_mutex_unlock(application_mutex);
+	g_object_get(audio,
+		     "output-soundcard", &output_soundcard,
+		     NULL);
 
 	/* get recycling */
-	pthread_mutex_lock(source_mutex);
-
-	recycling = source->first_recycling;
-
-	pthread_mutex_unlock(source_mutex);
+	g_object_get(source,
+		     "first-recycling", &recycling,
+		     NULL);
 
 	/* instantiate template audio signal */
-	audio_signal = ags_audio_signal_new(soundcard,
+	audio_signal = ags_audio_signal_new(output_soundcard,
 					    (GObject *) recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
@@ -1029,247 +968,179 @@ ags_syncsynth_set_pads(AgsAudio *audio, GType type,
 
 
 void
-ags_syncsynth_input_map_recall(AgsSyncsynth *syncsynth, guint input_pad_start)
+ags_syncsynth_input_map_recall(AgsSyncsynth *syncsynth,
+			       guint input_pad_start)
 {
   AgsAudio *audio;
-  AgsChannel *source, *current;
-
-  AgsConfig *config;
 
   GList *list;
 
-  gchar *str;
-
-  gboolean performance_mode;
-
-  audio = AGS_MACHINE(syncsynth)->audio;
+  guint input_pads;
+  guint audio_channels;
 
   if(syncsynth->mapped_input_pad > input_pad_start){
     return;
   }
 
-  config = ags_config_get_instance();
+  audio = AGS_MACHINE(syncsynth)->audio;
 
-  source = ags_channel_nth(audio->input,
-			   input_pad_start * audio->audio_channels);
+  /* get some fields */
+  g_object_get(audio,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
 
-  /* map dependending on output */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "engine-mode");
-  performance_mode = FALSE;
-  
   /* remap for input */
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "performance",
-			  12)){
-    current = source;
-
-    while(current != NULL){
-      /* ags-copy */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-copy",
-				0, audio->audio_channels, 
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      current = current->next_pad;
-    }
-    
-    /* set performance mode */
-    performance_mode = TRUE;
+  if(ags_recall_global_get_rt_safe() ||
+     ags_recall_global_get_performance_mode()){
+    /* ags-copy */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-copy",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }else{    
-    current = source;
-
-    while(current != NULL){
-      /* ags-buffer */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-buffer",
-				0, audio->audio_channels, 
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      current = current->next_pad;
-    }
+    /* ags-buffer */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-buffer",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }
   
-  /*  */
-  current = source;
+  /* ags-play */
+  ags_recall_factory_create(audio,
+			    NULL, NULL,
+			    "ags-play",
+			    0, audio_channels, 
+			    input_pad_start, input_pads,
+			    (AGS_RECALL_FACTORY_INPUT |
+			     AGS_RECALL_FACTORY_PLAY |
+			     AGS_RECALL_FACTORY_ADD),
+			    0);
 
-  while(current != NULL){
-    /* ags-play */
-    ags_recall_factory_create(audio,
-			      NULL, NULL,
-			      "ags-play",
-			      0, audio->audio_channels, 
-			      current->pad, current->pad + 1,
-			      (AGS_RECALL_FACTORY_INPUT |
-			       AGS_RECALL_FACTORY_PLAY |
-			       AGS_RECALL_FACTORY_ADD),
-			      0);
+  /* ags-feed */
+  ags_recall_factory_create(audio,
+			    NULL, NULL,
+			    "ags-feed",
+			    0, audio_channels, 
+			    input_pad_start, input_pads,
+			    (AGS_RECALL_FACTORY_INPUT |
+			     AGS_RECALL_FACTORY_PLAY |
+			     AGS_RECALL_FACTORY_RECALL | 
+			     AGS_RECALL_FACTORY_ADD),
+			    0);
 
-    current = current->next_pad;
-  }
-
-  /*  */
-  current = source;
-
-  while(current != NULL){
-    /* ags-feed */
-    ags_recall_factory_create(audio,
-			      NULL, NULL,
-			      "ags-feed",
-			      0, audio->audio_channels, 
-			      current->pad, current->pad + 1,
-			      (AGS_RECALL_FACTORY_INPUT |
-			       AGS_RECALL_FACTORY_PLAY |
-			       AGS_RECALL_FACTORY_RECALL | 
-			       AGS_RECALL_FACTORY_ADD),
-			      0);
-
-    current = current->next_pad;
-  }
-
-  /*  */
-  current = source;
-
-  while(current != NULL){
-    /* ags-envelope */
-    ags_recall_factory_create(audio,
-			      NULL, NULL,
-			      "ags-envelope",
-			      0, audio->audio_channels, 
-			      current->pad, current->pad + 1,
-			      (AGS_RECALL_FACTORY_INPUT |
-			       AGS_RECALL_FACTORY_PLAY |
-			       AGS_RECALL_FACTORY_RECALL | 
-			       AGS_RECALL_FACTORY_ADD),
-			      0);
-
-    current = current->next_pad;
-  }
+  /* ags-envelope */
+  ags_recall_factory_create(audio,
+			    NULL, NULL,
+			    "ags-envelope",
+			    0, audio_channels, 
+			    input_pad_start, input_pads,
+			    (AGS_RECALL_FACTORY_INPUT |
+			     AGS_RECALL_FACTORY_PLAY |
+			     AGS_RECALL_FACTORY_RECALL | 
+			     AGS_RECALL_FACTORY_ADD),
+			    0);
   
-  /*  */
-  current = source;
-
-  while(current != NULL){
-    /* ags-stream */
+  /* ags-stream */
+  if(!(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode())){
     ags_recall_factory_create(audio,
 			      NULL, NULL,
 			      "ags-stream",
-			      0, audio->audio_channels, 
-			      current->pad, current->pad + 1,
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_RECALL | 
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
+  }else{
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-rt-stream",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
 			      (AGS_RECALL_FACTORY_INPUT |
 			       AGS_RECALL_FACTORY_PLAY |
 			       AGS_RECALL_FACTORY_RECALL | 
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
-
-    current = current->next_pad;
   }
   
-  syncsynth->mapped_input_pad = audio->input_pads;
+  syncsynth->mapped_input_pad = input_pads;
 }
 
 void
-ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth, guint output_pad_start)
+ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth,
+				guint output_pad_start)
 {
   AgsAudio *audio;
-  AgsChannel *source, *input, *current;
 
-  AgsConfig *config;
-
-  gchar *str;
-
-  gboolean performance_mode;
-
-  audio = AGS_MACHINE(syncsynth)->audio;
+  guint input_pad_start;
+  guint output_pads, input_pads;
+  guint audio_channels;
 
   if(syncsynth->mapped_output_pad > output_pad_start){
     return;
   }
 
-  config = ags_config_get_instance();
-  
-  source = ags_channel_nth(audio->output,
-			   output_pad_start * audio->audio_channels);
+  audio = AGS_MACHINE(syncsynth)->audio;
 
+  /* get some fields */
+  g_object_get(audio,
+	       "output-pads", &output_pads,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
   
-  /* map dependending on output */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "engine-mode");
-  performance_mode = FALSE;
-
   /* remap for input */
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "performance",
-			  12)){
-    input = audio->input;
-
-    while(input != NULL){
-      /* ags-copy */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-copy",
-				0, audio->audio_channels, 
-				input->pad, input->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      input = input->next_pad;
-    }
-    
-    /* set performance mode */
-    performance_mode = TRUE;
+  if(ags_recall_global_get_rt_safe() ||
+     ags_recall_global_get_performance_mode()){
+    /* ags-copy */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-copy",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }else{    
-    input = audio->input;
-
-    while(input != NULL){
-      /* ags-buffer */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-buffer",
-				0, audio->audio_channels, 
-				input->pad, input->pad + 1,
-				(AGS_RECALL_FACTORY_INPUT |
-				 AGS_RECALL_FACTORY_RECALL |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      input = input->next_pad;
-    }
+    /* ags-buffer */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-buffer",
+			      0, audio_channels, 
+			      input_pad_start, input_pads,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_RECALL |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }
 
-  if(!performance_mode){
-    current = source;
-
-    while(current != NULL){
-      /* ags-stream */
-      ags_recall_factory_create(audio,
-				NULL, NULL,
-				"ags-stream",
-				0, audio->audio_channels,
-				current->pad, current->pad + 1,
-				(AGS_RECALL_FACTORY_OUTPUT |
-				 AGS_RECALL_FACTORY_PLAY |
-				 AGS_RECALL_FACTORY_ADD),
-				0);
-
-      current = current->next_pad;
-    }
+  if(!(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode())){
+    /* ags-stream */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-stream",
+			      0, audio_channels,
+			      output_pad_start, output_pads,
+			      (AGS_RECALL_FACTORY_OUTPUT |
+			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
   }
   
   syncsynth->mapped_output_pad = audio->output_pads;
@@ -1282,14 +1153,20 @@ ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth, guint output_pad_start)
  * 
  * Add @oscillator to @synthsync.
  * 
- * Since: 0.9.7
+ * Since: 1.0.0
  */
 void
 ags_syncsynth_add_oscillator(AgsSyncsynth *syncsynth,
 			     AgsOscillator *oscillator)
 {
+  AgsAudio *audio;
+  
   GtkHBox *hbox;
   GtkCheckButton *check_button;
+
+  audio = AGS_MACHINE(syncsynth)->audio;
+  ags_audio_add_synth_generator(audio,
+				ags_synth_generator_new());
   
   hbox = (GtkHBox *) gtk_hbox_new(FALSE,
 				  0);
@@ -1322,14 +1199,29 @@ ags_syncsynth_add_oscillator(AgsSyncsynth *syncsynth,
  * 
  * Remove nth oscillator.
  * 
- * Since: 0.9.7
+ * Since: 1.0.0
  */
 void
 ags_syncsynth_remove_oscillator(AgsSyncsynth *syncsynth,
 				guint nth)
 {
+  AgsAudio *audio;
+  
   GList *list, *list_start;
+  GList *start_synth_generator;
+  
+  audio = AGS_MACHINE(syncsynth)->audio;
+  g_object_get(audio,
+	       "synth-generator", &start_synth_generator,
+	       NULL);
 
+  start_synth_generator = g_list_reverse(start_synth_generator);
+  ags_audio_remove_synth_generator(audio,
+				   g_list_nth_data(start_synth_generator,
+						   nth));
+
+  g_list_free(start_synth_generator);
+  
   list =
     list_start = gtk_container_get_children(syncsynth->oscillator);
 
@@ -1349,7 +1241,7 @@ ags_syncsynth_remove_oscillator(AgsSyncsynth *syncsynth,
  * 
  * Reset loop spin buttons.
  * 
- * Since: 0.9.7
+ * Since: 1.0.0
  */
 void
 ags_syncsynth_reset_loop(AgsSyncsynth *syncsynth)
@@ -1393,7 +1285,7 @@ ags_syncsynth_reset_loop(AgsSyncsynth *syncsynth)
  * 
  * Update audio data.
  * 
- * Since: 0.9.7
+ * Since: 1.0.0
  */
 void
 ags_syncsynth_update(AgsSyncsynth *syncsynth)
@@ -1407,56 +1299,34 @@ ags_syncsynth_update(AgsSyncsynth *syncsynth)
   AgsClearAudioSignal *clear_audio_signal;
   AgsApplySynth *apply_synth;
 
-  AgsMutexManager *mutex_manager;
-  AgsThread *main_loop;
-  AgsTaskThread *task_thread;
+  AgsGuiThread *gui_thread;
 
   AgsApplicationContext *application_context;
   
   GList *list, *list_start;
+  GList *start_synth_generator, *synth_generator;    
   GList *child_start;
   GList *task;
   
   guint input_lines;
+  guint buffer_size;
+  guint format;
   guint wave;
   guint attack, frame_count;
   gdouble frequency, phase, start_frequency;
   guint loop_start, loop_end;
   gdouble volume;
 
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *channel_mutex;
-  pthread_mutex_t *application_mutex;
+  AgsComplex **sync_point;
+  guint sync_point_count;
 
-  gdk_threads_enter();
-  
   window = (AgsWindow *) gtk_widget_get_toplevel((GtkWidget *) syncsynth);
-  application_context = (AgsApplicationContext *) window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  application_context = (AgsApplicationContext *) window->application_context;
+  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
 
   audio = AGS_MACHINE(syncsynth)->audio;
 
-  /* get audio loop */
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
-
-  pthread_mutex_unlock(application_mutex);
-
-  /* get task thread */
-  task_thread = (AgsTaskThread *) ags_thread_find_type(main_loop,
-						       AGS_TYPE_TASK_THREAD);
-
-  /* lookup audio mutex */
-  pthread_mutex_lock(application_mutex);
-    
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /*  */
   start_frequency = (gdouble) gtk_spin_button_get_value_as_float(syncsynth->lower);
 
@@ -1464,69 +1334,63 @@ ags_syncsynth_update(AgsSyncsynth *syncsynth)
   loop_end = (guint) gtk_spin_button_get_value_as_int(syncsynth->loop_end);
 
   /* clear input */
-  pthread_mutex_lock(audio_mutex);
-
-  channel = audio->input;
-
-  pthread_mutex_unlock(audio_mutex);
+  g_object_get(audio,
+	       "input", &channel,
+	       NULL);
 
   task = NULL;
 
   while(channel != NULL){
+    AgsRecycling *first_recycling;
     AgsAudioSignal *template;
+
+    GList *start_list;
     
-    /* lookup channel mutex */
-    pthread_mutex_lock(application_mutex);
-
-    channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     (GObject *) channel);
-  
-    pthread_mutex_unlock(application_mutex);
+    g_object_get(channel,
+		 "first-recycling", &first_recycling,
+		 NULL);
 
+    g_object_get(first_recycling,
+		 "audio-signal", &start_list,
+		 NULL);
+    
     /* clear task */
-    pthread_mutex_lock(channel_mutex);
-
-    template = ags_audio_signal_get_template(channel->first_recycling->audio_signal);
-
-    pthread_mutex_unlock(channel_mutex);
+    template = ags_audio_signal_get_template(start_list);
 
     clear_audio_signal = ags_clear_audio_signal_new(template);
     task = g_list_prepend(task,
 			  clear_audio_signal);
+
+    g_list_free(start_list);
     
     /* iterate */
-    pthread_mutex_lock(channel_mutex);
-
-    channel = channel->next;
-
-    pthread_mutex_unlock(channel_mutex);
+    g_object_get(channel,
+		 "next", &channel,
+		 NULL);
   }
 
   /* write input */
   list =
     list_start = gtk_container_get_children(syncsynth->oscillator);
 
-  /* get soundcard */
-  pthread_mutex_lock(audio_mutex);
+  /* get some fields */
+  g_object_get(audio,
+	       "input", &channel,
+	       "input-lines", &input_lines,
+	       "synth-generator", &start_synth_generator,
+	       NULL);
 
-  channel = audio->input;
+  g_object_get(channel,
+	       "buffer-size", &buffer_size,
+	       "format", &format,
+	       NULL);
 
-  input_lines = audio->input_lines;
-  
-  pthread_mutex_unlock(audio_mutex);
+  synth_generator = start_synth_generator;
 
   while(list != NULL){
-    guint sync_mode;
+    guint i;
     gboolean do_sync;
     
-    /* lookup channel mutex */
-    pthread_mutex_lock(application_mutex);
-
-    channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     (GObject *) channel);
-  
-    pthread_mutex_unlock(application_mutex);
-
     /* do it so */
     child_start = gtk_container_get_children(list->data);
 
@@ -1534,60 +1398,94 @@ ags_syncsynth_update(AgsSyncsynth *syncsynth)
 
     g_list_free(child_start);
         
-    wave = (guint) gtk_combo_box_get_active(oscillator->wave) + 1;
+    wave = (guint)  + 1;
     attack = (guint) gtk_spin_button_get_value_as_int(oscillator->attack);
     frame_count = (guint) gtk_spin_button_get_value_as_int(oscillator->frame_count);
     phase = (gdouble) gtk_spin_button_get_value_as_float(oscillator->phase);
     frequency = (gdouble) gtk_spin_button_get_value_as_float(oscillator->frequency);
     volume = (gdouble) gtk_spin_button_get_value_as_float(oscillator->volume);
 
-    apply_synth = ags_apply_synth_new(channel, input_lines,
-				      wave,
-				      attack, frame_count,
-				      frequency, phase, start_frequency,
-				      volume,
-				      loop_start, loop_end);
+    g_object_set(synth_generator->data,
+		 "format", format,
+		 "delay", (gdouble) attack / buffer_size,
+		 "attack", attack,
+		 "frame-count", frame_count,
+		 "oscillator", gtk_combo_box_get_active(oscillator->wave),
+		 "frequency", frequency,
+		 "phase", phase,
+		 "volume", volume,
+		 NULL);
 
     do_sync = gtk_toggle_button_get_active(oscillator->do_sync);
 
-    sync_mode = 1 << gtk_combo_box_get_active(oscillator->sync_mode);
-    
-    g_object_set(apply_synth,
-		 "do-sync", do_sync,
-		 "sync-mode", sync_mode,
-		 "base-note", syncsynth->lower->adjustment->value,
-		 NULL);
-    
+    if(do_sync){
+      sync_point_count = oscillator->sync_point_count;
+      
+      /* free previous sync point */
+      if(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point != NULL){
+	for(i = 0; i < AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point_count; i++){
+	  ags_complex_free(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i]);
+	}
+
+	free(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point);
+      }
+
+      /* set new sync point */
+      if(sync_point_count > 0){
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point = (AgsComplex **) malloc(sync_point_count * sizeof(AgsComplex *));
+      }else{
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point = NULL;
+      }
+
+      AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point_count = sync_point_count;
+
+      for(i = 0; i < sync_point_count; i++){
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i] = ags_complex_alloc();
+
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i][0][0] = gtk_spin_button_get_value(oscillator->sync_point[2 * i]);
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i][0][1] = gtk_spin_button_get_value(oscillator->sync_point[2 * i + 1]);
+      }
+    }else{
+      for(i = 0; i < AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point_count; i++){
+	ags_complex_free(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i]);
+      }
+
+      free(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point);
+      
+      AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point = NULL;
+      AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point_count = 0;
+    }
+					
+    apply_synth = ags_apply_synth_new(synth_generator->data,
+				      channel,
+				      start_frequency, input_lines);
+        
     task = g_list_prepend(task,
 			  apply_synth);
 
     /* iterate */
-    pthread_mutex_lock(channel_mutex);
-
-    channel = channel->next;
-
-    pthread_mutex_unlock(channel_mutex);
+    synth_generator = synth_generator->next;
     
     list = list->next;
   }
+
+  g_list_free(start_synth_generator);
   
   g_list_free(list_start);
-  
-  gdk_threads_leave();
 
-  ags_task_thread_append_tasks(task_thread,
-			       g_list_reverse(task));
+  ags_gui_thread_schedule_task_list(gui_thread,
+				    g_list_reverse(task));
 }
 
 /**
  * ags_syncsynth_new:
  * @soundcard: the assigned soundcard.
  *
- * Creates an #AgsSyncsynth
+ * Create a new instance of #AgsSyncsynth
  *
- * Returns: a new #AgsSyncsynth
+ * Returns: the new #AgsSyncsynth
  *
- * Since: 0.9.7
+ * Since: 2.0.0
  */
 AgsSyncsynth*
 ags_syncsynth_new(GObject *soundcard)
@@ -1597,8 +1495,8 @@ ags_syncsynth_new(GObject *soundcard)
   syncsynth = (AgsSyncsynth *) g_object_new(AGS_TYPE_SYNCSYNTH,
 					    NULL);
 
-  g_object_set(G_OBJECT(AGS_MACHINE(syncsynth)->audio),
-	       "soundcard", soundcard,
+  g_object_set(AGS_MACHINE(syncsynth)->audio,
+	       "output-soundcard", soundcard,
 	       NULL);
 
   return(syncsynth);
