@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,31 +19,12 @@
 
 #include <ags/audio/ags_audio_application_context.h>
 
-#include <ags/util/ags_id_generator.h>
-
-#include <ags/object/ags_distributed_manager.h>
-#include <ags/object/ags_connectable.h>
-#include <ags/object/ags_config.h>
-#include <ags/object/ags_main_loop.h>
-#include <ags/object/ags_soundcard.h>
-#include <ags/object/ags_sequencer.h>
-
-#include <ags/file/ags_file.h>
-#include <ags/file/ags_file_stock.h>
-#include <ags/file/ags_file_id_ref.h>
-
-#include <ags/thread/ags_concurrency_provider.h>
-#include <ags/thread/ags_mutex_manager.h>
-#include <ags/thread/ags_thread-posix.h>
-#include <ags/thread/ags_thread_pool.h>
-#include <ags/thread/ags_task_thread.h>
-#include <ags/thread/ags_single_thread.h>
-#include <ags/thread/ags_autosave_thread.h>
+#include <ags/libags.h>
 
 #include <ags/audio/ags_sound_provider.h>
 #include <ags/audio/ags_devout.h>
 #include <ags/audio/ags_midiin.h>
-#include <ags/audio/ags_recall_channel_run_dummy.h>
+#include <ags/audio/ags_generic_recall_channel_run.h>
 #include <ags/audio/ags_recall_ladspa.h>
 #include <ags/audio/ags_recall_ladspa_run.h>
 #include <ags/audio/ags_recall_lv2.h>
@@ -51,6 +32,12 @@
 #include <ags/audio/ags_recall_dssi.h>
 #include <ags/audio/ags_recall_dssi_run.h>
 
+#include <ags/audio/core-audio/ags_core_audio_midiin.h>
+#include <ags/audio/core-audio/ags_core_audio_server.h>
+#include <ags/audio/core-audio/ags_core_audio_client.h>
+#include <ags/audio/core-audio/ags_core_audio_port.h>
+#include <ags/audio/core-audio/ags_core_audio_devout.h>
+
 #include <ags/audio/jack/ags_jack_midiin.h>
 #include <ags/audio/jack/ags_jack_server.h>
 #include <ags/audio/jack/ags_jack_client.h>
@@ -97,16 +84,10 @@
 #include <ags/audio/thread/ags_soundcard_thread.h>
 #include <ags/audio/thread/ags_sequencer_thread.h>
 #include <ags/audio/thread/ags_export_thread.h>
-#include <ags/audio/thread/ags_record_thread.h>
-#include <ags/audio/thread/ags_recycling_thread.h>
-#include <ags/audio/thread/ags_iterator_thread.h>
 
 #include <sys/types.h>
 #include <pwd.h>
 
-#include <jack/jslist.h>
-#include <jack/jack.h>
-#include <jack/control.h>
 #include <stdbool.h>
 
 #include <ags/i18n.h>
@@ -124,6 +105,9 @@ void ags_audio_application_context_get_property(GObject *gobject,
 						guint prop_id,
 						GValue *value,
 						GParamSpec *param_spec);
+void ags_audio_application_context_dispose(GObject *gobject);
+void ags_audio_application_context_finalize(GObject *gobject);
+
 void ags_audio_application_context_connect(AgsConnectable *connectable);
 void ags_audio_application_context_disconnect(AgsConnectable *connectable);
 
@@ -144,9 +128,10 @@ void ags_audio_application_context_set_soundcard(AgsSoundProvider *sound_provide
 GList* ags_audio_application_context_get_sequencer(AgsSoundProvider *sound_provider);
 void ags_audio_application_context_set_sequencer(AgsSoundProvider *sound_provider,
 						 GList *sequencer);
-GList* ags_audio_application_context_get_distributed_manager(AgsSoundProvider *sound_provider);
-void ags_audio_application_context_dispose(GObject *gobject);
-void ags_audio_application_context_finalize(GObject *gobject);
+GList* ags_audio_application_context_get_sound_server(AgsSoundProvider *sound_provider);
+GList* ags_audio_application_context_get_audio(AgsSoundProvider *sound_provider);
+void ags_audio_application_context_set_audio(AgsSoundProvider *sound_provider,
+					     GList *soundcard);
 
 void ags_audio_application_context_register_types(AgsApplicationContext *application_context);
 void ags_audio_application_context_load_config(AgsApplicationContext *application_context);
@@ -256,7 +241,7 @@ ags_audio_application_context_class_init(AgsAudioApplicationContextClass *audio_
    *
    * The assigned soundcard.
    * 
-   * Since: 0.4
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_object("soundcard",
 				   i18n_pspec("soundcard of audio application context"),
@@ -300,11 +285,17 @@ ags_audio_application_context_sound_provider_interface_init(AgsSoundProviderInte
 {
   sound_provider->get_soundcard = ags_audio_application_context_get_soundcard;
   sound_provider->set_soundcard = ags_audio_application_context_set_soundcard;
+
   sound_provider->get_default_soundcard_thread = ags_audio_application_context_get_default_soundcard_thread;
   sound_provider->set_default_soundcard_thread = ags_audio_application_context_set_default_soundcard_thread;
+
   sound_provider->get_sequencer = ags_audio_application_context_get_sequencer;
   sound_provider->set_sequencer = ags_audio_application_context_set_sequencer;
-  sound_provider->get_distributed_manager = ags_audio_application_context_get_distributed_manager;
+
+  sound_provider->get_sound_server = ags_audio_application_context_get_sound_server;
+
+  sound_provider->get_audio = ags_audio_application_context_get_audio;
+  sound_provider->set_audio = ags_audio_application_context_set_audio;
 }
 
 void
@@ -317,7 +308,8 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
   GObject *sequencer;
   AgsJackServer *jack_server;
   AgsPulseServer *pulse_server;
-
+  AgsCoreAudioServer *core_audio_server;
+  
   AgsThread *soundcard_thread;
   AgsThread *sequencer_thread;
   AgsThread *export_thread;
@@ -325,13 +317,13 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
   AgsConfig *config;
 
   GList *list;
-  JSList *jslist;
 
   gchar *soundcard_group;
   gchar *sequencer_group;
   gchar *str;
 
   guint i;
+  gboolean has_core_audio;
   gboolean has_pulse;
   gboolean has_jack;
 
@@ -350,13 +342,22 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
 	       NULL);
 
   /* distributed manager */
-  audio_application_context->distributed_manager = NULL;
+  audio_application_context->sound_server = NULL;
+
+  /* core audio server */
+  core_audio_server = ags_core_audio_server_new((GObject *) audio_application_context,
+						NULL);
+  audio_application_context->sound_server = g_list_append(audio_application_context->sound_server,
+							  core_audio_server);
+  g_object_ref(G_OBJECT(core_audio_server));
+
+  has_core_audio = FALSE;
 
   /* pulse server */
   pulse_server = ags_pulse_server_new((GObject *) audio_application_context,
 				      NULL);
-  audio_application_context->distributed_manager = g_list_append(audio_application_context->distributed_manager,
-								 pulse_server);
+  audio_application_context->sound_server = g_list_append(audio_application_context->sound_server,
+							  pulse_server);
   g_object_ref(G_OBJECT(pulse_server));
 
   has_pulse = FALSE;
@@ -364,8 +365,8 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
   /* jack server */
   jack_server = ags_jack_server_new((GObject *) audio_application_context,
 				    NULL);
-  audio_application_context->distributed_manager = g_list_append(audio_application_context->distributed_manager,
-								 jack_server);
+  audio_application_context->sound_server = g_list_append(audio_application_context->sound_server,
+							  jack_server);
   g_object_ref(G_OBJECT(jack_server));
 
   has_jack = FALSE;
@@ -398,17 +399,24 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
     
     if(str != NULL){
       if(!g_ascii_strncasecmp(str,
-			      "pulse",
-			      6)){
-	soundcard = ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(pulse_server),
-							       TRUE);
+			      "core-audio",
+			      11)){
+	soundcard = ags_sound_server_register_soundcard(AGS_SOUND_SERVER(core_audio_server),
+							TRUE);
+
+	has_core_audio = TRUE;
+      }else if(!g_ascii_strncasecmp(str,
+				    "pulse",
+				    6)){
+	soundcard = ags_sound_server_register_soundcard(AGS_SOUND_SERVER(pulse_server),
+							TRUE);
 
 	has_pulse = TRUE;
       }else if(!g_ascii_strncasecmp(str,
-			      "jack",
-			      5)){
-	soundcard = ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(jack_server),
-							       TRUE);
+				    "jack",
+				    5)){
+	soundcard = ags_sound_server_register_soundcard(AGS_SOUND_SERVER(jack_server),
+							TRUE);
 
 	has_jack = TRUE;
       }else if(!g_ascii_strncasecmp(str,
@@ -561,8 +569,8 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
       if(!g_ascii_strncasecmp(str,
 			      "jack",
 			      5)){
-	sequencer = ags_distributed_manager_register_sequencer(AGS_DISTRIBUTED_MANAGER(jack_server),
-							       FALSE);
+	sequencer = ags_sound_server_register_sequencer(AGS_SOUND_SERVER(jack_server),
+							FALSE);
 
 	has_jack = TRUE;
       }else if(!g_ascii_strncasecmp(str,
@@ -626,7 +634,7 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
     if(jack_enabled){
     GObject *tmp;
     
-    tmp = ags_distributed_manager_register_sequencer(AGS_DISTRIBUTED_MANAGER(jack_server),
+    tmp = ags_sound_server_register_sequencer(AGS_SOUND_SERVER(jack_server),
     FALSE);
 
     if(tmp != NULL){
@@ -641,7 +649,11 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
   
   /* AgsServer */
   audio_application_context->server = ags_server_new((GObject *) audio_application_context);
+  g_object_ref(audio_application_context->server);
 
+  /* AgsAudio */
+  audio_application_context->audio = NULL;
+  
   /* AgsAudioLoop */
   audio_loop = ags_audio_loop_new((GObject *) soundcard,
 				  (GObject *) audio_application_context);
@@ -685,6 +697,8 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
       AGS_JACK_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
     }else if(AGS_IS_PULSE_DEVOUT(list->data)){
       AGS_PULSE_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
+    }else if(AGS_IS_CORE_AUDIO_DEVOUT(list->data)){
+      AGS_CORE_AUDIO_DEVOUT(list->data)->notify_soundcard = notify_soundcard;
     }
 
     ags_task_thread_append_cyclic_task(AGS_APPLICATION_CONTEXT(audio_application_context)->task_thread,
@@ -760,6 +774,10 @@ ags_audio_application_context_init(AgsAudioApplicationContext *audio_application
   audio_application_context->thread_pool = AGS_TASK_THREAD(AGS_APPLICATION_CONTEXT(audio_application_context)->task_thread)->thread_pool;
 
   /* launch */
+  if(has_core_audio){
+    ags_core_audio_server_connect_client(core_audio_server);
+  }
+
   if(has_pulse){
     ags_pulse_server_connect_client(pulse_server);
 
@@ -901,14 +919,50 @@ ags_audio_application_context_set_worker(AgsConcurrencyProvider *concurrency_pro
 GList*
 ags_audio_application_context_get_soundcard(AgsSoundProvider *sound_provider)
 {
-  return(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->soundcard);
+  GList *soundcard;
+
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get soundcard */
+  pthread_mutex_lock(application_context_mutex);
+  
+  soundcard = g_list_copy(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->soundcard);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(soundcard);
 }
 
 void
 ags_audio_application_context_set_soundcard(AgsSoundProvider *sound_provider,
 					    GList *soundcard)
 {
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* set soundcard */
+  pthread_mutex_lock(application_context_mutex);
+
+  if(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->soundcard != NULL){
+    g_list_free(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->soundcard);
+  }
+
   AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->soundcard = soundcard;
+
+  pthread_mutex_unlock(application_context_mutex);
 }
 
 GObject*
@@ -927,20 +981,123 @@ ags_audio_application_context_set_default_soundcard_thread(AgsSoundProvider *sou
 GList*
 ags_audio_application_context_get_sequencer(AgsSoundProvider *sound_provider)
 {
-  return(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->sequencer);
+  GList *sequencer;
+
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get sequencer */
+  pthread_mutex_lock(application_context_mutex);
+  
+  sequencer = g_list_copy(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->sequencer);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(sequencer);
 }
 
 void
 ags_audio_application_context_set_sequencer(AgsSoundProvider *sound_provider,
 					    GList *sequencer)
 {
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* set sequencer */
+  pthread_mutex_lock(application_context_mutex);
+
+  if(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->sequencer != NULL){
+    g_list_free(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->sequencer);
+  }
+  
   AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->sequencer = sequencer;
+
+  pthread_mutex_unlock(application_context_mutex);
 }
 
 GList*
-ags_audio_application_context_get_distributed_manager(AgsSoundProvider *sound_provider)
+ags_audio_application_context_get_sound_server(AgsSoundProvider *sound_provider)
 {
-  return(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->distributed_manager);
+  GList *sound_server;
+
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get sound server */
+  pthread_mutex_lock(application_context_mutex);
+  
+  sound_server = g_list_copy(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->sound_server);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(sound_server);
+}
+
+GList*
+ags_audio_application_context_get_audio(AgsSoundProvider *sound_provider)
+{
+  GList *audio;
+
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get audio */
+  pthread_mutex_lock(application_context_mutex);
+  
+  audio = g_list_copy(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->audio);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(audio);
+}
+
+void
+ags_audio_application_context_set_audio(AgsSoundProvider *sound_provider,
+					GList *audio)
+{
+  pthread_mutex_t *application_context_mutex;
+
+  /* get mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = AGS_APPLICATION_CONTEXT(sound_provider)->obj_mutex;
+
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* set audio */
+  pthread_mutex_lock(application_context_mutex);
+
+  if(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->audio != NULL){
+    g_list_free(AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->audio);
+  }
+
+  AGS_AUDIO_APPLICATION_CONTEXT(sound_provider)->audio = audio;
+
+  pthread_mutex_unlock(application_context_mutex);
 }
 
 void
@@ -1019,8 +1176,8 @@ ags_audio_application_context_dispose(GObject *gobject)
   }
 
   /* distributed manager */
-  if(audio_application_context->distributed_manager != NULL){
-    list = audio_application_context->distributed_manager;
+  if(audio_application_context->sound_server != NULL){
+    list = audio_application_context->sound_server;
 
     while(list != NULL){
       g_object_set(list->data,
@@ -1030,10 +1187,10 @@ ags_audio_application_context_dispose(GObject *gobject)
       list = list->next;
     }
 
-    g_list_free_full(audio_application_context->distributed_manager,
+    g_list_free_full(audio_application_context->sound_server,
 		     g_object_unref);
 
-    audio_application_context->distributed_manager = NULL;
+    audio_application_context->sound_server = NULL;
   }
 
   /* call parent */
@@ -1073,8 +1230,8 @@ ags_audio_application_context_finalize(GObject *gobject)
 		     g_object_unref);
   }
   
-  if(audio_application_context->distributed_manager != NULL){
-    g_list_free_full(audio_application_context->distributed_manager,
+  if(audio_application_context->sound_server != NULL){
+    g_list_free_full(audio_application_context->sound_server,
 		     g_object_unref);
   }
 
@@ -1171,9 +1328,6 @@ ags_audio_application_context_register_types(AgsApplicationContext *application_
   ags_audio_loop_get_type();
   ags_soundcard_thread_get_type();
   ags_export_thread_get_type();
-  ags_record_thread_get_type();
-  ags_iterator_thread_get_type();
-  ags_recycling_thread_get_type();
 
   /* register recalls */
   ags_play_audio_get_type();
@@ -1197,7 +1351,7 @@ ags_audio_application_context_register_types(AgsApplicationContext *application_
   ags_peak_channel_run_get_type();
 
   ags_recall_ladspa_get_type();
-  ags_recall_channel_run_dummy_get_type();
+  ags_generic_recall_channel_run_get_type();
   ags_recall_ladspa_run_get_type();
 
   ags_delay_audio_get_type();
