@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,8 +19,6 @@
 
 #include <ags/audio/ags_note.h>
 
-#include <ags/object/ags_connectable.h>
-
 #include <ags/audio/midi/ags_midi_buffer_util.h>
 
 #include <stdlib.h>
@@ -29,7 +27,6 @@
 #include <ags/i18n.h>
 
 void ags_note_class_init(AgsNoteClass *note);
-void ags_note_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_note_init(AgsNote *note);
 void ags_note_set_property(GObject *gobject,
 			   guint prop_id,
@@ -39,13 +36,11 @@ void ags_note_get_property(GObject *gobject,
 			   guint prop_id,
 			   GValue *value,
 			   GParamSpec *param_spec);
-void ags_note_connect(AgsConnectable *connectable);
-void ags_note_disconnect(AgsConnectable *connectable);
 void ags_note_finalize(GObject *gobject);
 
 /**
  * SECTION:ags_note
- * @short_description: Note class.
+ * @short_description: note class
  * @title: AgsNote
  * @section_id:
  * @include: ags/audio/ags_note.h
@@ -53,23 +48,29 @@ void ags_note_finalize(GObject *gobject);
  * #AgsNote represents a tone.
  */
 
-static gpointer ags_note_parent_class = NULL;
-
 enum{
   PROP_0,
   PROP_X0,
   PROP_X1,
   PROP_Y,
+  PROP_RT_OFFSET,
+  PROP_RT_ATTACK,
   PROP_STREAM_DELAY,
   PROP_STREAM_ATTACK,
+  PROP_STREAM_FRAME_COUNT,
   PROP_ATTACK,
   PROP_DECAY,
   PROP_SUSTAIN,
   PROP_RELEASE,
   PROP_RATIO,
   PROP_NOTE_NAME,
+  PROP_FREQUENCY,
 };
 
+static gpointer ags_note_parent_class = NULL;
+
+static pthread_mutex_t ags_note_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 GType
 ags_note_get_type()
 {
@@ -88,20 +89,10 @@ ags_note_get_type()
       (GInstanceInitFunc) ags_note_init,
     };
 
-    static const GInterfaceInfo ags_connectable_interface_info = {
-      (GInterfaceInitFunc) ags_note_connectable_interface_init,
-      NULL, /* interface_finalize */
-      NULL, /* interface_data */
-    };
-
     ags_type_note = g_type_register_static(G_TYPE_OBJECT,
 					   "AgsNote",
 					   &ags_note_info,
 					   0);
-    
-    g_type_add_interface_static(ags_type_note,
-				AGS_TYPE_CONNECTABLE,
-				&ags_connectable_interface_info);
   }
 
   return(ags_type_note);
@@ -128,13 +119,13 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * Note offset x0.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("x0",
 				 i18n_pspec("offset x0"),
 				 i18n_pspec("The first x offset"),
 				 0,
-				 65535,
+				 G_MAXUINT32,
 				 0,
 				 G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
@@ -146,13 +137,13 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * Note offset x1.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("x1",
 				 i18n_pspec("offset x1"),
 				 i18n_pspec("The last x offset"),
 				 0,
-				 65535,
+				 G_MAXUINT32,
 				 0,
 				 G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
@@ -164,32 +155,67 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * Note offset y.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("y",
 				 i18n_pspec("offset y"),
 				 i18n_pspec("The y offset"),
 				 0,
-				 65535,
+				 G_MAXUINT32,
 				 0,
 				 G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_Y,
 				  param_spec);
 
+  /**
+   * AgsNote:rt-offset:
+   *
+   * Note realtime offset.
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_uint64("rt-offset",
+				   i18n_pspec("realtime offset"),
+				   i18n_pspec("The realtime offset"),
+				   0,
+				   G_MAXUINT64,
+				   0,
+				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_RT_OFFSET,
+				  param_spec);
   
+  /**
+   * AgsNote:rt-attack:
+   *
+   * Note realtime attack.
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_uint("rt-attack",
+				 i18n_pspec("realtime attack"),
+				 i18n_pspec("The realtime attack"),
+				 0,
+				 G_MAXUINT32,
+				 0,
+				 G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_RT_ATTACK,
+				  param_spec);
+
   /**
    * AgsNote:stream-delay:
    *
    * The stream's delay.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_double("stream-delay",
 				   i18n_pspec("delay of stream"),
 				   i18n_pspec("The delay of the stream"),
 				   0.0,
-				   65535.0,
+				   G_MAXDOUBLE,
 				   0.0,
 				   G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
@@ -201,25 +227,43 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * The stream's attack.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
-  param_spec = g_param_spec_uint("stream-attack",
-				 i18n_pspec("offset stream-attack"),
-				 i18n_pspec("The first x offset"),
-				 0,
-				 65535,
-				 0,
-				 G_PARAM_READABLE | G_PARAM_WRITABLE);
+  param_spec = g_param_spec_double("stream-attack",
+				   i18n_pspec("stream attack offset"),
+				   i18n_pspec("The first x offset"),
+				   0.0,
+				   G_MAXDOUBLE,
+				   0.0,
+				   G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_STREAM_ATTACK,
 				  param_spec);
 
+  /**
+   * AgsNote:stream-frame-count:
+   *
+   * The stream's frame count.
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_uint64("stream-frame-count",
+				   i18n_pspec("stream frame count"),
+				   i18n_pspec("The stream frame count"),
+				   0,
+				   G_MAXUINT64,
+				   0,
+				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_STREAM_FRAME_COUNT,
+				  param_spec);
+  
   /**
    * AgsNote:attack:
    *
    * Envelope attack.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_boxed("attack",
 				  i18n_pspec("envelope's attack"),
@@ -235,7 +279,7 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * Envelope decay.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_boxed("decay",
 				  i18n_pspec("envelope's decay"),
@@ -251,7 +295,7 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * Envelope sustain.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_boxed("sustain",
 				  i18n_pspec("envelope's sustain"),
@@ -267,7 +311,7 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * Envelope release.
    * 
-   * Since: 0.7.2
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_boxed("release",
 				  i18n_pspec("envelope's release"),
@@ -283,7 +327,7 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * Envelope ratio.
    * 
-   * Since: 0.7.42
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_boxed("ratio",
 				  i18n_pspec("envelope's ratio"),
@@ -299,7 +343,7 @@ ags_note_class_init(AgsNoteClass *note)
    *
    * The note's name.
    * 
-   * Since: 0.9.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("note-name",
 				   i18n_pspec("note name"),
@@ -309,13 +353,24 @@ ags_note_class_init(AgsNoteClass *note)
   g_object_class_install_property(gobject,
 				  PROP_NOTE_NAME,
 				  param_spec);
-}
 
-void
-ags_note_connectable_interface_init(AgsConnectableInterface *connectable)
-{
-  connectable->connect = ags_note_connect;
-  connectable->disconnect = ags_note_disconnect;
+  /**
+   * AgsNote:frequency:
+   *
+   * The note's frequency.
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_double("frequency",
+				   i18n_pspec("frequency"),
+				   i18n_pspec("The note's frequency"),
+				   0.0,
+				   G_MAXDOUBLE,
+				   0.0,
+				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_FREQUENCY,
+				  param_spec);
 }
 
 void
@@ -323,12 +378,36 @@ ags_note_init(AgsNote *note)
 {
   complex z;
   
+  pthread_mutex_t *mutex;
+  pthread_mutexattr_t *attr;
+
   note->flags = 0;
 
+  /* add note mutex */
+  note->obj_mutexattr = 
+    attr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+  pthread_mutexattr_init(attr);
+  pthread_mutexattr_settype(attr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+#ifdef __linux__
+  pthread_mutexattr_setprotocol(attr,
+				PTHREAD_PRIO_INHERIT);
+#endif
+
+  note->obj_mutex = 
+    mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(mutex,
+		     attr);  
+
+  /* fields */
   note->x[0] = 0;
-  note->x[1] = 0;
+  note->x[1] = 1;
   note->y = 0;
 
+  note->rt_offset = 0;
+  note->rt_attack = 0;
+  
   note->stream_delay = 0.0;
   note->stream_attack = 0;
 
@@ -356,34 +435,6 @@ ags_note_init(AgsNote *note)
   note->frequency = 440.0;
 }
 
-void
-ags_note_connect(AgsConnectable *connectable)
-{
-  AgsNote *note;
-
-  note = AGS_NOTE(connectable);
-
-  if((AGS_NOTE_CONNECTED & (note->flags)) != 0){
-    return;
-  }
-
-  note->flags |= AGS_NOTE_CONNECTED;
-}
-
-void
-ags_note_disconnect(AgsConnectable *connectable)
-{
-  AgsNote *note;
-
-  note = AGS_NOTE(connectable);
-
-  if((AGS_NOTE_CONNECTED & (note->flags)) == 0){
-    return;
-  }
-
-  note->flags &= (~AGS_NOTE_CONNECTED);
-}
-
 void
 ags_note_set_property(GObject *gobject,
 		      guint prop_id,
@@ -392,32 +443,88 @@ ags_note_set_property(GObject *gobject,
 {
   AgsNote *note;
 
+  pthread_mutex_t *note_mutex;
+
   note = AGS_NOTE(gobject);
 
+  /* get note mutex */
+  pthread_mutex_lock(ags_note_get_class_mutex());
+  
+  note_mutex = note->obj_mutex;
+  
+  pthread_mutex_unlock(ags_note_get_class_mutex());
+
   switch(prop_id){
   case PROP_X0:
     {
+      pthread_mutex_lock(note_mutex);
+
       note->x[0] = g_value_get_uint(value);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_X1:
     {
+      pthread_mutex_lock(note_mutex);
+
       note->x[1] = g_value_get_uint(value);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_Y:
     {
+      pthread_mutex_lock(note_mutex);
+
       note->y = g_value_get_uint(value);
+
+      pthread_mutex_unlock(note_mutex);
+    }
+    break;
+  case PROP_RT_OFFSET:
+    {
+      pthread_mutex_lock(note_mutex);
+
+      note->rt_offset = g_value_get_uint64(value);
+
+      pthread_mutex_unlock(note_mutex);
+    }
+    break;
+  case PROP_RT_ATTACK:
+    {
+      pthread_mutex_lock(note_mutex);
+
+      note->rt_attack = g_value_get_uint(value);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_STREAM_DELAY:
     {
+      pthread_mutex_lock(note_mutex);
+
       note->stream_delay = g_value_get_double(value);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_STREAM_ATTACK:
     {
-      note->stream_attack = g_value_get_uint(value);
+      pthread_mutex_lock(note_mutex);
+
+      note->stream_attack = g_value_get_double(value);
+
+      pthread_mutex_unlock(note_mutex);
+    }
+    break;
+  case PROP_STREAM_FRAME_COUNT:
+    {
+      pthread_mutex_lock(note_mutex);
+
+      note->stream_frame_count = g_value_get_uint64(value);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_ATTACK:
@@ -426,8 +533,12 @@ ags_note_set_property(GObject *gobject,
 
       attack = (AgsComplex *) g_value_get_boxed(value);
 
+      pthread_mutex_lock(note_mutex);
+
       ags_complex_set(&(note->attack),
 		      ags_complex_get(attack));
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_SUSTAIN:
@@ -436,8 +547,12 @@ ags_note_set_property(GObject *gobject,
 
       sustain = (AgsComplex *) g_value_get_boxed(value);
 
+      pthread_mutex_lock(note_mutex);
+
       ags_complex_set(&(note->sustain),
 		      ags_complex_get(sustain));
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_DECAY:
@@ -446,8 +561,12 @@ ags_note_set_property(GObject *gobject,
 
       decay = (AgsComplex *) g_value_get_boxed(value);
 
+      pthread_mutex_lock(note_mutex);
+
       ags_complex_set(&(note->decay),
 		      ags_complex_get(decay));
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_RELEASE:
@@ -456,8 +575,12 @@ ags_note_set_property(GObject *gobject,
 
       release = (AgsComplex *) g_value_get_boxed(value);
 
+      pthread_mutex_lock(note_mutex);
+
       ags_complex_set(&(note->release),
 		      ags_complex_get(release));
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_RATIO:
@@ -466,8 +589,12 @@ ags_note_set_property(GObject *gobject,
 
       ratio = (AgsComplex *) g_value_get_boxed(value);
 
+      pthread_mutex_lock(note_mutex);
+
       ags_complex_set(&(note->ratio),
 		      ags_complex_get(ratio));
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_NOTE_NAME:
@@ -476,7 +603,11 @@ ags_note_set_property(GObject *gobject,
       
       note_name = g_value_get_string(value);
       
+      pthread_mutex_lock(note_mutex);
+
       if(note_name == note->note_name){
+	pthread_mutex_unlock(note_mutex);
+
 	return;
       }
 
@@ -485,6 +616,17 @@ ags_note_set_property(GObject *gobject,
       }
 
       note->note_name = g_strdup(note_name);
+
+      pthread_mutex_unlock(note_mutex);
+    }
+    break;
+  case PROP_FREQUENCY:
+    {
+      pthread_mutex_lock(note_mutex);
+
+      note->frequency = g_value_get_double(value);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   default:
@@ -501,62 +643,151 @@ ags_note_get_property(GObject *gobject,
 {
   AgsNote *note;
 
+  pthread_mutex_t *note_mutex;
+
   note = AGS_NOTE(gobject);
 
+  /* get note mutex */
+  pthread_mutex_lock(ags_note_get_class_mutex());
+  
+  note_mutex = note->obj_mutex;
+  
+  pthread_mutex_unlock(ags_note_get_class_mutex());
+
   switch(prop_id){
   case PROP_X0:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_uint(value, note->x[0]);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_X1:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_uint(value, note->x[1]);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_Y:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_uint(value, note->y);
+
+      pthread_mutex_unlock(note_mutex);
+    }
+    break;
+  case PROP_RT_OFFSET:
+    {
+      pthread_mutex_lock(note_mutex);
+
+      g_value_set_uint64(value, note->rt_offset);
+
+      pthread_mutex_unlock(note_mutex);
+    }
+    break;
+  case PROP_RT_ATTACK:
+    {
+      pthread_mutex_lock(note_mutex);
+
+      g_value_set_uint(value, note->rt_attack);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_STREAM_DELAY:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_double(value, note->stream_delay);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_STREAM_ATTACK:
     {
-      g_value_set_uint(value, note->stream_attack);
+      pthread_mutex_lock(note_mutex);
+
+      g_value_set_double(value, note->stream_attack);
+
+      pthread_mutex_unlock(note_mutex);
+    }
+    break;
+  case PROP_STREAM_FRAME_COUNT:
+    {
+      pthread_mutex_lock(note_mutex);
+
+      g_value_set_uint64(value, note->stream_frame_count);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_ATTACK:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_boxed(value, note->attack);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_SUSTAIN:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_boxed(value, note->sustain);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_DECAY:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_boxed(value, note->decay);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_RELEASE:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_boxed(value, note->release);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_RATIO:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_boxed(value, note->ratio);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   case PROP_NOTE_NAME:
     {
+      pthread_mutex_lock(note_mutex);
+
       g_value_set_string(value, note->note_name);
+
+      pthread_mutex_unlock(note_mutex);
+    }
+    break;
+  case PROP_FREQUENCY:
+    {
+      pthread_mutex_lock(note_mutex);
+
+      g_value_set_double(value, note->frequency);
+
+      pthread_mutex_unlock(note_mutex);
     }
     break;
   default:
@@ -572,6 +803,12 @@ ags_note_finalize(GObject *gobject)
 
   note = AGS_NOTE(gobject);
 
+  pthread_mutex_destroy(note->obj_mutex);
+  free(note->obj_mutex);
+
+  pthread_mutexattr_destroy(note->obj_mutexattr);
+  free(note->obj_mutexattr);
+
   /* note name */
   if(note->note_name != NULL){
     free(note->note_name);
@@ -581,6 +818,126 @@ ags_note_finalize(GObject *gobject)
   G_OBJECT_CLASS(ags_note_parent_class)->finalize(gobject);
 }
 
+/**
+ * ags_note_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_note_get_class_mutex()
+{
+  return(&ags_note_class_mutex);
+}
+
+/**
+ * ags_note_test_flags:
+ * @note: the #AgsNote
+ * @flags: the flags
+ * 
+ * Test @flags to be set on @note.
+ * 
+ * Returns: %TRUE if flags are set, else %FALSE
+ * 
+ * Since: 2.0.0
+ */
+gboolean
+ags_note_test_flags(AgsNote *note, guint flags)
+{
+  gboolean retval;
+  
+  pthread_mutex_t *note_mutex;
+
+  if(!AGS_IS_NOTE(note)){
+    return(FALSE);
+  }
+      
+  /* get note mutex */
+  pthread_mutex_lock(ags_note_get_class_mutex());
+  
+  note_mutex = note->obj_mutex;
+  
+  pthread_mutex_unlock(ags_note_get_class_mutex());
+
+  /* test */
+  pthread_mutex_lock(note_mutex);
+
+  retval = (flags & (note->flags)) ? TRUE: FALSE;
+  
+  pthread_mutex_unlock(note_mutex);
+
+  return(retval);
+}
+
+/**
+ * ags_note_set_flags:
+ * @note: the #AgsNote
+ * @flags: the flags
+ * 
+ * Set @flags on @note.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_note_set_flags(AgsNote *note, guint flags)
+{
+  pthread_mutex_t *note_mutex;
+
+  if(!AGS_IS_NOTE(note)){
+    return;
+  }
+      
+  /* get note mutex */
+  pthread_mutex_lock(ags_note_get_class_mutex());
+  
+  note_mutex = note->obj_mutex;
+  
+  pthread_mutex_unlock(ags_note_get_class_mutex());
+
+  /* set */
+  pthread_mutex_lock(note_mutex);
+
+  note->flags |= flags;
+  
+  pthread_mutex_unlock(note_mutex);
+}
+
+/**
+ * ags_note_unset_flags:
+ * @note: the #AgsNote
+ * @flags: the flags
+ * 
+ * Unset @flags on @note.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_note_unset_flags(AgsNote *note, guint flags)
+{
+  pthread_mutex_t *note_mutex;
+
+  if(!AGS_IS_NOTE(note)){
+    return;
+  }
+      
+  /* get note mutex */
+  pthread_mutex_lock(ags_note_get_class_mutex());
+  
+  note_mutex = note->obj_mutex;
+  
+  pthread_mutex_unlock(ags_note_get_class_mutex());
+
+  /* unset */
+  pthread_mutex_lock(note_mutex);
+
+  note->flags &= (~flags);
+  
+  pthread_mutex_unlock(note_mutex);
+}
+
 /**
  * ags_note_sort_func:
  * @a: an #AgsNote
@@ -590,29 +947,42 @@ ags_note_finalize(GObject *gobject)
  * 
  * Returns: 0 if equal, -1 if smaller and 1 if bigger offset
  *
- * Since: 0.9.0
+ * Since: 2.0.0
  */
 gint
 ags_note_sort_func(gconstpointer a,
 		   gconstpointer b)
 {
+  guint a_x0, b_x0;
+  guint a_y, b_y;
+  
   if(a == NULL || b == NULL){
     return(0);
   }
+
+  g_object_get(a,
+	       "x0", &a_x0,
+	       "y", &a_y,
+	       NULL);
     
-  if(AGS_NOTE(a)->x[0] == AGS_NOTE(b)->x[0]){
-    if(AGS_NOTE(a)->y == AGS_NOTE(b)->y){
+  g_object_get(b,
+	       "x0", &b_x0,
+	       "y", &b_y,
+	       NULL);
+
+  if(a_x0 == b_x0){
+    if(a_y == b_y){
       return(0);
     }
 
-    if(AGS_NOTE(a)->y < AGS_NOTE(b)->y){
+    if(a_y < b_y){
       return(-1);
     }else{
       return(1);
     }
   }
 
-  if(AGS_NOTE(a)->x[0] < AGS_NOTE(b)->x[0]){
+  if(a_x0 < b_x0){
     return(-1);
   }else{
     return(1);
@@ -621,15 +991,15 @@ ags_note_sort_func(gconstpointer a,
 
 /**
  * ags_note_find_prev:
- * @note: a #GList containing #AgsNote
+ * @note: the #GList-struct containing #AgsNote
  * @x0: x offset
  * @y:  y offset
  * 
  * Find prev note having the same y offset.
  *
- * Returns: the matching entry as #GList if first entry's x offset bigger than @x0, else %NULL
+ * Returns: the matching entry as #GList-struct if first entry's x offset bigger than @x0, else %NULL
  *
- * Since: 0.7.2
+ * Since: 2.0.0
  */
 GList*
 ags_note_find_prev(GList *note,
@@ -637,22 +1007,40 @@ ags_note_find_prev(GList *note,
 {
   GList *current_match;
 
-  if(note == NULL ||
-     AGS_NOTE(note->data)->x[0] > x0){
+  guint current_x0, current_y;
+  
+  if(note == NULL){
+    return(NULL);
+  }
+
+  g_object_get(note->data,
+	       "x0", &current_x0,
+	       NULL);
+  
+  if(current_x0 > x0){
     return(NULL);
   }
 
   current_match = NULL;
   
   while(note != NULL){
-    if(AGS_NOTE(note->data)->y == y){
+    g_object_get(note->data,
+		 "y", &current_y,
+		 NULL);
+    
+    if(current_y == y){
       current_match = note;
 
       note = note->next;
+      
       continue;
     }
 
-    if(AGS_NOTE(note->data)->x[0] > x0){
+    g_object_get(note->data,
+		 "x0", &current_x0,
+		 NULL);
+
+    if(current_x0 > x0){
       return(current_match);
     }
 
@@ -664,15 +1052,15 @@ ags_note_find_prev(GList *note,
 
 /**
  * ags_note_find_next:
- * @note: a #GList containing #AgsNote
+ * @note: the #GList-struct containing #AgsNote
  * @x0: x offset
  * @y:  y offset
  * 
  * Find next note having the same y offset.
  *
- * Returns: the matching entry as #GList if last entry's x offset smaller than @x0, else %NULL
+ * Returns: the matching entry as #GList-struct if last entry's x offset smaller than @x0, else %NULL
  *
- * Since: 0.7.2
+ * Since: 2.0.0
  */
 GList*
 ags_note_find_next(GList *note,
@@ -680,24 +1068,40 @@ ags_note_find_next(GList *note,
 {
   GList *current_match;
 
-  note = g_list_last(note);
+  guint current_x0, current_y;
 
-  if(note == NULL ||
-     AGS_NOTE(note->data)->x[0] < x0){
+  if(note == NULL){
+    return(NULL);
+  }
+  
+  g_object_get(note->data,
+	       "x0", &current_x0,
+	       NULL);
+  
+  if(current_x0 < x0){
     return(NULL);
   }
 
   current_match = NULL;
 
   while(note != NULL){
-    if(AGS_NOTE(note->data)->y == y){
+    g_object_get(note->data,
+		 "y", &current_y,
+		 NULL);
+    
+    if(current_y == y){
       current_match = note;
 
       note = note->prev;
+      
       continue;
     }
 
-    if(AGS_NOTE(note->data)->x[0] < x0){
+    g_object_get(note->data,
+		 "x0", &current_x0,
+		 NULL);
+
+    if(current_x0 < x0){
       return(current_match);
     }
 
@@ -722,7 +1126,7 @@ ags_note_find_next(GList *note,
  * 
  * Returns: the delta-time
  * 
- * Since: 0.9.0
+ * Since: 2.0.0
  */
 glong
 ags_note_length_to_smf_delta_time(guint note_length,
@@ -750,7 +1154,7 @@ ags_note_length_to_smf_delta_time(guint note_length,
  * 
  * Returns: the note length
  * 
- * Since: 0.9.0
+ * Since: 2.0.0
  */
 guint
 ags_note_smf_delta_time_to_length(glong delta_time,
@@ -775,7 +1179,7 @@ ags_note_smf_delta_time_to_length(glong delta_time,
  *
  * Returns: The sequencer raw midi as array.
  *
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 unsigned char*
 ags_note_to_raw_midi(AgsNote *note,
@@ -941,7 +1345,7 @@ ags_note_to_raw_midi(AgsNote *note,
  * 
  * Returns: the raw-midi buffer
  * 
- * Since: 0.9.0
+ * Since: 2.0.0
  */
 unsigned char*
 ags_note_to_raw_midi_extended(AgsNote *note,
@@ -967,7 +1371,7 @@ ags_note_to_raw_midi_extended(AgsNote *note,
  *
  * Returns: The sequencer events as array.
  *
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 snd_seq_event_t*
 ags_note_to_seq_event(AgsNote *note,
@@ -999,7 +1403,7 @@ ags_note_to_seq_event(AgsNote *note,
  * 
  * Returns: an array of snd_seq_event_t structs
  * 
- * Since: 0.9.0
+ * Since: 2.0.0
  */
 snd_seq_event_t*
 ags_note_to_seq_event_extended(AgsNote *note,
@@ -1026,9 +1430,9 @@ ags_note_to_seq_event_extended(AgsNote *note,
  *
  * Parse @raw_midi data and convert to #AgsNote.
  *
- * Returns: a #GList containing the notes
+ * Returns: a #GList-struct containing the notes
  *
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 GList*
 ags_note_from_raw_midi(unsigned char *raw_midi,
@@ -1058,9 +1462,9 @@ ags_note_from_raw_midi(unsigned char *raw_midi,
  *
  * Parse @raw_midi data and convert to #AgsNote.
  *
- * Returns: a #GList containing the notes
+ * Returns: a #GList-struct containing the notes
  *
- * Since: 0.9.0
+ * Since: 2.0.0
  */
 GList*
 ags_note_from_raw_midi_extended(unsigned char *raw_midi,
@@ -1087,9 +1491,9 @@ ags_note_from_raw_midi_extended(unsigned char *raw_midi,
  *
  * Convert ALSA sequencer data @event to #AgsNote.
  *
- * Returns: a #GList containing the notes
+ * Returns: a #GList-struct containing the notes
  *
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 GList*
 ags_note_from_seq_event(snd_seq_event_t *event,
@@ -1106,7 +1510,7 @@ ags_note_from_seq_event(snd_seq_event_t *event,
 }
 
 /**
- * ags_note_from_raw_midi:
+ * ags_note_from_seq_event_extended:
  * @event: the snd_seq_event_t struct array
  * @nn: numerator
  * @dd: denominator
@@ -1119,9 +1523,9 @@ ags_note_from_seq_event(snd_seq_event_t *event,
  *
  * Parse @raw_midi data and convert to #AgsNote.
  *
- * Returns: a #GList containing the notes
+ * Returns: a #GList-struct containing the notes
  *
- * Since: 0.9.0
+ * Since: 2.0.0
  */
 GList*
 ags_note_from_seq_event_extended(snd_seq_event_t *event,
@@ -1147,81 +1551,101 @@ ags_note_from_seq_event_extended(snd_seq_event_t *event,
  *
  * Returns: the duplicated #AgsNote.
  *
- * Since: 0.4
+ * Since: 2.0.0
  */
 AgsNote*
 ags_note_duplicate(AgsNote *note)
 {
-  AgsNote *copy;
+  AgsNote *note_copy;
 
-  if(note->x[0] == note->x[1]){
+  pthread_mutex_t *note_mutex;
+
+  if(!AGS_IS_NOTE(note)){
     return(NULL);
   }
   
-  copy = ags_note_new();
+  /* get note mutex */
+  pthread_mutex_lock(ags_note_get_class_mutex());
+  
+  note_mutex = note->obj_mutex;
+  
+  pthread_mutex_unlock(ags_note_get_class_mutex());
 
-  copy->flags = 0;
+  /* instantiate note */  
+  note_copy = ags_note_new();
 
-  copy->note_name = g_strdup(note->note_name);
+  note_copy->flags = 0;
+
+  pthread_mutex_lock(note_mutex);
+
+  note_copy->note_name = g_strdup(note->note_name);
   
   if(note->x[0] < note->x[1]){
-    copy->x[0] = note->x[0];
-    copy->x[1] = note->x[1];
+    note_copy->x[0] = note->x[0];
+    note_copy->x[1] = note->x[1];
   }else{
-    copy->x[0] = note->x[1];
-    copy->x[1] = note->x[0];
+    note_copy->x[0] = note->x[1];
+    note_copy->x[1] = note->x[0];
   }
   
-  copy->y = note->y;
+  note_copy->y = note->y;
 
-  copy->stream_delay = note->stream_delay;
-  copy->stream_attack = note->stream_attack;
+  note_copy->stream_delay = note->stream_delay;
+  note_copy->stream_attack = note->stream_attack;
   
-  copy->attack[0] = note->attack[0];
-  copy->attack[1] = note->attack[1];
+  note_copy->attack[0] = note->attack[0];
+  note_copy->attack[1] = note->attack[1];
 
-  copy->decay[0] = note->decay[0];
-  copy->decay[1] = note->decay[1];
+  note_copy->decay[0] = note->decay[0];
+  note_copy->decay[1] = note->decay[1];
 
-  copy->sustain[0] = note->sustain[0];
-  copy->sustain[1] = note->sustain[1];
+  note_copy->sustain[0] = note->sustain[0];
+  note_copy->sustain[1] = note->sustain[1];
 
-  copy->release[0] = note->release[0];
-  copy->release[1] = note->release[1];
+  note_copy->release[0] = note->release[0];
+  note_copy->release[1] = note->release[1];
 
-  copy->ratio[0] = note->ratio[0];
-  copy->ratio[1] = note->ratio[1];
+  note_copy->ratio[0] = note->ratio[0];
+  note_copy->ratio[1] = note->ratio[1];
 
-  return(copy);
+  pthread_mutex_unlock(note_mutex);
+
+  return(note_copy);
 }
 
 /**
  * ags_note_new:
  *
- * Creates an #AgsNote
+ * Creates a new instance of #AgsNote
  *
- * Returns: a new #AgsNote
+ * Returns: the new #AgsNote
  *
- * Since: 0.4
+ * Since: 2.0.0
  */
 AgsNote*
 ags_note_new()
 {
   AgsNote *note;
 
-  note = (AgsNote *) g_object_new(AGS_TYPE_NOTE, NULL);
+  note = (AgsNote *) g_object_new(AGS_TYPE_NOTE,
+				  NULL);
 
   return(note);
 }
 
 /**
- * ags_note_new:
+ * ags_note_new_with_offset:
+ * @x0: x0
+ * @x1: x1
+ * @y: y
+ * @stream_delay: delay
+ * @stream_attack: attack
  *
- * Creates an #AgsNote
+ * Creates a new instance of #AgsNote
  *
- * Returns: a new #AgsNote
+ * Returns: the new #AgsNote
  *
- * Since: 0.7.2
+ * Since: 2.0.0
  */
 AgsNote*
 ags_note_new_with_offset(guint x0, guint x1,
@@ -1240,4 +1664,3 @@ ags_note_new_with_offset(guint x0, guint x1,
 
   return(note);
 }
-
