@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -22,6 +22,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <pthread.h>
+
 #include <ags/i18n.h>
 
 void ags_preset_class_init(AgsPresetClass *preset);
@@ -48,6 +50,8 @@ void ags_preset_finalize(GObject *gobject);
 
 static gpointer ags_preset_parent_class = NULL;
 
+static pthread_mutex_t ags_preset_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 enum{
   PROP_0,
   PROP_AUDIO,
@@ -110,7 +114,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The #AgsAudio belonging to.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("audio",
 				   i18n_pspec("audio"),
@@ -126,7 +130,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The preset's scope.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("scope",
 				   i18n_pspec("scope"),
@@ -142,7 +146,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The preset name.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("preset-name",
 				   i18n_pspec("preset-name"),
@@ -158,7 +162,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The start audio channel to apply.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("audio-channel-start",
 				 i18n_pspec("audio-channel-start"),
@@ -176,7 +180,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The end audio channel to apply.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("audio-channel-end",
 				 i18n_pspec("audio-channel-end"),
@@ -194,7 +198,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The start pad to apply.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("pad-start",
 				 i18n_pspec("pad-start"),
@@ -212,7 +216,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The end pad to apply.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("pad-end",
 				 i18n_pspec("pad-end"),
@@ -230,7 +234,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The start x to apply.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("x-start",
 				 i18n_pspec("x-start"),
@@ -248,7 +252,7 @@ ags_preset_class_init(AgsPresetClass *preset)
    *
    * The end x to apply.
    * 
-   * Since: 0.8.5
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("x-end",
 				 i18n_pspec("x-end"),
@@ -271,13 +275,35 @@ ags_preset_error_quark()
 void
 ags_preset_init(AgsPreset *preset)
 {
+  pthread_mutex_t *mutex;
+  pthread_mutexattr_t *attr;
+
   preset->flags = 0;
 
+  /* add preset mutex */
+  preset->obj_mutexattr = 
+    attr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+  pthread_mutexattr_init(attr);
+  pthread_mutexattr_settype(attr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+#ifdef __linux__
+  pthread_mutexattr_setprotocol(attr,
+				PTHREAD_PRIO_INHERIT);
+#endif
+
+  preset->obj_mutex = 
+    mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(mutex,
+		     attr);  
+
+  /* common fields */
   preset->audio = NULL;
   
   preset->scope = NULL;
   preset->preset_name = NULL;
 
+  /* insets */
   preset->audio_channel_start = 0;
   preset->audio_channel_end = 0;
 
@@ -290,8 +316,10 @@ ags_preset_init(AgsPreset *preset)
   preset->x_start = 0;
   preset->x_end = 0;
 
-  preset->parameter = NULL;
+  /* preset value */
   preset->n_params = 0;
+  preset->parameter_name = NULL;
+  preset->value = NULL;
 }
 
 void
@@ -302,8 +330,17 @@ ags_preset_set_property(GObject *gobject,
 {
   AgsPreset *preset;
 
+  pthread_mutex_t *preset_mutex;
+
   preset = AGS_PRESET(gobject);
 
+  /* get preset mutex */
+  pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+  preset_mutex = preset->obj_mutex;
+  
+  pthread_mutex_unlock(ags_preset_get_class_mutex());
+
   switch(prop_id){
   case PROP_AUDIO:
     {
@@ -311,7 +348,11 @@ ags_preset_set_property(GObject *gobject,
 
       audio = (GObject *) g_value_get_object(value);
 
+      pthread_mutex_lock(preset_mutex);
+
       if(preset->audio == audio){
+	pthread_mutex_unlock(preset_mutex);
+
 	return;
       }
 
@@ -324,6 +365,8 @@ ags_preset_set_property(GObject *gobject,
       }
 
       preset->audio = audio;
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_SCOPE:
@@ -332,11 +375,15 @@ ags_preset_set_property(GObject *gobject,
       
       scope = g_value_get_string(value);
 
+      pthread_mutex_lock(preset_mutex);
+
       if(preset->scope != NULL){
 	g_free(preset->scope);
       }
       
       preset->scope = g_strdup(scope);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_PRESET_NAME:
@@ -345,41 +392,69 @@ ags_preset_set_property(GObject *gobject,
       
       preset_name = g_value_get_string(value);
 
+      pthread_mutex_lock(preset_mutex);
+
       if(preset->preset_name != NULL){
 	g_free(preset->preset_name);
       }
       
       preset->preset_name = g_strdup(preset_name);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_AUDIO_CHANNEL_START:
     {
+      pthread_mutex_lock(preset_mutex);
+
       preset->audio_channel_start = g_value_get_uint(value);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_AUDIO_CHANNEL_END:
     {
+      pthread_mutex_lock(preset_mutex);
+
       preset->audio_channel_end = g_value_get_uint(value);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_PAD_START:
     {
+      pthread_mutex_lock(preset_mutex);
+
       preset->pad_start = g_value_get_uint(value);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_PAD_END:
     {
+      pthread_mutex_lock(preset_mutex);
+
       preset->pad_end = g_value_get_uint(value);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_X_START:
     {
+      pthread_mutex_lock(preset_mutex);
+
       preset->x_start = g_value_get_uint(value);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_X_END:
     {
+      pthread_mutex_lock(preset_mutex);
+
       preset->x_end = g_value_get_uint(value);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   default:
@@ -395,60 +470,105 @@ ags_preset_get_property(GObject *gobject,
 {
   AgsPreset *preset;
 
+  pthread_mutex_t *preset_mutex;
+
   preset = AGS_PRESET(gobject);
 
+  /* get preset mutex */
+  pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+  preset_mutex = preset->obj_mutex;
+  
+  pthread_mutex_unlock(ags_preset_get_class_mutex());
+
   switch(prop_id){
   case PROP_AUDIO:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_object(value, preset->audio);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_SCOPE:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_string(value,
 			 preset->scope);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_PRESET_NAME:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_string(value,
 			 preset->preset_name);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_AUDIO_CHANNEL_START:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_uint(value,
 		       preset->audio_channel_start);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_AUDIO_CHANNEL_END:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_uint(value,
 		       preset->audio_channel_end);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_PAD_START:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_uint(value,
 		       preset->pad_start);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_PAD_END:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_uint(value,
 		       preset->pad_end);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_X_START:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_uint(value,
 		       preset->x_start);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   case PROP_X_END:
     {
+      pthread_mutex_lock(preset_mutex);
+
       g_value_set_uint(value,
 		       preset->x_end);
+
+      pthread_mutex_unlock(preset_mutex);
     }
     break;
   default:
@@ -461,8 +581,16 @@ ags_preset_finalize(GObject *gobject)
 {
   AgsPreset *preset;
 
+  guint i;
+  
   preset = AGS_PRESET(gobject);
 
+  pthread_mutex_destroy(preset->obj_mutex);
+  free(preset->obj_mutex);
+
+  pthread_mutexattr_destroy(preset->obj_mutexattr);
+  free(preset->obj_mutexattr);
+
   if(preset->audio != NULL){
     g_object_unref(preset->audio);
   }
@@ -470,10 +598,140 @@ ags_preset_finalize(GObject *gobject)
   g_free(preset->scope);
   g_free(preset->preset_name);
 
+  /* paramenter name */
+  g_strfreev(preset->parameter_name);
+  
+  /* unset value */
+  for(i = 0; i < preset->n_params; i++){
+    g_value_unset(&(preset->value[i]));
+  }
+
+  g_free(preset->value);
+  
   /* call parent */
   G_OBJECT_CLASS(ags_preset_parent_class)->finalize(gobject);
 }
 
+/**
+ * ags_preset_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_preset_get_class_mutex()
+{
+  return(&ags_preset_class_mutex);
+}
+
+/**
+ * ags_preset_test_flags:
+ * @preset: the #AgsPreset
+ * @flags: the flags
+ *
+ * Test @flags to be set on @preset.
+ * 
+ * Returns: %TRUE if flags are set, else %FALSE
+ *
+ * Since: 2.0.0
+ */
+gboolean
+ags_preset_test_flags(AgsPreset *preset, guint flags)
+{
+  gboolean retval;  
+  
+  pthread_mutex_t *preset_mutex;
+
+  if(!AGS_IS_PRESET(preset)){
+    return(FALSE);
+  }
+
+  /* get preset mutex */
+  pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+  preset_mutex = preset->obj_mutex;
+  
+  pthread_mutex_unlock(ags_preset_get_class_mutex());
+
+  /* test */
+  pthread_mutex_lock(preset_mutex);
+
+  retval = (flags & (preset->flags)) ? TRUE: FALSE;
+  
+  pthread_mutex_unlock(preset_mutex);
+
+  return(retval);
+}
+
+/**
+ * ags_preset_set_flags:
+ * @preset: the #AgsPreset
+ * @flags: the flags
+ *
+ * Set flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_preset_set_flags(AgsPreset *preset, guint flags)
+{
+  pthread_mutex_t *preset_mutex;
+
+  if(!AGS_IS_PRESET(preset)){
+    return;
+  }
+
+  /* get preset mutex */
+  pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+  preset_mutex = preset->obj_mutex;
+  
+  pthread_mutex_unlock(ags_preset_get_class_mutex());
+
+  /* set flags */
+  pthread_mutex_lock(preset_mutex);
+
+  preset->flags |= flags;
+
+  pthread_mutex_unlock(preset_mutex);
+}
+
+/**
+ * ags_preset_unset_flags:
+ * @preset: the #AgsPreset
+ * @flags: the flags
+ *
+ * Unset flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_preset_unset_flags(AgsPreset *preset, guint flags)
+{
+  pthread_mutex_t *preset_mutex;
+
+  if(!AGS_IS_PRESET(preset)){
+    return;
+  }
+
+  /* get preset mutex */
+  pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+  preset_mutex = preset->obj_mutex;
+  
+  pthread_mutex_unlock(ags_preset_get_class_mutex());
+
+  /* set flags */
+  pthread_mutex_lock(preset_mutex);
+
+  preset->flags &= (~flags);
+
+  pthread_mutex_unlock(preset_mutex);
+}
+
 /**
  * ags_preset_find_scope:
  * @preset: the #GList-struct containing #AgsPreset
@@ -483,18 +741,35 @@ ags_preset_finalize(GObject *gobject)
  * 
  * Returns: the next matching #AgsPreset
  * 
- * Since: 0.8.5
+ * Since: 2.0.0
  */
 GList*
 ags_preset_find_scope(GList *preset,
 		      gchar *scope)
 {
+  pthread_mutex_t *preset_mutex;
+
   while(preset != NULL){
+    /* get preset mutex */
+    pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+    preset_mutex = AGS_PRESET(preset->data)->obj_mutex;
+  
+    pthread_mutex_unlock(ags_preset_get_class_mutex());
+
+    /* compare scope */
+    pthread_mutex_lock(preset_mutex);
+    
     if(!g_strcmp0(AGS_PRESET(preset->data)->scope,
 		  scope)){
+      pthread_mutex_unlock(preset_mutex);
+      
       return(preset);
     }
 
+    pthread_mutex_unlock(preset_mutex);
+
+    /* iterate */
     preset = preset->next;
   }
   
@@ -510,18 +785,35 @@ ags_preset_find_scope(GList *preset,
  * 
  * Returns: the next matching #AgsPreset
  * 
- * Since: 0.8.5
+ * Since: 2.0.0
  */
 GList*
 ags_preset_find_name(GList *preset,
 		     gchar *preset_name)
 {
+  pthread_mutex_t *preset_mutex;
+
   while(preset != NULL){
+    /* get preset mutex */
+    pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+    preset_mutex = AGS_PRESET(preset->data)->obj_mutex;
+  
+    pthread_mutex_unlock(ags_preset_get_class_mutex());
+
+    /* compare scope */
+    pthread_mutex_lock(preset_mutex);
+    
     if(!g_strcmp0(AGS_PRESET(preset->data)->preset_name,
 		  preset_name)){
+      pthread_mutex_unlock(preset_mutex);
+      
       return(preset);
     }
 
+    pthread_mutex_unlock(preset_mutex);
+
+    /* iterate */
     preset = preset->next;
   }
   
@@ -536,7 +828,7 @@ ags_preset_find_name(GList *preset,
  *
  * Add parameter to @preset.
  *
- * Since: 0.8.5
+ * Since: 2.0.0
  */
 gboolean
 ags_preset_add_parameter(AgsPreset *preset,
@@ -546,27 +838,40 @@ ags_preset_add_parameter(AgsPreset *preset,
   guint i;
   gboolean found;
   
+  pthread_mutex_t *preset_mutex;
+
   if(!AGS_IS_PRESET(preset)){
     return(FALSE);
   }
 
+  /* get preset mutex */
+  pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+  preset_mutex = preset->obj_mutex;
+  
+  pthread_mutex_unlock(ags_preset_get_class_mutex());
+
   /* match or allocate */
+  pthread_mutex_lock(preset_mutex);
+
   found = FALSE;
 
-  if(preset->parameter == NULL){
-    preset->parameter = (GParameter *) malloc(sizeof(GParameter));
+  if(preset->parameter_name == NULL){
+    preset->parameter_name = (gchar **) malloc(2 * sizeof(gchar *));
 
-    preset->parameter[0].name = NULL;
-    memset(&(preset->parameter[0].value), 0, sizeof(GValue));
+    preset->parameter_name[0] = NULL;
+    preset->parameter_name[1] = NULL;
 
-    g_value_init(&(preset->parameter[0].value),
+    preset->value = g_new0(GValue,
+			   1);
+    g_value_init(&(preset->value[0]),
 		 G_VALUE_TYPE(value));
     
     preset->n_params += 1;
     nth = 0;
   }else{
     for(i = 0; i < preset->n_params; i++){
-      if(!g_strcmp0(preset->parameter[i].name,
+      if(!g_strcmp0(preset->parameter_name[i],
 		    param_name)){
 	nth = i;
 	found = TRUE;
@@ -576,13 +881,19 @@ ags_preset_add_parameter(AgsPreset *preset,
     }
 
     if(!found){
-      preset->parameter = (GParameter *) realloc(preset->parameter,
-						 (preset->n_params + 1) * sizeof(GParameter));
+      preset->parameter_name = (gchar **) realloc(preset->parameter_name,
+						  (preset->n_params + 2) * sizeof(gchar *));
 
-      preset->parameter[preset->n_params].name = NULL;
-      memset(&(preset->parameter[preset->n_params].value), 0, sizeof(GValue));
+      preset->parameter_name[preset->n_params] = NULL;
+      preset->parameter_name[preset->n_params + 1] = NULL;
 
-      g_value_init(&(preset->parameter[preset->n_params].value),
+      preset->value = g_renew(GValue,
+			      preset->value,
+			      preset->n_params + 1);
+      
+      memset(&(preset->value[preset->n_params]), 0, sizeof(GValue));
+
+      g_value_init(&(preset->value[preset->n_params]),
 		   G_VALUE_TYPE(value));
       
       preset->n_params += 1;
@@ -591,13 +902,13 @@ ags_preset_add_parameter(AgsPreset *preset,
   }
 
   /* set value */  
-  if(preset->parameter[nth].name != NULL){
-    g_free(preset->parameter[nth].name);
-  }
+  g_free(preset->parameter_name[nth]);
 
-  preset->parameter[nth].name = g_strdup(param_name);
+  preset->parameter_name[nth] = g_strdup(param_name);
   g_value_copy(value,
-	       &(preset->parameter[nth].value));
+	       &(preset->value[nth]));
+
+  pthread_mutex_unlock(preset_mutex);
 
   return(!found);
 }
@@ -609,45 +920,88 @@ ags_preset_add_parameter(AgsPreset *preset,
  * 
  * Remove parameter of @preset.
  * 
- * Since: 0.8.5
+ * Since: 2.0.0
  */
 void
 ags_preset_remove_parameter(AgsPreset *preset,
 			    guint nth)
 {
-  GParameter *parameter;
+  GValue *value;
 
-  if(!AGS_IS_PRESET(preset) ||
-     preset->n_params == 0 ||
-     nth >= preset->n_params){
+  gchar **parameter_name;
+
+  guint i;
+  
+  pthread_mutex_t *preset_mutex;
+
+  if(!AGS_IS_PRESET(preset)){
     return;
   }
 
-  parameter = preset->parameter;
+  /* get preset mutex */
+  pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+  preset_mutex = preset->obj_mutex;
+  
+  pthread_mutex_unlock(ags_preset_get_class_mutex());
 
-  if(preset->n_params == 1){
-    preset->parameter = NULL;
+  /* check boundaries */
+  pthread_mutex_lock(preset_mutex);
+  
+  if(preset->n_params == 0 ||
+     nth >= preset->n_params){
+    pthread_mutex_unlock(preset_mutex);
 
+    return;
+  }
+  
+  parameter_name = preset->parameter_name;
+  value = preset->value;
+
+  if(preset->n_params == 1){    
+    preset->parameter_name = NULL;
     preset->n_params = 0;
   }else{
-    preset->parameter = (GParameter *) malloc((preset->n_params - 1) * sizeof(GParameter));
-
+    preset->parameter_name = (gchar **) malloc((preset->n_params) * sizeof(gchar *));
+    preset->value = g_new(GValue,
+			  preset->n_params - 1);
+    
     if(nth > 0){
-      memcpy(preset->parameter,
-	     parameter,
-	     nth * sizeof(GParameter));
+      memcpy(preset->parameter_name,
+	     parameter_name,
+	     nth * sizeof(gchar *));
+
+      memcpy(preset->value,
+	     value,
+	     nth * sizeof(GValue));
     }
 
     if(nth + 1 < preset->n_params){
-      memcpy(&(preset->parameter[nth]),
-	     &(parameter[nth + 1]),
-	     (preset->n_params - (nth + 1)) * sizeof(GParameter));    
+      memcpy(&(preset->parameter_name[nth]),
+	     &(parameter_name[nth + 1]),
+	     (preset->n_params - (nth + 1)) * sizeof(gchar *));    
+
+      memcpy(&(preset->value[nth]),
+	     &(value[nth + 1]),
+	     (preset->n_params - (nth + 1)) * sizeof(GValue));
     }
 
     preset->n_params -= 1;
+
+    preset->parameter_name[preset->n_params] = NULL;
   }
-    
-  free(parameter);
+
+  /* free old preset array */
+  g_free(parameter_name[nth]);
+  g_free(parameter_name);
+
+  for(i = 0; i < preset->n_params + 1; i++){
+    g_value_unset(&(value[i]));
+  }
+  
+  g_free(value);
+  
+  pthread_mutex_unlock(preset_mutex);
 }
 
 /**
@@ -660,7 +1014,7 @@ ags_preset_remove_parameter(AgsPreset *preset,
  * Get parameter specified by @param_name. If parameter not available
  * the @error is set to indicate the failure.
  *
- * Since: 0.8.5
+ * Since: 2.0.0
  */
 void
 ags_preset_get_parameter(AgsPreset *preset,
@@ -668,34 +1022,44 @@ ags_preset_get_parameter(AgsPreset *preset,
 			 GError **error)
 {
   guint i;
+  gboolean success;
+  
+  pthread_mutex_t *preset_mutex;
 
   if(!AGS_IS_PRESET(preset)){
     return;
   }
 
-  if(preset->parameter == NULL){
-    if(error != NULL){
-      g_set_error(error,
-		  AGS_PRESET_ERROR,
-		  AGS_PRESET_ERROR_NO_SUCH_PARAMETER,
-		  "unable to find parameter: %s",
-		  param_name);
-    }
+  /* get preset mutex */
+  pthread_mutex_lock(ags_preset_get_class_mutex());
+  
+  preset_mutex = preset->obj_mutex;
+  
+  pthread_mutex_unlock(ags_preset_get_class_mutex());
 
-    return;
-  }
+  /* find */
+  pthread_mutex_lock(preset_mutex);
+
+  success = FALSE;
   
-  for(i = 0; i < preset->n_params; i++){
-    if(!g_strcmp0(preset->parameter[i].name,
-		  param_name)){
-      g_value_copy(&(preset->parameter[i].value),
-		   value);
-      
-      return;
+  if(preset->parameter_name != NULL){
+    for(i = 0; i < preset->n_params; i++){
+      if(!g_strcmp0(preset->parameter_name[i],
+		    param_name)){
+	g_value_copy(&(preset->value[i]),
+		     value);
+
+	success = TRUE;      
+
+	break;
+      }
     }
   }
+
+  pthread_mutex_unlock(preset_mutex);
   
-  if(error != NULL){
+  /* report error */
+  if(!success && error != NULL){
     g_set_error(error,
 		AGS_PRESET_ERROR,
 		AGS_PRESET_ERROR_NO_SUCH_PARAMETER,
@@ -707,11 +1071,11 @@ ags_preset_get_parameter(AgsPreset *preset,
 /**
  * ags_preset_new:
  *
- * Creates an #AgsPreset
+ * Create a new instance of #AgsPreset
  *
- * Returns: a new #AgsPreset
+ * Returns: the new #AgsPreset
  *
- * Since: 0.8.5
+ * Since: 2.0.0
  */
 AgsPreset*
 ags_preset_new()
