@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,24 +19,10 @@
 
 #include <ags/audio/ags_synth_generator.h>
 
-#include <ags/util/ags_id_generator.h>
-
-#include <ags/object/ags_config.h>
-#include <ags/object/ags_plugin.h>
-#include <ags/object/ags_soundcard.h>
-
-#include <ags/thread/ags_timestamp.h>
-
-#include <ags/thread/file/ags_thread_file_xml.h>
-
-#include <ags/file/ags_file_stock.h>
-#include <ags/file/ags_file.h>
-#include <ags/file/ags_file_id_ref.h>
-#include <ags/file/ags_file_lookup.h>
-#include <ags/file/ags_file_launch.h>
-#include <ags/file/ags_file_link.h>
+#include <ags/libags.h>
 
 #include <ags/audio/ags_audio_signal.h>
+#include <ags/audio/ags_audio_buffer_util.h>
 #include <ags/audio/ags_synth_util.h>
 
 #include <math.h>
@@ -77,8 +63,9 @@ enum{
   PROP_SAMPLERATE,
   PROP_BUFFER_SIZE,
   PROP_FORMAT,
-  PROP_N_FRAMES,
+  PROP_DELAY,
   PROP_ATTACK,
+  PROP_FRAME_COUNT,
   PROP_OSCILLATOR,
   PROP_FREQUENCY,
   PROP_PHASE,
@@ -150,7 +137,7 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The samplerate to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("samplerate",
 				 i18n_pspec("using samplerate"),
@@ -168,7 +155,7 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The buffer size to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("buffer-size",
 				 i18n_pspec("using buffer size"),
@@ -186,7 +173,7 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The format to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("format",
 				 i18n_pspec("using format"),
@@ -200,21 +187,39 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
 				  param_spec);
 
   /**
-   * AgsSynthGenerator:n-frames:
+   * AgsSynthGenerator:frame-count:
    *
-   * The n-frames to be used.
+   * The frame count to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
-  param_spec = g_param_spec_uint("n-frames",
-				 i18n_pspec("apply n-frames"),
-				 i18n_pspec("To apply n-frames"),
+  param_spec = g_param_spec_uint("frame-count",
+				 i18n_pspec("apply frame count"),
+				 i18n_pspec("To apply frame count"),
 				 0,
 				 G_MAXUINT32,
 				 0,
 				 G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
-				  PROP_N_FRAMES,
+				  PROP_FRAME_COUNT,
+				  param_spec);
+
+  /**
+   * AgsAudioSignal:delay:
+   *
+   * The delay to be used.
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_double("delay",
+				   i18n_pspec("using delay"),
+				   i18n_pspec("The delay to be used"),
+				   0.0,
+				   65535.0,
+				   0.0,
+				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_DELAY,
 				  param_spec);
 
   /**
@@ -222,7 +227,7 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The attack to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("attack",
 				 i18n_pspec("apply attack"),
@@ -240,7 +245,7 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The oscillator to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("oscillator",
 				 i18n_pspec("using oscillator"),
@@ -258,7 +263,7 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The frequency to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_double("frequency",
 				   i18n_pspec("using frequency"),
@@ -276,7 +281,7 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The phase to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_double("phase",
 				   i18n_pspec("using phase"),
@@ -294,13 +299,13 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The volume to be used.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_double("volume",
 				   i18n_pspec("using volume"),
 				   i18n_pspec("The volume to be used"),
 				   0.0,
-				   65535.0,
+				   2.0,
 				   0.0,
 				   G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
@@ -312,7 +317,7 @@ ags_synth_generator_class_init(AgsSynthGeneratorClass *synth_generator)
    *
    * The assigned timestamp.
    * 
-   * Since: 0.7.45
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("timestamp",
 				   i18n_pspec("timestamp"),
@@ -337,78 +342,23 @@ void
 ags_synth_generator_init(AgsSynthGenerator *synth_generator)
 {
   AgsConfig *config;
-  
-  gchar *str;
-  
+
+  synth_generator->flags = 0;
+
   /* config */
   config = ags_config_get_instance();
 
-  /* base init */
-  synth_generator->flags = 0;
-
-  synth_generator->samplerate = AGS_SOUNDCARD_DEFAULT_SAMPLERATE;
-  synth_generator->buffer_size = AGS_SOUNDCARD_DEFAULT_BUFFER_SIZE;
-  synth_generator->format = AGS_SOUNDCARD_DEFAULT_FORMAT;
-
-  /* samplerate */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_SOUNDCARD,
-			     "samplerate");
-
-  if(str == NULL){
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_SOUNDCARD_0,
-			       "samplerate");
-  }
-  
-  if(str != NULL){
-    synth_generator->samplerate = g_ascii_strtoull(str,
-						   NULL,
-						   10);
-
-    free(str);
-  }
-
-  /* buffer size */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_SOUNDCARD,
-			     "buffer-size");
-
-  if(str == NULL){
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_SOUNDCARD_0,
-			       "buffer-size");
-  }
-  
-  if(str != NULL){
-    synth_generator->buffer_size = g_ascii_strtoull(str,
-						    NULL,
-						    10);
-
-    free(str);
-  }
-
-  /* format */
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_SOUNDCARD,
-			     "format");
-
-  if(str == NULL){
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_SOUNDCARD_0,
-			       "format");
-  }
-  
-  if(str != NULL){
-    synth_generator->format = g_ascii_strtoull(str,
-					       NULL,
-					       10);
-
-    free(str);
-  }
+  /* presets */
+  synth_generator->samplerate = ags_soundcard_helper_config_get_samplerate(config);
+  synth_generator->buffer_size = ags_soundcard_helper_config_get_buffer_size(config);
+  synth_generator->format = ags_soundcard_helper_config_get_format(config);
 
   /* more base init */
-  synth_generator->n_frames = 0;
+  synth_generator->frame_count = 0;
+  synth_generator->loop_start = 0;
+  synth_generator->loop_end = 0;
+
+  synth_generator->delay = 0.0;
   synth_generator->attack = 0;
 
   synth_generator->oscillator = 0;
@@ -416,6 +366,13 @@ ags_synth_generator_init(AgsSynthGenerator *synth_generator)
   synth_generator->frequency = 0.0;
   synth_generator->phase = 0.0;
   synth_generator->volume = 1.0;
+
+  synth_generator->sync_relative = FALSE;
+  synth_generator->sync_point = NULL;
+  synth_generator->sync_point_count = 0;
+
+  synth_generator->damping = ags_complex_alloc();
+  synth_generator->vibration = ags_complex_alloc();
   
   /* timestamp */
   synth_generator->timestamp = NULL;
@@ -447,9 +404,9 @@ ags_synth_generator_set_property(GObject *gobject,
       synth_generator->format = g_value_get_uint(value);
     }
     break;
-  case PROP_N_FRAMES:
+  case PROP_DELAY:
     {
-      synth_generator->n_frames = g_value_get_uint(value);
+      synth_generator->delay = g_value_get_double(value);
     }
     break;
   case PROP_ATTACK:
@@ -457,6 +414,11 @@ ags_synth_generator_set_property(GObject *gobject,
       synth_generator->attack = g_value_get_uint(value);
     }
     break;
+  case PROP_FRAME_COUNT:
+    {
+      synth_generator->frame_count = g_value_get_uint(value);
+    }
+    break;
   case PROP_OSCILLATOR:
     {
       synth_generator->oscillator = g_value_get_uint(value);
@@ -530,9 +492,9 @@ ags_synth_generator_get_property(GObject *gobject,
       g_value_set_uint(value, synth_generator->format);
     }
     break;
-  case PROP_N_FRAMES:
+  case PROP_DELAY:
     {
-      g_value_set_uint(value, synth_generator->n_frames);
+      g_value_set_uint(value, synth_generator->delay);
     }
     break;
   case PROP_ATTACK:
@@ -540,6 +502,11 @@ ags_synth_generator_get_property(GObject *gobject,
       g_value_set_uint(value, synth_generator->attack);
     }
     break;
+  case PROP_FRAME_COUNT:
+    {
+      g_value_set_uint(value, synth_generator->frame_count);
+    }
+    break;
   case PROP_OSCILLATOR:
     {
       g_value_set_uint(value, synth_generator->oscillator);
@@ -645,183 +612,108 @@ ags_synth_generator_finalize(GObject *gobject)
 /**
  * ags_synth_generator_compute:
  * @synth_generator: the #AgsSynthGenerator
+ * @audio_signal: the #AgsAudioSignal
  * @note: the note to compute
  * 
  * Compute synth for @note.
  *
  * Returns: an #AgsAudioSignal applied specified synth to stream
  * 
- * Since: 0.9.7
- */
-GObject*
-ags_synth_generator_compute(AgsSynthGenerator *synth_generator,
-			    gdouble note)
-{
-  AgsAudioSignal *audio_signal;
-
-  audio_signal = ags_audio_signal_new(NULL,
-				      NULL,
-				      NULL);
-  ags_audio_signal_stream_resize(audio_signal,
-				 ceil(synth_generator->n_frames / synth_generator->buffer_size));
-
-  ags_synth_generator_compute_with_audio_signal(synth_generator,
-						audio_signal,
-						note,
-						NULL, NULL,
-						AGS_SYNTH_GENERATOR_COMPUTE_FIXED_LENGTH);
-  
-  return(audio_signal);
-}
-
-/**
- * ags_synth_generator_compute_with_audio_signal:
- * @synth_generator: the #AgsSynthGenerator
- * @audio_signal: the #AgsAudioSignal
- * @note: the note to compute
- * @sync_start: not used for now
- * @sync_end: not used for now
- * @compute_flags: computation flags
- * 
- * Compute synth for @note for @audio_signal.
- * 
- * Since: 0.9.7
+ * Since: 2.0.0
  */
 void
-ags_synth_generator_compute_with_audio_signal(AgsSynthGenerator *synth_generator,
-					      GObject *audio_signal,
-					      gdouble note,
-					      AgsComplex *sync_start, AgsComplex *sync_end,
-					      guint compute_flags)
+ags_synth_generator_compute(AgsSynthGenerator *synth_generator,
+			    GObject *audio_signal,
+			    gdouble note)
 {
-  GList *stream;
-  
-  void *buffer;
-  
+  GList *stream_start, *stream;
+
+  gdouble delay;
+  guint attack;
+  guint frame_count;
+  guint buffer_size;
+  guint current_frame_count, requested_frame_count;
   gdouble samplerate;
-  gdouble start_frequency, frequency;
   gdouble current_frequency;
   gdouble phase, volume;
   gdouble current_phase;
   guint format;
   guint audio_buffer_util_format;
-  guint frame_count, stop_frame;
-  guint buffer_size;
+  guint current_attack, current_count;
   guint offset;
-  guint attack, frame_copy_count;
+  guint last_sync;
+  guint i, j;
+  gboolean sync_relative;
+  AgsComplex **sync_point;
+  guint sync_point_count;
+  gboolean synced;
 
-  gdouble current_rate;
-  gdouble xcross_factor;
-  guint xcross_count;
-  guint i;
-  gboolean initial_run;
+  delay = synth_generator->delay;
+  attack = synth_generator->attack;
 
+  frame_count = synth_generator->frame_count;
+  
+  buffer_size = AGS_AUDIO_SIGNAL(audio_signal)->buffer_size;
+
+  sync_relative = synth_generator->sync_relative;
+
+  sync_point = synth_generator->sync_point;
+  sync_point_count = synth_generator->sync_point_count;
+
+  current_frame_count = AGS_AUDIO_SIGNAL(audio_signal)->length * buffer_size;
+  requested_frame_count = (guint) ceil(((floor(delay) * buffer_size + attack) + frame_count) / buffer_size) * buffer_size;
+  
+  if(current_frame_count < requested_frame_count){
+    ags_audio_signal_stream_resize(audio_signal,
+				   ceil(requested_frame_count / buffer_size));
+  }
+  
+  /*  */
+  stream = 
+    stream_start = g_list_nth(AGS_AUDIO_SIGNAL(audio_signal)->stream,
+			      (guint) floor(delay));
+  
   samplerate = AGS_AUDIO_SIGNAL(audio_signal)->samplerate;
-  
-  start_frequency = 48.0;
-  frequency = synth_generator->frequency;
-  
-  current_frequency = (guint) ((double) frequency * exp2((double)((double) note + 48.0) / 12.0));
 
-  phase = synth_generator->phase;
+  current_frequency = (guint) ((double) synth_generator->frequency * exp2((double)((double) note + 48.0) / 12.0));
+
+  phase =
+    current_phase = synth_generator->phase;
   volume = synth_generator->volume;
 
   format = AGS_AUDIO_SIGNAL(audio_signal)->format;
   audio_buffer_util_format = ags_audio_buffer_util_format_from_soundcard(format);
 
-  frame_count = AGS_AUDIO_SIGNAL(audio_signal)->frame_count;
-  buffer_size = AGS_AUDIO_SIGNAL(audio_signal)->buffer_size;
+  current_attack = attack;
+  current_count = buffer_size - attack;
   
-  /* generate synth */
-  if((AGS_SYNTH_GENERATOR_COMPUTE_SYNC & compute_flags) != 0){
-    stream = AGS_AUDIO_SIGNAL(audio_signal)->stream_beginning;
-    xcross_count = 0;
-    
-    while(stream != NULL){
-      xcross_count += ags_synth_util_get_xcross_count(stream->data,
-						      audio_buffer_util_format,
-						      buffer_size);
-
-      stream = stream->next;
-    }
-  }
-
-  if(xcross_count == 0){
-    xcross_count = 1;
-  }
-
-  attack = synth_generator->attack;
-
-  if((AGS_SYNTH_GENERATOR_COMPUTE_FIXED_LENGTH & compute_flags) != 0){
-    stop_frame = synth_generator->n_frames;
-  }else{
-    stop_frame = frame_count;
-  }
-  
-  stream = AGS_AUDIO_SIGNAL(audio_signal)->stream_beginning;
-  stream = g_list_nth(stream,
-		      floor(attack / buffer_size));
-
-  attack %= buffer_size;
-
   offset = 0;
-  current_phase = phase;
+  last_sync = 0;
 
-  if((AGS_SYNTH_GENERATOR_COMPUTE_SYNC & compute_flags) != 0){
-    if(xcross_count == 0){
-      current_rate = current_frequency;
+  if(sync_point != NULL){
+    if(sync_point_count > 1 &&
+       floor(sync_point[1][0][0]) > 0.0){
+      if(sync_point[1][0][0] < current_count){
+	current_count = sync_point[1][0][0];
+      }
     }else{
-      current_rate = (frame_count / xcross_count) * (frame_count / samplerate);
-    }
-    
-    if((AGS_SYNTH_GENERATOR_COMPUTE_16HZ & compute_flags) != 0){
-      xcross_factor = current_rate * (16.0 / samplerate);
-    }else if((AGS_SYNTH_GENERATOR_COMPUTE_440HZ & compute_flags) != 0){
-      xcross_factor = current_rate * (440.0 / samplerate);
-    }else if((AGS_SYNTH_GENERATOR_COMPUTE_22000HZ & compute_flags) != 0){
-      xcross_factor = current_rate * (22000.0 / samplerate);
-    }else if((AGS_SYNTH_GENERATOR_COMPUTE_LIMIT & compute_flags) != 0){
-      xcross_factor = current_rate * samplerate;
-    }else if((AGS_SYNTH_GENERATOR_COMPUTE_NOHZ & compute_flags) != 0){
-      if(xcross_count != 0){
-	xcross_factor = (1.0 / xcross_count);
-      }else{
-	xcross_factor = 1.0;
+      if(sync_point[0][0][0] < current_count &&
+	 floor(sync_point[0][0][0]) > 0.0){
+	current_count = sync_point[0][0][0];
       }
-    }else if((AGS_SYNTH_GENERATOR_COMPUTE_FREQUENCY & compute_flags) != 0){
-      xcross_factor = current_rate * (frequency / samplerate);
-    }else if((AGS_SYNTH_GENERATOR_COMPUTE_NOTE & compute_flags) != 0){
-      xcross_factor = current_rate * (current_frequency / samplerate);
     }
   }
-
-  i = 0;
   
-  initial_run = TRUE;
+  synced = FALSE;
   
-  while(stream != NULL &&
-	offset < stop_frame){
-    if(initial_run){
-      if(buffer_size - attack < stop_frame){
-	frame_copy_count = buffer_size - attack;
-      }else{
-	frame_copy_count = stop_frame;
-      }
-    }else{
-      if(offset + buffer_size < stop_frame){
-	frame_copy_count = buffer_size;
-      }else{
-	frame_copy_count = stop_frame - offset;
-      }
-    }
-
+  for(i = attack, j = 0; i < frame_count + attack && stream != NULL;){
     switch(synth_generator->oscillator){
     case AGS_SYNTH_GENERATOR_OSCILLATOR_SIN:
       {
 	ags_synth_util_sin(stream->data,
 			   current_frequency, current_phase, volume,
 			   samplerate, audio_buffer_util_format,
-			   attack, frame_copy_count);
+			   current_attack, current_count);
       }
       break;
     case AGS_SYNTH_GENERATOR_OSCILLATOR_SAWTOOTH:
@@ -829,7 +721,7 @@ ags_synth_generator_compute_with_audio_signal(AgsSynthGenerator *synth_generator
 	ags_synth_util_sawtooth(stream->data,
 				current_frequency, current_phase, volume,
 				samplerate, audio_buffer_util_format,
-				attack, frame_copy_count);
+				current_attack, current_count);
       }
       break;
     case AGS_SYNTH_GENERATOR_OSCILLATOR_TRIANGLE:
@@ -837,7 +729,7 @@ ags_synth_generator_compute_with_audio_signal(AgsSynthGenerator *synth_generator
 	ags_synth_util_triangle(stream->data,
 				current_frequency, current_phase, volume,
 				samplerate, audio_buffer_util_format,
-				attack, frame_copy_count);
+				current_attack, current_count);
       }
       break;
     case AGS_SYNTH_GENERATOR_OSCILLATOR_SQUARE:
@@ -845,33 +737,68 @@ ags_synth_generator_compute_with_audio_signal(AgsSynthGenerator *synth_generator
 	ags_synth_util_square(stream->data,
 			      current_frequency, current_phase, volume,
 			      samplerate, audio_buffer_util_format,
-			      attack, frame_copy_count);
+			      current_attack, current_count);
       }
       break;
     default:
       g_message("unknown oscillator");
     }
+    
+    current_phase = (guint) ((offset + current_count) + phase) % (guint) floor(samplerate / current_frequency);
 
-    stream = stream->next;
-    offset += frame_copy_count;
+    if(sync_point != NULL){
+      if(floor(sync_point[j][0][0]) > 0.0 &&
+	 last_sync + sync_point[j][0][0] < offset + current_count){
+	current_phase = sync_point[j][0][1];
 
-    if((AGS_SYNTH_GENERATOR_COMPUTE_SYNC & compute_flags) == 0 ||
-       (xcross_factor * i) > (frame_count / xcross_count)){
-      current_phase = (guint) (offset - attack + phase) % (guint) floor(samplerate / current_frequency);
-
-      i = 0;
-    }else{
-      current_phase = phase;
-
-      i++;
+	synced = TRUE;
+      }
     }
     
-    attack = 0;
+    offset += current_count;
+    i += current_count;
 
-    initial_run = FALSE;
-  }
+    if(buffer_size > (current_attack + current_count)){
+      current_count = buffer_size - (current_attack + current_count);
+      current_attack = buffer_size - current_count;
+    }else{
+      current_count = buffer_size;
+      current_attack = 0;
+    }
+    
+    if(sync_point != NULL){
+      if(j + 1 < sync_point_count &&
+	 floor(sync_point[j + 1][0][0]) > 0.0){
+	if(sync_point[j + 1][0][0] < current_count){
+	  current_count = sync_point[j + 1][0][0];
+	}
+      }else{
+	if(floor(sync_point[0][0][0]) > 0.0 &&
+	   sync_point[0][0][0] < current_count){
+	  current_count = sync_point[0][0][0];
+	}
+      }
+    }
+    
+    if(sync_point != NULL){
+      if(synced){
+	last_sync = last_sync + sync_point[j][0][0];
+	j++;
 
-  free(buffer);
+	if(j >= sync_point_count ||
+	   floor(sync_point[j][0][0]) == 0.0){
+	  j = 0;
+	}
+
+	synced = FALSE;
+      }
+    }
+
+    if(i != 0 &&
+       i % buffer_size == 0){
+      stream = stream->next;
+    }
+  }  
 }
 
 /**
@@ -881,7 +808,7 @@ ags_synth_generator_compute_with_audio_signal(AgsSynthGenerator *synth_generator
  *
  * Returns: a new #AgsSynthGenerator
  *
- * Since: 0.7.45
+ * Since: 2.0.0
  */
 AgsSynthGenerator*
 ags_synth_generator_new()
