@@ -225,7 +225,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The file to parse data from.
    * 
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_pointer("file",
 				    i18n_pspec("the file stream"),
@@ -268,7 +268,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::midi-putc signal is emited during putting char to file.
    *
-   * Since: 0.7.90
+   * Since: 1.0.0
    */
   midi_builder_signals[MIDI_PUTC] =
     g_signal_new("midi-putc",
@@ -287,7 +287,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::on-error signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[ON_ERROR] =
     g_signal_new("on-error",
@@ -312,7 +312,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::append-header signal is emited during building of header.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[APPEND_HEADER] =
     g_signal_new("append-header",
@@ -320,7 +320,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_header),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 7,
 		 G_TYPE_UINT, G_TYPE_UINT,
 		 G_TYPE_UINT, G_TYPE_UINT,
@@ -336,7 +336,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::append-track signal is emited during building of track.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[APPEND_TRACK] =
     g_signal_new("append-track",
@@ -358,7 +358,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::key-on signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[KEY_ON] =
     g_signal_new("key-on",
@@ -366,7 +366,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_key_on),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -383,7 +383,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::key-off signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[KEY_OFF] =
     g_signal_new("key-off",
@@ -391,7 +391,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_key_off),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -408,7 +408,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::key-pressure signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[KEY_PRESSURE] =
     g_signal_new("key-pressure",
@@ -416,7 +416,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_key_pressure),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -433,7 +433,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::change-parameter signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[CHANGE_PARAMETER] =
     g_signal_new("change-parameter",
@@ -441,7 +441,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_parameter),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -458,7 +458,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::change-pitch-bend signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[CHANGE_PITCH_BEND] =
     g_signal_new("change-pitch-bend",
@@ -466,7 +466,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_pitch_bend),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -482,7 +482,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::change-program signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[CHANGE_PROGRAM] =
     g_signal_new("change-program",
@@ -490,7 +490,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_program),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -505,7 +505,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::change-channel-pressure signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[CHANGE_PRESSURE] =
     g_signal_new("change-channel-pressure",
@@ -513,7 +513,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_pressure),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -528,7 +528,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::sysex signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[SYSEX] =
     g_signal_new("sysex",
@@ -536,7 +536,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sysex),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_POINTER_UINT,
+		 ags_cclosure_marshal_VOID__UINT_POINTER_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_POINTER,
@@ -551,7 +551,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::quarter-frame signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[QUARTER_FRAME] =
     g_signal_new("quarter-frame",
@@ -559,7 +559,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_quarter_frame),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -573,7 +573,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::song-position signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[SONG_POSITION] =
     g_signal_new("song-position",
@@ -581,7 +581,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_song_position),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -594,7 +594,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::song-select signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[SONG_SELECT] =
     g_signal_new("song-select",
@@ -602,7 +602,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_song_select),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -614,7 +614,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::tune-request signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[TUNE_REQUEST] =
     g_signal_new("tune-request",
@@ -635,7 +635,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::sequence-number signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[SEQUENCE_NUMBER] =
     g_signal_new("sequence-number",
@@ -643,7 +643,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sequence_number),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -660,7 +660,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::smtpe signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[SMTPE] =
     g_signal_new("smtpe",
@@ -668,7 +668,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_smtpe),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 6,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -685,7 +685,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::tempo signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[TEMPO] =
     g_signal_new("tempo",
@@ -693,7 +693,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_tempo),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -709,7 +709,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::time-signature signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[TIME_SIGNATURE] =
     g_signal_new("time-signature",
@@ -717,7 +717,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_time_signature),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 5,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -734,7 +734,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    * 
    * The ::key-signature signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[KEY_SIGNATURE] =
     g_signal_new("key-signature",
@@ -742,7 +742,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_key_signature),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_BOOLEAN,
+		 ags_cclosure_marshal_VOID__UINT_UINT_BOOLEAN,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -758,7 +758,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::sequencer-meta-event signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[SEQUENCER_META_EVENT] =
     g_signal_new("sequencer-meta-event",
@@ -766,7 +766,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sequencer_meta_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -782,7 +782,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::text-event signal is emited during building of event.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   midi_builder_signals[TEXT_EVENT] =
     g_signal_new("text-event",
@@ -790,7 +790,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_text_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_STRING_UINT,
+		 ags_cclosure_marshal_VOID__UINT_STRING_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_STRING,
@@ -870,7 +870,7 @@ ags_midi_builder_finalize(GObject *gobject)
  * 
  * Returns: the newly allocated #AgsMidiBuilderHeader-struct
  * 
- * Since: 0.7.90
+ * Since: 1.0.0
  */
 AgsMidiBuilderHeader*
 ags_midi_builder_header_alloc()
@@ -899,7 +899,7 @@ ags_midi_builder_header_alloc()
  * 
  * Free MIDI builder header.
  * 
- * Since: 0.7.90
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_header_free(AgsMidiBuilderHeader *midi_builder_header)
@@ -918,7 +918,7 @@ ags_midi_builder_header_free(AgsMidiBuilderHeader *midi_builder_header)
  * 
  * Returns: the newly allocated #AgsMidiBuilderTrack-struct
  * 
- * Since: 0.7.90
+ * Since: 1.0.0
  */
 AgsMidiBuilderTrack*
 ags_midi_builder_track_alloc()
@@ -944,7 +944,7 @@ ags_midi_builder_track_alloc()
  * 
  * Free MIDI builder track.
  * 
- * Since: 0.7.90
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_track_free(AgsMidiBuilderTrack *midi_builder_track)
@@ -974,7 +974,7 @@ ags_midi_builder_track_free(AgsMidiBuilderTrack *midi_builder_track)
  * 
  * Returns: the matching #GList-struct
  * 
- * Since: 0.7.90
+ * Since: 1.0.0
  */
 GList*
 ags_midi_builder_track_find_delta_time_with_track_name(GList *midi_builder_track,
@@ -1002,7 +1002,7 @@ ags_midi_builder_track_find_delta_time_with_track_name(GList *midi_builder_track
  * 
  * Insert MIDI message.
  * 
- * Since: 0.7.90
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_track_insert_midi_message(AgsMidiBuilderTrack *midi_builder_track,
@@ -1068,7 +1068,7 @@ ags_midi_builder_track_insert_midi_message(AgsMidiBuilderTrack *midi_builder_tra
  * 
  * Returns: the buffer position before @delta_time
  * 
- * Since: 0.7.90
+ * Since: 1.0.0
  */
 unsigned char*
 ags_midi_builder_track_get_delta_time_offset(AgsMidiBuilderTrack *midi_builder_track,
@@ -1133,7 +1133,7 @@ ags_midi_builder_real_midi_putc(AgsMidiBuilder *midi_builder,
  *
  * Put char in MIDI file.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_midi_putc(AgsMidiBuilder *midi_builder,
@@ -1165,7 +1165,7 @@ ags_midi_builder_real_on_error(AgsMidiBuilder *midi_builder,
  *
  * Triggered as an error occurs.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_on_error(AgsMidiBuilder *midi_builder,
@@ -1227,7 +1227,7 @@ ags_midi_builder_real_append_header(AgsMidiBuilder *midi_builder,
  *
  * Appends MIDI header to @midi_builder.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_header(AgsMidiBuilder *midi_builder,
@@ -1295,7 +1295,7 @@ ags_midi_builder_real_append_track(AgsMidiBuilder *midi_builder,
  *
  * Append a track called @track_name to @midi_builder.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_track(AgsMidiBuilder *midi_builder,
@@ -1343,7 +1343,7 @@ ags_midi_builder_real_append_key_on(AgsMidiBuilder *midi_builder,
  *
  * Append key-on for @note to @midi_builder with key dynamics @velocity, at @delta_time.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_key_on(AgsMidiBuilder *midi_builder,
@@ -1397,7 +1397,7 @@ ags_midi_builder_real_append_key_off(AgsMidiBuilder *midi_builder,
  *
  * Append key-off for @note to @midi_builder with key dynamics @velocity, at @delta_time.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_key_off(AgsMidiBuilder *midi_builder,
@@ -1452,7 +1452,7 @@ ags_midi_builder_real_append_key_pressure(AgsMidiBuilder *midi_builder,
  *
  * Append key-pressure for @note to @midi_builder with key dynamics @pressure, at @delta_time.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_key_pressure(AgsMidiBuilder *midi_builder,
@@ -1506,7 +1506,7 @@ ags_midi_builder_real_append_change_parameter(AgsMidiBuilder *midi_builder,
  *
  * Appends change parameter.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_change_parameter(AgsMidiBuilder *midi_builder,
@@ -1560,7 +1560,7 @@ ags_midi_builder_real_append_change_pitch_bend(AgsMidiBuilder *midi_builder,
  *
  * Change pitch bend.
  * 
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_change_pitch_bend(AgsMidiBuilder *midi_builder,
@@ -1611,7 +1611,7 @@ ags_midi_builder_real_append_change_program(AgsMidiBuilder *midi_builder,
  *
  * Appends change program.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_change_program(AgsMidiBuilder *midi_builder,
@@ -1660,7 +1660,7 @@ ags_midi_builder_real_append_change_pressure(AgsMidiBuilder *midi_builder,
  * 
  * Appends change pressure.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_change_pressure(AgsMidiBuilder *midi_builder,
@@ -1707,7 +1707,7 @@ ags_midi_builder_real_append_sysex(AgsMidiBuilder *midi_builder,
  *
  * Appends sysex data.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_sysex(AgsMidiBuilder *midi_builder,
@@ -1754,7 +1754,7 @@ ags_midi_builder_real_append_quarter_frame(AgsMidiBuilder *midi_builder,
  *
  * Appends quarter frame.
  * 
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_quarter_frame(AgsMidiBuilder *midi_builder,
@@ -1799,7 +1799,7 @@ ags_midi_builder_real_append_song_position(AgsMidiBuilder *midi_builder,
  * 
  * Appends song position.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_song_position(AgsMidiBuilder *midi_builder,
@@ -1843,7 +1843,7 @@ ags_midi_builder_real_append_song_select(AgsMidiBuilder *midi_builder,
  * 
  * Appends song select.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_song_select(AgsMidiBuilder *midi_builder,
@@ -1884,7 +1884,7 @@ ags_midi_builder_real_append_tune_request(AgsMidiBuilder *midi_builder,
  * 
  * Appends tune request.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_tune_request(AgsMidiBuilder *midi_builder,
@@ -1926,7 +1926,7 @@ ags_midi_builder_real_append_sequence_number(AgsMidiBuilder *midi_builder,
  * 
  * Appends sequence number.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_sequence_number(AgsMidiBuilder *midi_builder,
@@ -1974,7 +1974,7 @@ ags_midi_builder_real_append_smtpe(AgsMidiBuilder *midi_builder,
  * 
  * Appends smtpe.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_smtpe(AgsMidiBuilder *midi_builder,
@@ -2019,7 +2019,7 @@ ags_midi_builder_real_append_tempo(AgsMidiBuilder *midi_builder,
  * 
  * Appends tempo.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_tempo(AgsMidiBuilder *midi_builder,
@@ -2066,7 +2066,7 @@ ags_midi_builder_real_append_time_signature(AgsMidiBuilder *midi_builder,
  *
  * Appends time signature.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_time_signature(AgsMidiBuilder *midi_builder,
@@ -2111,7 +2111,7 @@ ags_midi_builder_real_append_key_signature(AgsMidiBuilder *midi_builder,
  *
  * Appends key signature.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_key_signature(AgsMidiBuilder *midi_builder,
@@ -2157,7 +2157,7 @@ ags_midi_builder_real_append_sequencer_meta_event(AgsMidiBuilder *midi_builder,
  * 
  * Appends sequencer meta event.
  *
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_sequencer_meta_event(AgsMidiBuilder *midi_builder,
@@ -2208,7 +2208,7 @@ ags_midi_builder_real_append_text_event(AgsMidiBuilder *midi_builder,
  *
  * Appends text event.
  * 
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_text_event(AgsMidiBuilder *midi_builder,
@@ -2232,7 +2232,7 @@ ags_midi_builder_append_text_event(AgsMidiBuilder *midi_builder,
  * 
  * Append from XML node @node.
  * 
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_append_xml_node(AgsMidiBuilder *midi_builder,
@@ -3028,7 +3028,7 @@ ags_midi_builder_append_xml_node(AgsMidiBuilder *midi_builder,
  *
  * Builds from XML document @doc.
  * 
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_from_xml_doc(AgsMidiBuilder *midi_builder,
@@ -3071,7 +3071,7 @@ ags_midi_builder_from_xml_doc(AgsMidiBuilder *midi_builder,
  *
  * Build the MIDI data.
  * 
- * Since: 0.7.3
+ * Since: 1.0.0
  */
 void
 ags_midi_builder_build(AgsMidiBuilder *midi_builder)
@@ -3152,7 +3152,7 @@ ags_midi_builder_build(AgsMidiBuilder *midi_builder)
  *
  * Returns: an #AgsMidiBuilder
  * 
- * Since: 0.7.0
+ * Since: 1.0.0
  */
 AgsMidiBuilder*
 ags_midi_builder_new(FILE *file)
