@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -104,7 +104,7 @@ ags_midi_file_class_init(AgsMidiFileClass *midi_file)
    *
    * The assigned filename to perform input/output on.
    * 
-   * Since: 0.7.74
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("filename",
 				   i18n_pspec("assigned filename"),
@@ -116,6 +116,12 @@ ags_midi_file_class_init(AgsMidiFileClass *midi_file)
 				  param_spec);
 }
 
+GQuark
+ags_midi_file_error_quark()
+{
+  return(g_quark_from_static_string("ags-midi-file-error-quark"));
+}
+
 void
 ags_midi_file_init(AgsMidiFile *midi_file)
 {
@@ -138,6 +144,9 @@ ags_midi_file_init(AgsMidiFile *midi_file)
   midi_file->track = NULL;
 
   midi_file->current_track = NULL;
+
+  midi_file->notation = NULL;
+  midi_file->midi = NULL;
 }
 
 void
@@ -212,21 +221,32 @@ ags_midi_file_finalize(GObject *gobject)
 /**
  * ags_midi_file_open:
  * @midi_file: the #AgsMidiFile
+ * @filename: the filename
  *
  * Opens a MIDI file read-only.
  *
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 gboolean
-ags_midi_file_open(AgsMidiFile *midi_file)
+ags_midi_file_open(AgsMidiFile *midi_file,
+		   gchar *filename)
 {
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->filename == NULL){
     return(FALSE);
   }
 
+  g_object_set(midi_file,
+	       "filename", filename,
+	       NULL);
+  
+  if(!g_file_test(filename,
+		  (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))){
+    return(FALSE);
+  }
+
   midi_file->file = fopen(midi_file->filename,
 			  "r");
 
@@ -243,13 +263,13 @@ ags_midi_file_open(AgsMidiFile *midi_file)
  *
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 gboolean
 ags_midi_file_open_from_data(AgsMidiFile *midi_file,
 			     unsigned char *data, guint buffer_length)
 {
-  if(midi_file == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     return(FALSE);
   }
 
@@ -262,18 +282,32 @@ ags_midi_file_open_from_data(AgsMidiFile *midi_file,
 /**
  * ags_midi_file_rw_open:
  * @midi_file: the #AgsMidiFile
+ * @filename: the filename
+ * @create: %TRUE create file if not exists, else if %FALSE return
  * 
  * Opens a MIDI file with read-write permission.
  *
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 gboolean
-ags_midi_file_rw_open(AgsMidiFile *midi_file)
+ags_midi_file_rw_open(AgsMidiFile *midi_file,
+		      gchar *filename,
+		      gboolean create)
 {
-  if(midi_file == NULL ||
-     midi_file->filename == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
+     filename == NULL){
+    return(FALSE);
+  }
+
+  g_object_set(midi_file,
+	       "filename", filename,
+	       NULL);
+  
+  if(!create &&
+     !g_file_test(filename,
+		  (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))){
     return(FALSE);
   }
   
@@ -289,12 +323,12 @@ ags_midi_file_rw_open(AgsMidiFile *midi_file)
  *
  * Closes the file stream.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_close(AgsMidiFile *midi_file)
 {
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->file == NULL){
     return;
   }
@@ -312,7 +346,7 @@ ags_midi_file_close(AgsMidiFile *midi_file)
  *
  * Returns: the data array just read
  * 
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 unsigned char*
 ags_midi_file_read(AgsMidiFile *midi_file)
@@ -321,7 +355,7 @@ ags_midi_file_read(AgsMidiFile *midi_file)
 
   size_t n_read;
   
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->filename == NULL ||
      midi_file->file == NULL){
     return(NULL);
@@ -329,6 +363,10 @@ ags_midi_file_read(AgsMidiFile *midi_file)
   
   stat(midi_file->filename, &sb);
 
+  if(sb.st_size == 0){
+    return(NULL);
+  }
+  
   midi_file->buffer_length = sb.st_size + 1;
   midi_file->buffer = (unsigned char *) malloc(midi_file->buffer_length * sizeof(unsigned char));
   midi_file->buffer[sb.st_size] = EOF;
@@ -351,7 +389,7 @@ ags_midi_file_read(AgsMidiFile *midi_file)
  *
  * Writes @data to the file stream and to internal buffer, reallocates it if necessary.
  * 
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_write(AgsMidiFile *midi_file,
@@ -359,7 +397,7 @@ ags_midi_file_write(AgsMidiFile *midi_file,
 {
   unsigned char *start;
   
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->file == NULL){
     return;
   }
@@ -392,12 +430,12 @@ ags_midi_file_write(AgsMidiFile *midi_file,
  *
  * Seeks the file stream's offset.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_seek(AgsMidiFile *midi_file, guint position, gint whence)
 {
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->file == NULL){
     return;
   }
@@ -411,12 +449,12 @@ ags_midi_file_seek(AgsMidiFile *midi_file, guint position, gint whence)
  * 
  * Flushes file stream's data buffer to disc.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_flush(AgsMidiFile *midi_file)
 {
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->file == NULL){
     return;
   }
@@ -424,29 +462,85 @@ ags_midi_file_flush(AgsMidiFile *midi_file)
   fflush(midi_file->file);
 }
 
+/**
+ * ags_midi_file_read_byte:
+ * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
+ *  
+ * Reads a unsigned char quantity.
+ *
+ * Returns: the current value at file's iteration pointer
+ * 
+ * Since: 2.0.0
+ */
+unsigned char
+ags_midi_file_read_byte(AgsMidiFile *midi_file,
+			GError **error)
+{
+  unsigned char value;
+
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
+
+  if(midi_file->iter + 1 < midi_file->buffer + midi_file->buffer_length){
+    value = (midi_file->iter[0]);
+    
+    midi_file->iter += 1;
+  }else{
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
+  }
+  
+  return(value);
+}
+
 /**
  * ags_midi_file_read_gint16:
  * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
  *  
  * Reads a gint16 quantity.
  *
  * Returns: the current value at file's iteration pointer
  * 
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 gint16
-ags_midi_file_read_gint16(AgsMidiFile *midi_file)
+ags_midi_file_read_gint16(AgsMidiFile *midi_file,
+			  GError **error)
 {
   unsigned char str[2];
   gint16 value = 0;
 
-  str[0] = (midi_file->iter[0]);
-  str[1] = (midi_file->iter[1]);
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
 
-  midi_file->iter += 2;
-  
-  value = (str[0] & 0xff);
-  value = (value<<8) + (str[1] & 0xff);
+  if(midi_file->iter + 2 < midi_file->buffer + midi_file->buffer_length){
+    str[0] = (midi_file->iter[0]);
+    str[1] = (midi_file->iter[1]);
+    
+    midi_file->iter += 2;
+    
+    value = (str[0] & 0xff);
+    value = (value<<8) + (str[1] & 0xff);
+  }else{
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
+  }
   
   return(value);
 }
@@ -454,29 +548,46 @@ ags_midi_file_read_gint16(AgsMidiFile *midi_file)
 /**
  * ags_midi_file_read_gint24:
  * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
  *  
  * Reads a 24-bit quantity.
  *
  * Returns: the current value at file's iteration pointer
  * 
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 gint32
-ags_midi_file_read_gint24(AgsMidiFile *midi_file)
+ags_midi_file_read_gint24(AgsMidiFile *midi_file,
+			  GError **error)
 {
   unsigned char str[4];
   gint32 value = 0;
-  
-  str[0] = (unsigned char) 0x00;
-  str[1] = (midi_file->iter[0]);
-  str[2] = (midi_file->iter[1]);
-  str[3] = (midi_file->iter[2]);
 
-  midi_file->iter += 3;
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
   
-  value = (value<<8) + (str[1] & 0xff);
-  value = (value<<8) + (str[2] & 0xff);
-  value = (value<<8) + (str[3] & 0xff);
+  if(midi_file->iter + 3 < midi_file->buffer + midi_file->buffer_length){
+    str[0] = (unsigned char) 0x00;
+    str[1] = (midi_file->iter[0]);
+    str[2] = (midi_file->iter[1]);
+    str[3] = (midi_file->iter[2]);
+
+    midi_file->iter += 3;
+  
+    value = (value<<8) + (str[1] & 0xff);
+    value = (value<<8) + (str[2] & 0xff);
+    value = (value<<8) + (str[3] & 0xff);
+  }else{  
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
+  }
   
   return(value);
 }
@@ -484,30 +595,47 @@ ags_midi_file_read_gint24(AgsMidiFile *midi_file)
 /**
  * ags_midi_file_read_gint32:
  * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
  *  
  * Reads a gint32 quantity.
  *
  * Returns: the current value at file's iteration pointer
  * 
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 gint32
-ags_midi_file_read_gint32(AgsMidiFile *midi_file)
+ags_midi_file_read_gint32(AgsMidiFile *midi_file,
+			  GError **error)
 {
   unsigned char str[4];
   gint32 value;
   
-  str[0] = (midi_file->iter[0]);
-  str[1] = (midi_file->iter[1]);
-  str[2] = (midi_file->iter[2]);
-  str[3] = (midi_file->iter[3]);
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
   
-  midi_file->iter += 4;
+  if(midi_file->iter + 4 < midi_file->buffer + midi_file->buffer_length){
+    str[0] = (midi_file->iter[0]);
+    str[1] = (midi_file->iter[1]);
+    str[2] = (midi_file->iter[2]);
+    str[3] = (midi_file->iter[3]);
   
-  value = (str[0] & 0xff);
-  value = (value<<8) + (str[1] & 0xff);
-  value = (value<<8) + (str[2] & 0xff);
-  value = (value<<8) + (str[3] & 0xff);
+    midi_file->iter += 4;
+  
+    value = (str[0] & 0xff);
+    value = (value<<8) + (str[1] & 0xff);
+    value = (value<<8) + (str[2] & 0xff);
+    value = (value<<8) + (str[3] & 0xff);
+  }else{
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
+  }
   
   return(value);
 }
@@ -515,35 +643,64 @@ ags_midi_file_read_gint32(AgsMidiFile *midi_file)
 /**
  * ags_midi_file_read_varlength:
  * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
  *  
  * Reads a variable length quantity.
  *
  * Returns: the current value at file's iteration pointer
  * 
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 long
-ags_midi_file_read_varlength(AgsMidiFile *midi_file)
+ags_midi_file_read_varlength(AgsMidiFile *midi_file,
+			     GError **error)
 {
   long value;
   guint i;
   unsigned char c;
+  gboolean success;
   
-  c = midi_file->iter[0];
-  value = c;
-  i = 1;
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
 
-  midi_file->iter += 1;
+  success = TRUE;
   
-  if(c & 0x80){
+  if(midi_file->iter + 1 < midi_file->buffer + midi_file->buffer_length){
+    c = midi_file->iter[0];
+    value = c;
+    i = 1;
+
+    midi_file->iter += 1;
+  }else{
+    success = FALSE;
+  }
+  
+  if((c & 0x80) &&
+     success){
     value &= 0x7F;
    
     do{
-      //TODO:JK: unsafe
-      value = (value << 7) + ((c = (midi_file->iter[0])) & 0x7F);
-      i++;
-      midi_file->iter += 1;
-    }while(c & 0x80);
+      if(midi_file->iter + 1 < midi_file->buffer + midi_file->buffer_length){
+	//TODO:JK: unsafe
+	value = (value << 7) + ((c = (midi_file->iter[0])) & 0x7F);
+	i++;
+	midi_file->iter += 1;
+      }else{
+	success = FALSE;
+      }
+    }while((c & 0x80) && success);
+  }
+
+  if(!success){
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
   }
   
   return(value);
@@ -553,21 +710,38 @@ ags_midi_file_read_varlength(AgsMidiFile *midi_file)
  * ags_midi_file_read_text:
  * @midi_file: the #AgsMidiFile
  * @length: the number of bytes to be read, or as long valid string for -1
+ * @error: the #GError pointer return location
  *  
  * Reads a string.
  *
  * Returns: the string at file's iteration pointer
  * 
- * Since: 0.7.1
+ * Since: 2.0.0
  */
 unsigned char*
 ags_midi_file_read_text(AgsMidiFile *midi_file,
-			gint length)
+			gint length,
+			GError **error)
 {
   unsigned char *text;
   gchar c;
   guint i;
+  
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(NULL);
+  }
 
+  if(midi_file->iter + length >= midi_file->buffer + midi_file->buffer_length){
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+
+    return(NULL);
+  }
+  
   text = (unsigned char *) malloc((AGS_MIDI_FILE_MAX_TEXT_LENGTH + 1) * sizeof(unsigned char));
   memset(text, 0, AGS_MIDI_FILE_MAX_TEXT_LENGTH * sizeof(unsigned char));
   i = 0;
@@ -591,6 +765,34 @@ ags_midi_file_read_text(AgsMidiFile *midi_file,
   return(text);
 }
 
+/**
+ * ags_midi_file_write_byte:
+ * @midi_file: the #AgsMidiFile
+ * @val: the value to write
+ *
+ * Writes a unsigned char quantity to internal buffer.
+ *
+ * Since: 1.0.0
+ */
+void
+ags_midi_file_write_byte(AgsMidiFile *midi_file, unsigned char val)
+{
+  if(midi_file->iter + 1 >= (midi_file->buffer + midi_file->buffer_length)){
+    size_t new_length;
+
+    new_length = (midi_file->iter + 1) - (midi_file->buffer + midi_file->buffer_length);
+    
+    midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
+						  new_length * sizeof(unsigned char));
+
+    midi_file->buffer_length = new_length;
+  }
+
+  midi_file->iter[0] = val;
+  
+  midi_file->iter += 1;
+}
+
 /**
  * ags_midi_file_write_gint16:
  * @midi_file: the #AgsMidiFile
@@ -598,15 +800,15 @@ ags_midi_file_read_text(AgsMidiFile *midi_file,
  *
  * Writes a gint16 quantity to internal buffer.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_write_gint16(AgsMidiFile *midi_file, gint16 val)
 {
-  if(&(midi_file->iter[2]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + 2) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[2]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + 2) - (midi_file->buffer + midi_file->buffer_length);
     
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
@@ -627,15 +829,15 @@ ags_midi_file_write_gint16(AgsMidiFile *midi_file, gint16 val)
  *
  * Writes a 24-bit quantity to internal buffer.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_write_gint24(AgsMidiFile *midi_file, gint32 val)
 {
-  if(&(midi_file->iter[3]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + 3) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[3]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + 3) - (midi_file->buffer + midi_file->buffer_length);
     
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
@@ -657,15 +859,15 @@ ags_midi_file_write_gint24(AgsMidiFile *midi_file, gint32 val)
  *
  * Writes a gint32 quantity to internal buffer.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_write_gint32(AgsMidiFile *midi_file, gint32 val)
 {
-  if(&(midi_file->iter[4]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + 4) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[4]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + 4) - (midi_file->buffer + midi_file->buffer_length);
     
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
@@ -688,7 +890,7 @@ ags_midi_file_write_gint32(AgsMidiFile *midi_file, gint32 val)
  *
  * Writes a variable length quantity to internal buffer.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_write_varlength(AgsMidiFile *midi_file, long val)
@@ -708,10 +910,10 @@ ags_midi_file_write_varlength(AgsMidiFile *midi_file, long val)
   }while(0x80 & c);
 
   /* realloc buffer if needed */
-  if(&(midi_file->iter[i]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + i) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[i]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + i) - (midi_file->buffer + midi_file->buffer_length);
     
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
@@ -735,7 +937,7 @@ ags_midi_file_write_varlength(AgsMidiFile *midi_file, long val)
  *
  * Writes a string to internal buffer up to length bytes.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_write_text(AgsMidiFile *midi_file,
@@ -747,10 +949,10 @@ ags_midi_file_write_text(AgsMidiFile *midi_file,
     return;
   }
 
-  if(&(midi_file->iter[length]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + length) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[length]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + length) - (midi_file->buffer + midi_file->buffer_length);
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
 
@@ -768,16 +970,18 @@ ags_midi_file_write_text(AgsMidiFile *midi_file,
  * ags_midi_file_read_header:
  * @midi_file: the #AgsMidiFile
  * @buffer_length: pointer to return buffer length or %NULL
+ * @error: the #GError pointer return location
  *
  * Reads the MIDI file's header and positions internal buffer pointer just behind it.
  *
  * Returns: the header's bytes
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 unsigned char*
 ags_midi_file_read_header(AgsMidiFile *midi_file,
-			  guint *buffer_length)
+			  guint *buffer_length,
+			  GError **error)
 {
   static gchar header[] = "MThd";
 
@@ -787,7 +991,9 @@ ags_midi_file_read_header(AgsMidiFile *midi_file,
   guint n;
   gchar c;
 
-  if(midi_file == NULL){
+  GError *local_error;
+  
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     if(buffer_length != NULL){
       *buffer_length = 0;
     }
@@ -817,10 +1023,21 @@ ags_midi_file_read_header(AgsMidiFile *midi_file,
   length += 4;
 
   /* get some values */
-  midi_file->offset = (guint) ags_midi_file_read_gint32(midi_file);
-  midi_file->format = (guint) ags_midi_file_read_gint16(midi_file);
-  midi_file->count = (guint) ags_midi_file_read_gint16(midi_file);
-  midi_file->division = (guint) ags_midi_file_read_gint16(midi_file);
+  local_error = NULL;
+  midi_file->offset = (guint) ags_midi_file_read_gint32(midi_file,
+							&local_error);
+  
+  local_error = NULL;
+  midi_file->format = (guint) ags_midi_file_read_gint16(midi_file,
+							&local_error);
+
+  local_error = NULL;
+  midi_file->count = (guint) ags_midi_file_read_gint16(midi_file,
+						       &local_error);
+
+  local_error = NULL;
+  midi_file->division = (guint) ags_midi_file_read_gint16(midi_file,
+							  &local_error);
 
   if((midi_file->division) & 0x8000){
     /* SMPTE */
@@ -848,7 +1065,7 @@ ags_midi_file_read_header(AgsMidiFile *midi_file,
  *
  * Write header bytes.
  *
- * Since: 0.7.1
+ * Since: 1.0.0
  */
 void
 ags_midi_file_write_header(AgsMidiFile *midi_file,
@@ -856,14 +1073,14 @@ ags_midi_file_write_header(AgsMidiFile *midi_file,
 {
   guint i;
   
-  if(midi_file == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     return;
   }
 
-  if(&(midi_file->iter[length]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + length) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[length]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + length) - (midi_file->buffer + midi_file->buffer_length);
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
 
@@ -877,9 +1094,22 @@ ags_midi_file_write_header(AgsMidiFile *midi_file,
   midi_file->iter += length;
 }
 
+/**
+ * ags_midi_file_read_header:
+ * @midi_file: the #AgsMidiFile
+ * @buffer_length: pointer to return buffer length or %NULL
+ * @error: the #GError pointer return location
+ *
+ * Reads the MIDI file's track data.
+ *
+ * Returns: the track's bytes
+ *
+ * Since: 1.0.0
+ */
 unsigned char*
 ags_midi_file_read_track_data(AgsMidiFile *midi_file,
-			      guint *buffer_length)
+			      guint *buffer_length,
+			      GError **error)
 {
   gchar *track_name;
   unsigned char *data, *start;
@@ -892,8 +1122,10 @@ ags_midi_file_read_track_data(AgsMidiFile *midi_file,
   gboolean end_of_track;
 
   static gchar track[] = "MTrk";
+
+  GError *local_error;
   
-  if(midi_file == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     if(buffer_length != NULL){
       *buffer_length = 0;
     }
@@ -923,12 +1155,16 @@ ags_midi_file_read_track_data(AgsMidiFile *midi_file,
   start = midi_file->iter - 4;
   
   /* offset */
-  ags_midi_file_read_gint32(midi_file);
+  local_error = NULL;
+  ags_midi_file_read_gint32(midi_file,
+			    &local_error);
 
   end_of_track = FALSE;
   
   while(!end_of_track){
-    delta_time = ags_midi_file_read_varlength(midi_file);
+    local_error = NULL;
+    delta_time = ags_midi_file_read_varlength(midi_file,
+					      &local_error);
     
     status = midi_file->iter[0];
     midi_file->iter += 1;
@@ -1029,9 +1265,14 @@ ags_midi_file_read_track_data(AgsMidiFile *midi_file,
 	      guint text_length;
 	      
 	      /* These are all text events */
-	      text_length = ags_midi_file_read_varlength(midi_file);
+	      local_error = NULL;
+	      text_length = ags_midi_file_read_varlength(midi_file,
+							 &local_error);
+
+	      local_error = NULL;
 	      text = ags_midi_file_read_text(midi_file,
-					     text_length);
+					     text_length,
+					     &local_error);
 
 	      g_free(text);
 	    }
@@ -1139,14 +1380,14 @@ ags_midi_file_write_track_data(AgsMidiFile *midi_file,
 {
   guint i;
   
-  if(midi_file == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     return;
   }
 
-  if(&(midi_file->iter[length]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + length) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[length]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + length) - (midi_file->buffer + midi_file->buffer_length);
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
 
@@ -1160,6 +1401,18 @@ ags_midi_file_write_track_data(AgsMidiFile *midi_file,
   midi_file->iter += length;
 }
 
+void
+ags_midi_file_read_notation(AgsMidiFile *midi_file)
+{
+  //TODO:JK: implement me
+}
+
+void
+ags_mid_file_read_midi(AgsMidiFile *midi_file)
+{
+  //TODO:JK: implement me
+}
+
 AgsMidiFile*
 ags_midi_file_new(gchar *filename)
 {
