@@ -163,7 +163,7 @@ ags_file_class_init(AgsFileClass *file)
    *
    * The assigned filename to open and read from.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_string("filename",
 				   i18n_pspec("filename to read or write"),
@@ -179,7 +179,7 @@ ags_file_class_init(AgsFileClass *file)
    *
    * The charset encoding to use.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_string("encoding",
 				   i18n_pspec("encoding to use"),
@@ -195,7 +195,7 @@ ags_file_class_init(AgsFileClass *file)
    *
    * The format of embedded audio data.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_string("audio-format",
 				   i18n_pspec("audio format to use"),
@@ -211,7 +211,7 @@ ags_file_class_init(AgsFileClass *file)
    *
    * The encoding to use for embedding audio data.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_string("audio-encoding",
 				   i18n_pspec("audio encoding to use"),
@@ -227,7 +227,7 @@ ags_file_class_init(AgsFileClass *file)
    *
    * The assigned xml-doc.
    * 
-   * Since: 0.7.3
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_pointer("xml-doc",
 				    i18n_pspec("xml document of file"),
@@ -242,7 +242,7 @@ ags_file_class_init(AgsFileClass *file)
    *
    * The application context assigned with.
    *
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_object("application-context",
 				   i18n_pspec("application context of file"),
@@ -273,7 +273,7 @@ ags_file_class_init(AgsFileClass *file)
    * 
    * Open @file with appropriate filename.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   file_signals[OPEN] =
     g_signal_new("open",
@@ -294,7 +294,7 @@ ags_file_class_init(AgsFileClass *file)
    * 
    * Open @file from a buffer containing the file.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   file_signals[OPEN_FROM_DATA] =
     g_signal_new("open-from-data",
@@ -302,7 +302,7 @@ ags_file_class_init(AgsFileClass *file)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsFileClass, open_from_data),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__STRING_UINT_POINTER,
+		 ags_cclosure_marshal_VOID__STRING_UINT_POINTER,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_STRING,
 		 G_TYPE_UINT,
@@ -316,7 +316,7 @@ ags_file_class_init(AgsFileClass *file)
    * 
    * Open @file in read-write mode.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   file_signals[RW_OPEN] =
     g_signal_new("rw-open",
@@ -324,7 +324,7 @@ ags_file_class_init(AgsFileClass *file)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsFileClass, rw_open),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__BOOLEAN_POINTER,
+		 ags_cclosure_marshal_VOID__BOOLEAN_POINTER,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_BOOLEAN,
 		 G_TYPE_POINTER);
@@ -335,7 +335,7 @@ ags_file_class_init(AgsFileClass *file)
    * 
    * Write XML Document to disk.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   file_signals[WRITE] =
     g_signal_new("write",
@@ -362,7 +362,7 @@ ags_file_class_init(AgsFileClass *file)
    * Resolve references and generate thus XPath expressions just
    * before writing to disk.
    *
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   file_signals[WRITE_RESOLVE] =
     g_signal_new("write_resolve",
@@ -379,7 +379,7 @@ ags_file_class_init(AgsFileClass *file)
    *
    * Read a XML document from disk with specified filename.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   file_signals[READ] =
     g_signal_new("read",
@@ -397,7 +397,7 @@ ags_file_class_init(AgsFileClass *file)
    * Resolve XPath expressions to their counterpart the newly created
    * instances refering to.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   file_signals[READ_RESOLVE] =
     g_signal_new("read_resolve",
@@ -414,7 +414,7 @@ ags_file_class_init(AgsFileClass *file)
    *
    * Hook after reading XML document to update or start the application.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   file_signals[READ_START] =
     g_signal_new("read_start",
@@ -646,7 +646,7 @@ ags_file_finalize(GObject *gobject)
  *
  * Returns: the md5 checksum
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 gchar*
 ags_file_str2md5(gchar *content, guint content_length)
@@ -668,7 +668,7 @@ ags_file_str2md5(gchar *content, guint content_length)
  *
  * Adds @id_ref to @file.
  * 
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_file_add_id_ref(AgsFile *file, GObject *id_ref)
@@ -691,7 +691,7 @@ ags_file_add_id_ref(AgsFile *file, GObject *id_ref)
  * 
  * Returns: the matching #GObject
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 GObject*
 ags_file_find_id_ref_by_node(AgsFile *file, xmlNode *node)
@@ -723,7 +723,7 @@ ags_file_find_id_ref_by_node(AgsFile *file, xmlNode *node)
  * 
  * Returns: the matching #GObject
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 GObject*
 ags_file_find_id_ref_by_xpath(AgsFile *file, gchar *xpath)
@@ -784,7 +784,7 @@ ags_file_find_id_ref_by_xpath(AgsFile *file, gchar *xpath)
  * 
  * Returns: the matching #GObject
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 GObject*
 ags_file_find_id_ref_by_reference(AgsFile *file, gpointer ref)
@@ -814,7 +814,7 @@ ags_file_find_id_ref_by_reference(AgsFile *file, gpointer ref)
  *
  * Add @file_lookup for later invoking.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_file_add_lookup(AgsFile *file, GObject *file_lookup)
@@ -836,7 +836,7 @@ ags_file_add_lookup(AgsFile *file, GObject *file_lookup)
  *
  * Add @file_launch for later invoking.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_file_add_launch(AgsFile *file, GObject *file_launch)
@@ -885,7 +885,7 @@ ags_file_real_open(AgsFile *file,
  *
  * Opens the file specified by :filename property.
  *
- * Since: 0.4.0 
+ * Since: 1.0.0 
  */
 void
 ags_file_open(AgsFile *file,
@@ -936,7 +936,7 @@ ags_file_real_open_from_data(AgsFile *file,
  *
  * Opens the file provided by @data.
  *
- * Since: 0.4.0 
+ * Since: 1.0.0 
  */
 void
 ags_file_open_from_data(AgsFile *file,
@@ -977,7 +977,7 @@ ags_file_real_rw_open(AgsFile *file,
  *
  * Opens the file specified by :filename property in read-write mode.
  *
- * Since: 0.4.0 
+ * Since: 1.0.0 
  */
 void
 ags_file_rw_open(AgsFile *file,
@@ -1001,7 +1001,7 @@ ags_file_rw_open(AgsFile *file,
  *
  * Opens the file specified by @filename property.
  *
- * Since: 0.4.0 
+ * Since: 1.0.0 
  */
 void
 ags_file_open_filename(AgsFile *file,
@@ -1117,7 +1117,7 @@ ags_file_real_write(AgsFile *file)
  *
  * Write the XML document to disk.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_file_write(AgsFile *file)
@@ -1245,7 +1245,7 @@ ags_file_real_write_resolve(AgsFile *file)
  *
  * Resolve references to XPath expressions.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_file_write_resolve(AgsFile *file)
@@ -1316,7 +1316,7 @@ ags_file_real_read(AgsFile *file)
  *
  * Read XML document from disk.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_file_read(AgsFile *file)
@@ -1351,7 +1351,7 @@ ags_file_real_read_resolve(AgsFile *file)
  *
  * Resolve XPath expressions to references.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_file_read_resolve(AgsFile *file)
@@ -1384,7 +1384,7 @@ ags_file_real_read_start(AgsFile *file)
  *
  * Update or start the application.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_file_read_start(AgsFile *file)
@@ -1504,7 +1504,7 @@ ags_file_write_application_context(AgsFile *file, xmlNode *parent, GObject *appl
  *
  * Returns: a new #AgsFile
  *
- * Since: 0.3
+ * Since: 1.0.0
  */
 AgsFile*
 ags_file_new()
