@@ -121,7 +121,7 @@ ags_turtle_class_init(AgsTurtleClass *turtle)
    *
    * The assigned filename.
    * 
-   * Since: 0.7.0
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_string("filename",
 				   i18n_pspec("filename of turtle"),
@@ -137,7 +137,7 @@ ags_turtle_class_init(AgsTurtleClass *turtle)
    *
    * The assigned xml-doc.
    * 
-   * Since: 0.7.3
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_pointer("xml-doc",
 				    i18n_pspec("xml document of turtle"),
@@ -257,7 +257,7 @@ ags_turtle_finalize(GObject *gobject)
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_iriref(gchar *offset,
@@ -309,7 +309,7 @@ ags_turtle_read_iriref(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_pname_ns(gchar *offset,
@@ -350,7 +350,7 @@ ags_turtle_read_pname_ns(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_pname_ln(gchar *offset,
@@ -398,7 +398,7 @@ ags_turtle_read_pname_ln(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_blank_node_label(gchar *offset,
@@ -503,7 +503,7 @@ ags_turtle_read_blank_node_label(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_langtag(gchar *offset,
@@ -552,7 +552,7 @@ ags_turtle_read_langtag(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_boolean(gchar *offset,
@@ -599,7 +599,7 @@ ags_turtle_read_boolean(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_integer(gchar *offset,
@@ -646,7 +646,7 @@ ags_turtle_read_integer(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_decimal(gchar *offset,
@@ -693,7 +693,7 @@ ags_turtle_read_decimal(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_double(gchar *offset,
@@ -740,7 +740,7 @@ ags_turtle_read_double(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_exponent(gchar *offset,
@@ -787,7 +787,7 @@ ags_turtle_read_exponent(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_string(gchar *offset,
@@ -835,7 +835,7 @@ ags_turtle_read_string(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_string_literal_quote(gchar *offset,
@@ -886,7 +886,7 @@ ags_turtle_read_string_literal_quote(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_string_literal_single_quote(gchar *offset,
@@ -937,7 +937,7 @@ ags_turtle_read_string_literal_single_quote(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_string_literal_long_quote(gchar *offset,
@@ -975,7 +975,7 @@ ags_turtle_read_string_literal_long_quote(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_string_literal_long_single_quote(gchar *offset,
@@ -1013,7 +1013,7 @@ ags_turtle_read_string_literal_long_single_quote(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_uchar(gchar *offset,
@@ -1068,7 +1068,7 @@ ags_turtle_read_uchar(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_echar(gchar *offset,
@@ -1104,7 +1104,7 @@ ags_turtle_read_echar(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_ws(gchar *offset,
@@ -1134,7 +1134,7 @@ ags_turtle_read_ws(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_anon(gchar *offset,
@@ -1220,7 +1220,7 @@ ags_turtle_read_anon(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_pn_chars_base(gchar *offset,
@@ -1274,7 +1274,7 @@ ags_turtle_read_pn_chars_base(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_pn_chars_u(gchar *offset,
@@ -1303,7 +1303,7 @@ ags_turtle_read_pn_chars_u(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_pn_chars(gchar *offset,
@@ -1360,7 +1360,7 @@ ags_turtle_read_pn_chars(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_pn_prefix(gchar *offset,
@@ -1425,7 +1425,7 @@ ags_turtle_read_pn_prefix(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_pn_local(gchar *offset,
@@ -1532,7 +1532,7 @@ ags_turtle_read_pn_local(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_plx(gchar *offset,
@@ -1560,7 +1560,7 @@ ags_turtle_read_plx(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_percent(gchar *offset,
@@ -1591,7 +1591,7 @@ ags_turtle_read_percent(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_hex(gchar *offset,
@@ -1618,7 +1618,7 @@ ags_turtle_read_hex(gchar *offset,
  *
  * Returns: a string on success otherwise %NULL
  * 
- * Since: 0.7.4
+ * Since: 1.0.0
  */
 gchar*
 ags_turtle_read_pn_local_esc(gchar *offset,
@@ -1650,7 +1650,7 @@ ags_turtle_read_pn_local_esc(gchar *offset,
  *
  * Returns: a #GList-struct containing xmlNode
  *
- * Since: 0.7.0
+ * Since: 1.0.0
  */
 GList*
 ags_turtle_find_xpath(AgsTurtle *turtle,
@@ -1695,7 +1695,7 @@ ags_turtle_find_xpath(AgsTurtle *turtle,
  *
  * Returns: a #GList-struct containing xmlNode
  *
- * Since: 0.7.0
+ * Since: 1.0.0
  */
 GList*
 ags_turtle_find_xpath_with_context_node(AgsTurtle *turtle,
@@ -1708,6 +1708,13 @@ ags_turtle_find_xpath_with_context_node(AgsTurtle *turtle,
   GList *list;
 
   guint i;
+
+  if(!AGS_IS_TURTLE(turtle) ||
+     turtle->doc == NULL ||
+     xpath == NULL ||
+     context_node == NULL){
+    return(NULL);
+  }
   
   xpath_context = xmlXPathNewContext(turtle->doc);
   xpath_context->node = context_node;
@@ -1754,7 +1761,7 @@ ags_turtle_string_convert(AgsTurtle *turtle, gchar *str)
  *
  * Returns: a #xmlDoc pointer
  *
- * Since: 0.7.0
+ * Since: 1.0.0
  */
 xmlDoc*
 ags_turtle_load(AgsTurtle *turtle,
@@ -3171,6 +3178,10 @@ ags_turtle_load(AgsTurtle *turtle,
     /* skip blanks and comments */
     iter = ags_turtle_load_skip_comments_and_blanks(&iter);
 
+    if(iter >= &(buffer[sb->st_size])){
+      break;
+    }
+    
     rdf_statement_node = ags_turtle_load_read_statement(&iter);
 
     if(rdf_statement_node != NULL){
@@ -3197,7 +3208,7 @@ ags_turtle_load(AgsTurtle *turtle,
  *
  * Returns: the new instance
  *
- * Since: 0.7.0
+ * Since: 1.0.0
  */
 AgsTurtle*
 ags_turtle_new(gchar *filename)
