@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -74,7 +74,7 @@ ags_soundcard_class_init(AgsSoundcardInterface *interface)
    * The ::tic signal is emitted every tic of the soundcard. This notifies
    * about a newly played buffer.
    *
-   * Since: 0.7.122
+   * Since: 2.0.0
    */
   soundcard_signals[TIC] =
     g_signal_new("tic",
@@ -93,7 +93,7 @@ ags_soundcard_class_init(AgsSoundcardInterface *interface)
    * The ::offset-changed signal notifies about changed position within
    * notation.
    *
-   * Since: 0.7.122
+   * Since: 2.0.0
    */
   soundcard_signals[OFFSET_CHANGED] =
     g_signal_new("offset-changed",
@@ -112,7 +112,7 @@ ags_soundcard_class_init(AgsSoundcardInterface *interface)
    * The ::stop signal is emitted every stop of the soundcard. This notifies
    * about a newly played buffer.
    *
-   * Since: 0.7.122
+   * Since: 2.0.0
    */
   soundcard_signals[STOP] =
     g_signal_new("stop",
@@ -131,7 +131,7 @@ ags_soundcard_class_init(AgsSoundcardInterface *interface)
  *
  * Set application context.
  *
- * Since: 0.4.3
+ * Since: 2.0.0
  */
 void
 ags_soundcard_set_application_context(AgsSoundcard *soundcard,
@@ -154,7 +154,7 @@ ags_soundcard_set_application_context(AgsSoundcard *soundcard,
  *
  * Returns: #AgsApplicationContext
  *
- * Since: 0.4.3
+ * Since: 2.0.0
  */
 AgsApplicationContext*
 ags_soundcard_get_application_context(AgsSoundcard *soundcard)
@@ -168,50 +168,6 @@ ags_soundcard_get_application_context(AgsSoundcard *soundcard)
   return(soundcard_interface->get_application_context(soundcard));
 }
 
-/**
- * ags_soundcard_set_application_mutex:
- * @soundcard: the #AgsSoundcard
- * @application_mutex: the application mutex to set
- *
- * Set application mutex.
- *
- * Since: 0.7.0
- */
-void
-ags_soundcard_set_application_mutex(AgsSoundcard *soundcard,
-				    pthread_mutex_t *application_mutex)
-{
-  AgsSoundcardInterface *soundcard_interface;
-
-  g_return_if_fail(AGS_IS_SOUNDCARD(soundcard));
-  soundcard_interface = AGS_SOUNDCARD_GET_INTERFACE(soundcard);
-  g_return_if_fail(soundcard_interface->set_application_mutex);
-  soundcard_interface->set_application_mutex(soundcard,
-					       application_mutex);
-}
-
-/**
- * ags_soundcard_get_application_mutex:
- * @soundcard: the #AgsSoundcard
- *
- * Get application mutex. 
- *
- * Returns: #AgsApplicationMutex
- *
- * Since: 0.7.0
- */
-pthread_mutex_t*
-ags_soundcard_get_application_mutex(AgsSoundcard *soundcard)
-{
-  AgsSoundcardInterface *soundcard_interface;
-
-  g_return_val_if_fail(AGS_IS_SOUNDCARD(soundcard), NULL);
-  soundcard_interface = AGS_SOUNDCARD_GET_INTERFACE(soundcard);
-  g_return_val_if_fail(soundcard_interface->get_application_mutex, NULL);
-
-  return(soundcard_interface->get_application_mutex(soundcard));
-}
-
 /**
  * ags_soundcard_set_device:
  * @soundcard: the #AgsSoundcard
@@ -219,7 +175,7 @@ ags_soundcard_get_application_mutex(AgsSoundcard *soundcard)
  *
  * Set device.
  *
- * Since: 0.4.3
+ * Since: 2.0.0
  */
 void
 ags_soundcard_set_device(AgsSoundcard *soundcard,
@@ -242,7 +198,7 @@ ags_soundcard_set_device(AgsSoundcard *soundcard,
  *
  * Returns: the device's identifier
  *
- * Since: 0.4.3
+ * Since: 2.0.0
  */
 gchar*
 ags_soundcard_get_device(AgsSoundcard *soundcard)
@@ -266,7 +222,7 @@ ags_soundcard_get_device(AgsSoundcard *soundcard)
  *
  * Get presets. 
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_get_presets(AgsSoundcard *soundcard,
@@ -297,7 +253,7 @@ ags_soundcard_get_presets(AgsSoundcard *soundcard,
  *
  * Set presets. 
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_set_presets(AgsSoundcard *soundcard,
@@ -326,7 +282,7 @@ ags_soundcard_set_presets(AgsSoundcard *soundcard,
  *
  * Retrieve @card_id and @card_name as a list of strings.
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_list_cards(AgsSoundcard *soundcard,
@@ -354,7 +310,7 @@ ags_soundcard_list_cards(AgsSoundcard *soundcard,
  *
  * Retrieve detailed information of @card_id soundcard.
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_pcm_info(AgsSoundcard *soundcard, gchar *card_id,
@@ -375,6 +331,32 @@ ags_soundcard_pcm_info(AgsSoundcard *soundcard, gchar *card_id,
 				error);
 }
 
+/**
+ * ags_soundcard_get_capability:
+ * @soundcard: the #AgsSoundcard
+ *
+ * Retrieve #AgsSoundcardCapability-enum information.
+ *
+ * Returns: the capablities flags
+ * 
+ * Since: 2.0.0
+ */
+guint
+ags_soundcard_get_capability(AgsSoundcard *soundcard)
+{
+  AgsSoundcardInterface *soundcard_interface;
+
+  guint capability;
+  
+  g_return_val_if_fail(AGS_IS_SOUNDCARD(soundcard), 0);
+  soundcard_interface = AGS_SOUNDCARD_GET_INTERFACE(soundcard);
+  g_return_val_if_fail(soundcard_interface->pcm_info, 0);
+
+  capability = soundcard_interface->get_capability(soundcard);
+
+  return(capability);
+}
+
 /**
  * ags_soundcard_get_poll_fd:
  * @soundcard: the #AgsSoundcard
@@ -383,7 +365,7 @@ ags_soundcard_pcm_info(AgsSoundcard *soundcard, gchar *card_id,
  *
  * Returns: a #GList-struct containing poll file descriptors
  *
- * Since: 0.7.50
+ * Since: 2.0.0
  */
 GList*
 ags_soundcard_get_poll_fd(AgsSoundcard *soundcard)
@@ -405,7 +387,7 @@ ags_soundcard_get_poll_fd(AgsSoundcard *soundcard)
  *
  * Returns: %TRUE if available, else %FALSE
  *
- * Since: 0.7.50
+ * Since: 2.0.0
  */
 gboolean
 ags_soundcard_is_available(AgsSoundcard *soundcard)
@@ -427,7 +409,7 @@ ags_soundcard_is_available(AgsSoundcard *soundcard)
  *
  * Returns: %TRUE if starting, else %FALSE
  *
- * Since: 0.4.3
+ * Since: 2.0.0
  */
 gboolean
 ags_soundcard_is_starting(AgsSoundcard *soundcard)
@@ -449,7 +431,7 @@ ags_soundcard_is_starting(AgsSoundcard *soundcard)
  *
  * Returns: %TRUE if playing, else %FALSE
  *
- * Since: 0.4.3
+ * Since: 2.0.0
  */
 gboolean
 ags_soundcard_is_playing(AgsSoundcard *soundcard)
@@ -471,7 +453,7 @@ ags_soundcard_is_playing(AgsSoundcard *soundcard)
  *
  * Returns: %TRUE if recording, else %FALSE
  *
- * Since: 0.7.0
+ * Since: 2.0.0
  */
 gboolean
 ags_soundcard_is_recording(AgsSoundcard *soundcard)
@@ -493,7 +475,7 @@ ags_soundcard_is_recording(AgsSoundcard *soundcard)
  *
  * Returns: playback time as string
  *
- * Since: 0.7.53
+ * Since: 2.0.0
  */
 gchar*
 ags_soundcard_get_uptime(AgsSoundcard *soundcard)
@@ -514,7 +496,7 @@ ags_soundcard_get_uptime(AgsSoundcard *soundcard)
  *
  * Initializes the soundcard for playback.
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_play_init(AgsSoundcard *soundcard,
@@ -536,7 +518,7 @@ ags_soundcard_play_init(AgsSoundcard *soundcard,
  *
  * Plays the current buffer of soundcard.
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_play(AgsSoundcard *soundcard,
@@ -558,7 +540,7 @@ ags_soundcard_play(AgsSoundcard *soundcard,
  *
  * Initializes the soundcard for recordback.
  *
- * Since: 0.7.0
+ * Since: 2.0.0
  */
 void
 ags_soundcard_record_init(AgsSoundcard *soundcard,
@@ -580,7 +562,7 @@ ags_soundcard_record_init(AgsSoundcard *soundcard,
  *
  * Records the current buffer of soundcard.
  *
- * Since: 0.7.0
+ * Since: 2.0.0
  */
 void
 ags_soundcard_record(AgsSoundcard *soundcard,
@@ -601,7 +583,7 @@ ags_soundcard_record(AgsSoundcard *soundcard,
  *
  * Stops the soundcard from playing to it.
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_stop(AgsSoundcard *soundcard)
@@ -617,7 +599,7 @@ ags_soundcard_stop(AgsSoundcard *soundcard)
  *
  * Every call to play may generate a tic. 
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_tic(AgsSoundcard *soundcard)
@@ -632,7 +614,7 @@ ags_soundcard_tic(AgsSoundcard *soundcard)
  *
  * Callback when counter expires minor note offset.
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_offset_changed(AgsSoundcard *soundcard,
@@ -652,7 +634,7 @@ ags_soundcard_offset_changed(AgsSoundcard *soundcard,
  *
  * Returns: current playback buffer
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void*
 ags_soundcard_get_buffer(AgsSoundcard *soundcard)
@@ -674,7 +656,7 @@ ags_soundcard_get_buffer(AgsSoundcard *soundcard)
  *
  * Returns: next playback buffer
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void*
 ags_soundcard_get_next_buffer(AgsSoundcard *soundcard)
@@ -696,7 +678,7 @@ ags_soundcard_get_next_buffer(AgsSoundcard *soundcard)
  *
  * Returns: prev playback buffer
  *
- * Since: 0.9.17
+ * Since: 2.0.0
  */
 void*
 ags_soundcard_get_prev_buffer(AgsSoundcard *soundcard)
@@ -710,6 +692,52 @@ ags_soundcard_get_prev_buffer(AgsSoundcard *soundcard)
   return(soundcard_interface->get_prev_buffer(soundcard));
 }
 
+/**
+ * ags_soundcard_lock_buffer:
+ * @soundcard: the #AgsSoundcard
+ * @buffer: the buffer to lock
+ *
+ * Lock @buffer. 
+ *
+ * Since: 2.0.0
+ */
+void
+ags_soundcard_lock_buffer(AgsSoundcard *soundcard,
+			  void *buffer)
+{
+  AgsSoundcardInterface *soundcard_interface;
+
+  g_return_if_fail(AGS_IS_SOUNDCARD(soundcard));
+  soundcard_interface = AGS_SOUNDCARD_GET_INTERFACE(soundcard);
+  g_return_if_fail(soundcard_interface->lock_buffer);
+
+  soundcard_interface->lock_buffer(soundcard,
+				   buffer);
+}
+
+/**
+ * ags_soundcard_unlock_buffer:
+ * @soundcard: the #AgsSoundcard
+ * @buffer: the buffer to unlock
+ *
+ * Unlock @buffer. 
+ *
+ * Since: 2.0.0
+ */
+void
+ags_soundcard_unlock_buffer(AgsSoundcard *soundcard,
+			    void *buffer)
+{
+  AgsSoundcardInterface *soundcard_interface;
+
+  g_return_if_fail(AGS_IS_SOUNDCARD(soundcard));
+  soundcard_interface = AGS_SOUNDCARD_GET_INTERFACE(soundcard);
+  g_return_if_fail(soundcard_interface->unlock_buffer);
+
+  soundcard_interface->unlock_buffer(soundcard,
+				     buffer);
+}
+
 /**
  * ags_soundcard_set_bpm:
  * @soundcard: the #AgsSoundcard
@@ -717,7 +745,7 @@ ags_soundcard_get_prev_buffer(AgsSoundcard *soundcard)
  *
  * Set current playback bpm. 
  *
- * Since: 0.4.3
+ * Since: 2.0.0
  */
 void
 ags_soundcard_set_bpm(AgsSoundcard *soundcard,
@@ -740,7 +768,7 @@ ags_soundcard_set_bpm(AgsSoundcard *soundcard,
  *
  * Returns: bpm
  *
- * Since: 0.4.3
+ * Since: 2.0.0
  */
 gdouble
 ags_soundcard_get_bpm(AgsSoundcard *soundcard)
@@ -761,7 +789,7 @@ ags_soundcard_get_bpm(AgsSoundcard *soundcard)
  *
  * Set current playback delay factor. 
  *
- * Since: 0.7.0
+ * Since: 2.0.0
  */
 void
 ags_soundcard_set_delay_factor(AgsSoundcard *soundcard,
@@ -784,7 +812,7 @@ ags_soundcard_set_delay_factor(AgsSoundcard *soundcard,
  *
  * Returns: delay factor
  *
- * Since: 0.7.0
+ * Since: 2.0.0
  */
 gdouble
 ags_soundcard_get_delay_factor(AgsSoundcard *soundcard)
@@ -806,7 +834,7 @@ ags_soundcard_get_delay_factor(AgsSoundcard *soundcard)
  *
  * Returns: delay
  *
- * Since: 0.7.100
+ * Since: 2.0.0
  */
 gdouble
 ags_soundcard_get_absolute_delay(AgsSoundcard *soundcard)
@@ -828,7 +856,7 @@ ags_soundcard_get_absolute_delay(AgsSoundcard *soundcard)
  *
  * Returns: delay
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 gdouble
 ags_soundcard_get_delay(AgsSoundcard *soundcard)
@@ -850,7 +878,7 @@ ags_soundcard_get_delay(AgsSoundcard *soundcard)
  *
  * Returns: attack
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 guint
 ags_soundcard_get_attack(AgsSoundcard *soundcard)
@@ -872,7 +900,7 @@ ags_soundcard_get_attack(AgsSoundcard *soundcard)
  *
  * Returns: offset
  *
- * Since: 0.7.6
+ * Since: 2.0.0
  */
 guint
 ags_soundcard_get_delay_counter(AgsSoundcard *soundcard)
@@ -893,7 +921,7 @@ ags_soundcard_get_delay_counter(AgsSoundcard *soundcard)
  *
  * Set current playback note offset. 
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 void
 ags_soundcard_set_note_offset(AgsSoundcard *soundcard,
@@ -916,7 +944,7 @@ ags_soundcard_set_note_offset(AgsSoundcard *soundcard,
  *
  * Returns: offset
  *
- * Since: 0.4.2
+ * Since: 2.0.0
  */
 guint
 ags_soundcard_get_note_offset(AgsSoundcard *soundcard)
@@ -937,7 +965,7 @@ ags_soundcard_get_note_offset(AgsSoundcard *soundcard)
  *
  * Set current playback note offset. 
  *
- * Since: 0.7.103
+ * Since: 2.0.0
  */
 void
 ags_soundcard_set_note_offset_absolute(AgsSoundcard *soundcard,
@@ -960,7 +988,7 @@ ags_soundcard_set_note_offset_absolute(AgsSoundcard *soundcard,
  *
  * Returns: offset
  *
- * Since: 0.7.103
+ * Since: 2.0.0
  */
 guint
 ags_soundcard_get_note_offset_absolute(AgsSoundcard *soundcard)
@@ -983,7 +1011,7 @@ ags_soundcard_get_note_offset_absolute(AgsSoundcard *soundcard)
  *
  * Set loop parameters of @soundcard.
  *
- * Since: 0.7.35
+ * Since: 2.0.0
  */
 void
 ags_soundcard_set_loop(AgsSoundcard *soundcard,
@@ -1009,7 +1037,7 @@ ags_soundcard_set_loop(AgsSoundcard *soundcard,
  *
  * Get loop parameters of @soundcard.
  * 
- * Since: 0.7.35
+ * Since: 2.0.0
  */
 void
 ags_soundcard_get_loop(AgsSoundcard *soundcard,
@@ -1034,7 +1062,7 @@ ags_soundcard_get_loop(AgsSoundcard *soundcard,
  *
  * Returns: offset
  *
- * Since: 0.7.35
+ * Since: 2.0.0
  */
 guint
 ags_soundcard_get_loop_offset(AgsSoundcard *soundcard)
@@ -1047,47 +1075,3 @@ ags_soundcard_get_loop_offset(AgsSoundcard *soundcard)
 
   return(soundcard_interface->get_loop_offset(soundcard));
 }
-
-/**
- * ags_soundcard_set_audio:
- * @soundcard: the #AgsSoundcard
- * @audio: the audio as #GList-struct to set
- *
- * Set audio. 
- *
- * Since: 0.4.3
- */
-void
-ags_soundcard_set_audio(AgsSoundcard *soundcard,
-			GList *audio)
-{
-  AgsSoundcardInterface *soundcard_interface;
-
-  g_return_if_fail(AGS_IS_SOUNDCARD(soundcard));
-  soundcard_interface = AGS_SOUNDCARD_GET_INTERFACE(soundcard);
-  g_return_if_fail(soundcard_interface->set_audio);
-  soundcard_interface->set_audio(soundcard,
-				 audio);
-}
-
-/**
- * ags_soundcard_get_audio:
- * @soundcard: the #AgsSoundcard
- *
- * Get audio. 
- *
- * Returns: a #GList-struct container #AgsAudio
- *
- * Since: 0.4.3
- */
-GList*
-ags_soundcard_get_audio(AgsSoundcard *soundcard)
-{
-  AgsSoundcardInterface *soundcard_interface;
-
-  g_return_val_if_fail(AGS_IS_SOUNDCARD(soundcard), NULL);
-  soundcard_interface = AGS_SOUNDCARD_GET_INTERFACE(soundcard);
-  g_return_val_if_fail(soundcard_interface->get_audio, NULL);
-
-  return(soundcard_interface->get_audio(soundcard));
-}
