@@ -19,6 +19,8 @@
 
 #include <ags/server/controller/ags_front_controller.h>
 
+#include <ags/util/ags_list_util.h>
+
 #include <ags/object/ags_application_context.h>
 #include <ags/object/ags_connectable.h>
 #include <ags/object/ags_marshal.h>
@@ -32,6 +34,8 @@
 
 #include <ags/server/controller/ags_local_factory_controller.h>
 
+#include <string.h>
+
 #ifdef AGS_WITH_XMLRPC_C
 #include <xmlrpc-c/util.h>
 
@@ -173,7 +177,7 @@ ags_front_controller_class_init(AgsFrontControllerClass *front_controller)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsFrontControllerClass, authenticate),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__STRING_STRING_STRING_STRING,
+		 ags_cclosure_marshal_POINTER__STRING_STRING_STRING_STRING,
 		 G_TYPE_POINTER, 4,
 		 G_TYPE_STRING,
 		 G_TYPE_STRING,
@@ -201,7 +205,7 @@ ags_front_controller_class_init(AgsFrontControllerClass *front_controller)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsFrontControllerClass, do_request),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__OBJECT_STRING_STRING_STRING_POINTER,
+		 ags_cclosure_marshal_POINTER__OBJECT_STRING_STRING_STRING_POINTER,
 		 G_TYPE_POINTER, 5,
 		 G_TYPE_OBJECT,
 		 G_TYPE_STRING,
@@ -562,14 +566,17 @@ ags_front_controller_real_authenticate(AgsFrontController *front_controller,
   g_free(context_path);
 
   //TODO:JK: implement me
+
+  return(NULL);
 }
 
 /**
  * ags_front_controller_authenticate:
  * @front_controller: the #AgsFrontController
- * @login:
- * @password:
- * @certs:
+ * @authentication_module: the authentication module to use
+ * @login: the login
+ * @password: the password
+ * @certs: the certs data
  *
  * Authenticate to the server.
  *
@@ -688,6 +695,7 @@ ags_front_controller_real_do_request(AgsFrontController *front_controller,
   response = NULL;
   
   if(local_factory_controller != NULL &&
+     AGS_CONTROLLER(local_factory_controller)->context_path != NULL &&
      !g_ascii_strncasecmp(context_path,
 			  AGS_CONTROLLER(local_factory_controller)->context_path,
 			  strlen(AGS_CONTROLLER(local_factory_controller)->context_path))){
@@ -713,7 +721,6 @@ ags_front_controller_real_do_request(AgsFrontController *front_controller,
  * @context_path: the context path to access
  * @login: the login
  * @security_token: the security token
- * @certs: the certs
  * @params: the #GParameter-struct containing parameters
  * @n_params: the count of @params
  *
