@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,11 +19,11 @@
 
 #include <ags/thread/ags_thread-posix.h>
 
-#include <ags/object/ags_connectable.h>
-
-#include <ags/object/ags_marshal.h>
 #include <ags/object/ags_main_loop.h>
+#include <ags/object/ags_config.h>
+#include <ags/object/ags_connectable.h>
 #include <ags/object/ags_async_queue.h>
+#include <ags/object/ags_marshal.h>
 
 #include <ags/thread/ags_mutex_manager.h>
 #include <ags/thread/ags_condition_manager.h>
@@ -34,6 +34,11 @@
 #include <unistd.h>
 #include <math.h>
 
+#ifdef __APPLE__
+#include <mach/clock.h>
+#include <mach/mach.h>
+#endif
+
 #include <ags/i18n.h>
 
 #include <errno.h>
@@ -82,6 +87,7 @@ static void ags_thread_self_create();
 enum{
   PROP_0,
   PROP_FREQUENCY,
+  PROP_MAX_PRECISION,
 };
 
 enum{
@@ -105,7 +111,7 @@ static pthread_key_t ags_thread_key;
 /* Once-only initialisation of the key */
 static pthread_once_t ags_thread_key_once = PTHREAD_ONCE_INIT;
 
-static pthread_mutex_t class_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t ags_thread_class_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 GType
 ags_thread_get_type()
@@ -167,14 +173,32 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The frequency to run at in Hz.
    * 
-   * Since: 0.4.0
+   * Since: 1.0.0
    */
   param_spec = g_param_spec_double("frequency",
-				   i18n_pspec("JIFFIE"),
-				   i18n_pspec("JIFFIE"),
+				   i18n_pspec("frequency as JIFFIE"),
+				   i18n_pspec("frequency as JIFFIE"),
 				   0.01,
-				   1000.0,
-				   1000.0,
+				   AGS_THREAD_DEFAULT_MAX_PRECISION,
+				   AGS_THREAD_DEFAULT_JIFFIE,
+				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_FREQUENCY,
+				  param_spec);
+  
+  /**
+   * AgsThread:max-precision:
+   *
+   * The max-frequency to run at in Hz.
+   * 
+   * Since: 1.0.0
+   */
+  param_spec = g_param_spec_double("max-precision",
+				   i18n_pspec("max precision as JIFFIE"),
+				   i18n_pspec("The max precision as JIFFIE"),
+				   0.01,
+				   AGS_THREAD_MAX_PRECISION,
+				   AGS_THREAD_DEFAULT_MAX_PRECISION,
 				   G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_FREQUENCY,
@@ -200,7 +224,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * Returns: the number of cycles to perform
    * 
-   * Since: 0.6.0
+   * Since: 1.0.0
    */
   thread_signals[CLOCK] =
     g_signal_new("clock",
@@ -208,7 +232,7 @@ ags_thread_class_init(AgsThreadClass *thread)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (AgsThreadClass, clock),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_UINT__VOID,
+		 ags_cclosure_marshal_UINT__VOID,
 		 G_TYPE_UINT, 0);
 
 
@@ -218,7 +242,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::start() signal is invoked as thread started.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   thread_signals[START] =
     g_signal_new("start",
@@ -235,7 +259,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::run() signal is invoked during run loop.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   thread_signals[RUN] =
     g_signal_new("run",
@@ -252,7 +276,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::suspend() signal is invoked during suspending.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   thread_signals[SUSPEND] =
     g_signal_new("suspend",
@@ -269,7 +293,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::resume() signal is invoked during resuming.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   thread_signals[RESUME] =
     g_signal_new("resume",
@@ -287,7 +311,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    * The ::timelock() signal is invoked as standard compution
    * time exceeded.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   thread_signals[TIMELOCK] =
     g_signal_new("timelock",
@@ -304,7 +328,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::stop() signal is invoked as @thread stopped.
    * 
-   * Since: 0.5.0
+   * Since: 1.0.0
    */
   thread_signals[STOP] =
     g_signal_new("stop",
@@ -326,7 +350,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * Returns: the time spent
    *
-   * Since: 0.7.46
+   * Since: 1.0.0
    */
   thread_signals[INTERRUPTED] =
     g_signal_new("interrupted",
@@ -334,7 +358,7 @@ ags_thread_class_init(AgsThreadClass *thread)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (AgsThreadClass, interrupted),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_UINT__INT_UINT_POINTER,
+		 ags_cclosure_marshal_UINT__INT_UINT_POINTER,
 		 G_TYPE_UINT, 3,
 		 G_TYPE_INT, G_TYPE_UINT, G_TYPE_POINTER);
 }
@@ -353,6 +377,10 @@ ags_thread_init(AgsThread *thread)
 {
   AgsMutexManager *mutex_manager;
   AgsConditionManager *condition_manager;
+
+  AgsConfig *config;
+  
+  gchar *str;
   
   pthread_mutex_t *application_mutex;
   pthread_mutex_t *mutex;
@@ -360,8 +388,10 @@ ags_thread_init(AgsThread *thread)
   pthread_mutexattr_t *attr;
 
   int err;
-  
-  /* insert audio loop mutex */
+
+  config = ags_config_get_instance();
+
+  /* insert thread mutex */
   thread->obj_mutexattr =
     attr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
 
@@ -437,6 +467,18 @@ ags_thread_init(AgsThread *thread)
 
   thread->freq = AGS_THREAD_DEFAULT_JIFFIE;
 
+  thread->max_precision = AGS_THREAD_DEFAULT_MAX_PRECISION;
+
+  /* max precision */
+  str = ags_config_get_value(config,
+			     AGS_CONFIG_THREAD,
+			     "max-precision");
+  
+  if(str != NULL){
+    thread->max_precision = g_ascii_strtod(str,
+					   NULL);
+  }
+  
   thread->mutexattr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
 
   pthread_mutexattr_init(thread->mutexattr);
@@ -470,10 +512,14 @@ ags_thread_init(AgsThread *thread)
 		   FALSE);
 
   /* barrier */
+#ifndef __APPLE__
   thread->barrier = (pthread_barrier_t **) malloc(2 * sizeof(pthread_barrier_t *));
   thread->barrier[0] = (pthread_barrier_t *) malloc(sizeof(pthread_barrier_t));
   thread->barrier[1] = (pthread_barrier_t *) malloc(sizeof(pthread_barrier_t));
-  
+#else
+  thread->barrier = NULL;
+#endif
+
   thread->first_barrier = TRUE;
   thread->wait_count[0] = 1;
   thread->wait_count[1] = 1;
@@ -552,7 +598,35 @@ ags_thread_set_property(GObject *gobject,
       }
 
       thread->freq = freq;
-      thread->delay = (guint) ceil((AGS_THREAD_HERTZ_JIFFIE / thread->freq) / (AGS_THREAD_HERTZ_JIFFIE / AGS_THREAD_MAX_PRECISION));
+      thread->delay = (guint) ceil((AGS_THREAD_HERTZ_JIFFIE / thread->freq) / (AGS_THREAD_HERTZ_JIFFIE / thread->max_precision));
+      thread->tic_delay = 0;
+
+      if((AGS_THREAD_INTERMEDIATE_POST_SYNC & (g_atomic_int_get(&(thread->flags)))) != 0){
+	thread->tic_delay = thread->delay;
+      }else if((AGS_THREAD_INTERMEDIATE_PRE_SYNC & (g_atomic_int_get(&(thread->flags)))) != 0){
+	thread->tic_delay = 1;
+      }else{
+	thread->tic_delay = 0;
+      }
+    }
+    break;
+  case PROP_MAX_PRECISION:
+    {
+      gdouble max_precision;
+      gdouble old_max_precision;
+
+      max_precision = g_value_get_double(value);
+
+      if(max_precision == thread->max_precision){
+	return;
+      }
+
+      old_max_precision = thread->max_precision;
+      
+      thread->max_precision = max_precision;
+
+      thread->freq = thread->freq / old_max_precision * max_precision;
+      thread->delay = (guint) ceil((AGS_THREAD_HERTZ_JIFFIE / thread->freq) / (AGS_THREAD_HERTZ_JIFFIE / thread->max_precision));
       thread->tic_delay = 0;
 
       if((AGS_THREAD_INTERMEDIATE_POST_SYNC & (g_atomic_int_get(&(thread->flags)))) != 0){
@@ -586,6 +660,11 @@ ags_thread_get_property(GObject *gobject,
       g_value_set_double(value, thread->freq);
     }
     break;
+  case PROP_MAX_PRECISION:
+    {
+      g_value_set_double(value, thread->max_precision);
+    }
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, param_spec);
     break;
@@ -690,11 +769,15 @@ ags_thread_finalize(GObject *gobject)
   free(thread->computing_time);
 
   /*  */
-  free(thread->barrier[0]);
-  free(thread->barrier[1]);
-
-  free(thread->barrier);
+  if(thread->barrier != NULL){
+#ifndef __APPLE__
+    free(thread->barrier[0]);
+    free(thread->barrier[1]);
     
+    free(thread->barrier);
+#endif
+  }
+
   /*  */
   pthread_attr_getstack(thread_attr,
 			&stackaddr, &stacksize);
@@ -756,7 +839,7 @@ ags_thread_finalize(GObject *gobject)
   free(thread_ptr);
 
   if(stackaddr != NULL){
-    free(stackaddr);
+    //   free(stackaddr);
   }
   
   if(do_exit){
@@ -764,6 +847,147 @@ ags_thread_finalize(GObject *gobject)
   }
 }
 
+/**
+ * ags_thread_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_thread_get_class_mutex()
+{
+  return(&ags_thread_class_mutex);
+}
+
+/**
+ * ags_thread_test_flags:
+ * @thread: the #AgsThread
+ * @flags: the flags
+ *
+ * Test @flags to be set on @thread.
+ * 
+ * Returns: %TRUE if flags are set, else %FALSE
+ *
+ * Since: 2.0.0
+ */
+gboolean
+ags_thread_test_flags(AgsThread *thread, guint flags)
+{
+  gboolean retval;  
+
+  if(!AGS_IS_THREAD(thread)){
+    return(FALSE);
+  }
+  
+  retval = ((flags & (g_atomic_int_get(&(thread->flags)))) != 0) ? TRUE: FALSE;
+    
+  return(retval);
+}
+
+/**
+ * ags_thread_set_flags:
+ * @thread: the #AgsThread
+ * @flags: the flags
+ *
+ * Set flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_thread_set_flags(AgsThread *thread, guint flags)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return;
+  }
+
+  g_atomic_int_or(&(thread->flags), flags);
+}
+
+/**
+ * ags_thread_unset_flags:
+ * @thread: the #AgsThread
+ * @flags: the flags
+ *
+ * Unset flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_thread_unset_flags(AgsThread *thread, guint flags)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return;
+  }
+
+  g_atomic_int_and(&(thread->flags), (~flags));
+}
+
+/**
+ * ags_thread_test_sync_flags:
+ * @thread: the #AgsThread
+ * @sync_flags: the syn flags
+ *
+ * Test @sync_flags to be set on @thread.
+ * 
+ * Returns: %TRUE if sync flags are set, else %FALSE
+ *
+ * Since: 2.0.0
+ */
+gboolean
+ags_thread_test_sync_flags(AgsThread *thread, guint sync_flags)
+{
+  gboolean retval;  
+
+  if(!AGS_IS_THREAD(thread)){
+    return(FALSE);
+  }
+  
+  retval = ((sync_flags & (g_atomic_int_get(&(thread->sync_flags)))) != 0) ? TRUE: FALSE;
+    
+  return(retval);
+}
+
+/**
+ * ags_thread_set_sync_flags:
+ * @thread: the #AgsThread
+ * @sync_flags: the sync flags
+ *
+ * Set sync flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_thread_set_sync_flags(AgsThread *thread, guint sync_flags)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return;
+  }
+
+  g_atomic_int_or(&(thread->sync_flags), sync_flags);
+}
+
+/**
+ * ags_thread_unset_sync_flags:
+ * @thread: the #AgsThread
+ * @sync_flags: the sync flags
+ *
+ * Unset sync flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_thread_unset_sync_flags(AgsThread *thread, guint sync_flags)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return;
+  }
+
+  g_atomic_int_and(&(thread->sync_flags), (~sync_flags));
+}
+
 void
 ags_thread_resume_handler(int sig)
 {
@@ -810,7 +1034,7 @@ ags_thread_suspend_handler(int sig)
  * Unsets AGS_THREAD_WAIT_0, AGS_THREAD_WAIT_1 or AGS_THREAD_WAIT_2.
  * Additionaly the thread is woken up by this function if waiting.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_set_sync(AgsThread *thread, guint tic)
@@ -928,7 +1152,7 @@ ags_thread_set_sync(AgsThread *thread, guint tic)
  * 
  * Calls ags_thread_set_sync() on all threads.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_set_sync_all(AgsThread *thread, guint tic)
@@ -955,7 +1179,7 @@ ags_thread_set_sync_all(AgsThread *thread, guint tic)
     }
     
     /* increment delay counter and set run per cycle */
-    if(thread->freq >= AGS_THREAD_MAX_PRECISION){
+    if(thread->freq >= thread->max_precision){
       thread->tic_delay = 0;
     }else{
       if(thread->tic_delay < thread->delay){
@@ -994,7 +1218,7 @@ ags_thread_set_sync_all(AgsThread *thread, guint tic)
  * Reset all threads. E.g. suspended threads. A synchronization
  * stage after #AgsAsyncQueue run.
  * 
- * Since: 0.7.109
+ * Since: 1.0.0
  */
 void
 ags_thread_reset_all(AgsThread *thread)
@@ -1068,7 +1292,7 @@ ags_thread_reset_all(AgsThread *thread)
  * 
  * Locks the threads own mutex and sets the appropriate flag.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_lock(AgsThread *thread)
@@ -1109,7 +1333,7 @@ ags_thread_lock(AgsThread *thread)
  *
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 gboolean
 ags_thread_trylock(AgsThread *thread)
@@ -1151,7 +1375,7 @@ ags_thread_trylock(AgsThread *thread)
  *
  * Unlocks the threads own mutex and unsets the appropriate flag.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_unlock(AgsThread *thread)
@@ -1191,7 +1415,7 @@ ags_thread_unlock(AgsThread *thread)
  *
  * Returns: the toplevevel #AgsThread
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 AgsThread*
 ags_thread_get_toplevel(AgsThread *thread)
@@ -1215,7 +1439,7 @@ ags_thread_get_toplevel(AgsThread *thread)
  *
  * Returns: the very first #AgsThread within same tree level
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 AgsThread*
 ags_thread_first(AgsThread *thread)
@@ -1239,7 +1463,7 @@ ags_thread_first(AgsThread *thread)
  *
  * Returns: the very last @AgsThread within same tree level
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 AgsThread*
 ags_thread_last(AgsThread *thread)
@@ -1262,7 +1486,7 @@ ags_thread_last(AgsThread *thread)
  * 
  * Remove child of thread.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_remove_child(AgsThread *thread, AgsThread *child)
@@ -1314,7 +1538,7 @@ ags_thread_remove_child(AgsThread *thread, AgsThread *child)
  * 
  * Add child to thread.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_add_child(AgsThread *thread, AgsThread *child)
@@ -1332,7 +1556,7 @@ ags_thread_add_child(AgsThread *thread, AgsThread *child)
  * 
  * Add child to thread.
  *
- * Since: 0.4.2
+ * Since: 1.0.0
  */
 void
 ags_thread_add_child_extended(AgsThread *thread, AgsThread *child,
@@ -1417,7 +1641,7 @@ ags_thread_add_child_extended(AgsThread *thread, AgsThread *child,
  *
  * Returns: %TRUE if locked otherwise %FALSE
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 gboolean
 ags_thread_parental_is_locked(AgsThread *thread, AgsThread *parent)
@@ -1450,7 +1674,7 @@ ags_thread_parental_is_locked(AgsThread *thread, AgsThread *parent)
  *
  * Returns: %TRUE if locked otherwise %FALSE
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 gboolean
 ags_thread_sibling_is_locked(AgsThread *thread)
@@ -1484,7 +1708,7 @@ ags_thread_sibling_is_locked(AgsThread *thread)
  *
  * Returns: %TRUE if locked otherwise %FALSE
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 gboolean
 ags_thread_children_is_locked(AgsThread *thread)
@@ -1719,7 +1943,7 @@ ags_thread_is_tree_ready(AgsThread *thread,
  *
  * Returns: next matching #AgsThread
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 AgsThread*
 ags_thread_next_parent_locked(AgsThread *thread, AgsThread *parent)
@@ -1751,7 +1975,7 @@ ags_thread_next_parent_locked(AgsThread *thread, AgsThread *parent)
  *
  * Returns: next matching #AgsThread
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 AgsThread*
 ags_thread_next_sibling_locked(AgsThread *thread)
@@ -1789,7 +2013,7 @@ ags_thread_next_sibling_locked(AgsThread *thread)
  *
  * Returns: next matching #AgsThread
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 AgsThread*
 ags_thread_next_children_locked(AgsThread *thread)
@@ -1832,7 +2056,7 @@ ags_thread_next_children_locked(AgsThread *thread)
  *
  * Lock parent tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_lock_parent(AgsThread *thread, AgsThread *parent)
@@ -1862,7 +2086,7 @@ ags_thread_lock_parent(AgsThread *thread, AgsThread *parent)
  *
  * Lock sibling tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_lock_sibling(AgsThread *thread)
@@ -1898,7 +2122,7 @@ ags_thread_lock_sibling(AgsThread *thread)
  *
  * Lock child tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_lock_children(AgsThread *thread)
@@ -1941,7 +2165,7 @@ ags_thread_lock_all(AgsThread *thread)
  *
  * Unlock parent tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_unlock_parent(AgsThread *thread, AgsThread *parent)
@@ -1978,7 +2202,7 @@ ags_thread_unlock_parent(AgsThread *thread, AgsThread *parent)
  *
  * Unlock sibling tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_unlock_sibling(AgsThread *thread)
@@ -2021,7 +2245,7 @@ ags_thread_unlock_sibling(AgsThread *thread)
  *
  * Unlock child tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_unlock_children(AgsThread *thread)
@@ -2083,7 +2307,7 @@ ags_thread_unlock_all(AgsThread *thread)
  *
  * Wait on parent tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_wait_parent(AgsThread *thread, AgsThread *parent)
@@ -2121,7 +2345,7 @@ ags_thread_wait_parent(AgsThread *thread, AgsThread *parent)
  *
  * Wait on sibling tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_wait_sibling(AgsThread *thread)
@@ -2165,7 +2389,7 @@ ags_thread_wait_sibling(AgsThread *thread)
  *
  * Wait on child tree structure.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_wait_children(AgsThread *thread)
@@ -2223,7 +2447,7 @@ ags_thread_wait_children(AgsThread *thread)
  *
  * Signals the tree in higher levels.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_signal_parent(AgsThread *thread, AgsThread *parent,
@@ -2253,7 +2477,7 @@ ags_thread_signal_parent(AgsThread *thread, AgsThread *parent,
  *
  * Signals the tree on same level.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_signal_sibling(AgsThread *thread, gboolean broadcast)
@@ -2282,7 +2506,7 @@ ags_thread_signal_sibling(AgsThread *thread, gboolean broadcast)
  *
  * Signals the tree in lower levels.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_signal_children(AgsThread *thread, gboolean broadcast)
@@ -2321,6 +2545,11 @@ ags_thread_real_clock(AgsThread *thread)
 {
   AgsThread *main_loop, *async_queue;
   AgsMutexManager *mutex_manager;
+
+#ifdef __APPLE__
+  clock_serv_t cclock;
+  mach_timespec_t mts;
+#endif
   
   struct timespec time_now;
 
@@ -2356,7 +2585,7 @@ ags_thread_real_clock(AgsThread *thread)
 	thread->tic_delay = chaos_tree->tic_delay;
 
 	if((AGS_THREAD_INTERMEDIATE_PRE_SYNC & (g_atomic_int_get(&(thread->flags)))) != 0){
-	  if(thread->freq >= AGS_THREAD_MAX_PRECISION){
+	  if(thread->freq >= thread->max_precision){
 	    thread->tic_delay = 0;
 	  }else{
 	    if(thread->tic_delay < thread->delay){
@@ -2368,7 +2597,7 @@ ags_thread_real_clock(AgsThread *thread)
 	}
 	
 	if((AGS_THREAD_INTERMEDIATE_POST_SYNC & (g_atomic_int_get(&(thread->flags)))) != 0){
-	  if(thread->freq >= AGS_THREAD_MAX_PRECISION){
+	  if(thread->freq >= thread->max_precision){
 	    thread->tic_delay = 0;
 	  }else{
 	    if(thread->tic_delay > 0){
@@ -2507,7 +2736,7 @@ ags_thread_real_clock(AgsThread *thread)
 
   if(!AGS_IS_MAIN_LOOP(main_loop)){
     if(thread->tic_delay == 0){
-      if(thread->freq >= AGS_THREAD_MAX_PRECISION){
+      if(thread->freq >= thread->max_precision){
 	steps = 1.0 / thread->delay;
       }else{
 	steps = 1;
@@ -2550,15 +2779,15 @@ ags_thread_real_clock(AgsThread *thread)
   /* calculate main loop delay */
   pthread_mutex_lock(main_loop_mutex);
   
-  main_loop_delay = AGS_THREAD_MAX_PRECISION / main_loop->freq / (AGS_THREAD_HERTZ_JIFFIE / AGS_THREAD_MAX_PRECISION);
+  main_loop_delay = main_loop->max_precision / main_loop->freq / (AGS_THREAD_HERTZ_JIFFIE / main_loop->max_precision);
   
   pthread_mutex_unlock(main_loop_mutex);
   
   /* calculate thread delay */
   pthread_mutex_lock(mutex);
 
-  thread->delay = (guint) ceil((AGS_THREAD_HERTZ_JIFFIE / thread->freq) / (AGS_THREAD_HERTZ_JIFFIE / AGS_THREAD_MAX_PRECISION));
-  delay_per_hertz = AGS_THREAD_HERTZ_JIFFIE / AGS_THREAD_MAX_PRECISION;
+  thread->delay = (guint) ceil((AGS_THREAD_HERTZ_JIFFIE / thread->freq) / (AGS_THREAD_HERTZ_JIFFIE / thread->max_precision));
+  delay_per_hertz = AGS_THREAD_HERTZ_JIFFIE / thread->max_precision;
 
   if(thread->delay < thread->tic_delay){
     thread->tic_delay = thread->delay;
@@ -2574,7 +2803,17 @@ ags_thread_real_clock(AgsThread *thread)
   if((AGS_THREAD_IMMEDIATE_SYNC & (g_atomic_int_get(&(thread->flags)))) == 0){
     if((AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(thread->flags)))) != 0 &&
        (AGS_THREAD_INITIAL_SYNC & (g_atomic_int_get(&(thread->flags)))) != 0){
+#ifdef __APPLE__
+      host_get_clock_service(mach_host_self(), CALENDAR_CLOCK, &cclock);
+      
+      clock_get_time(cclock, &mts);
+      mach_port_deallocate(mach_task_self(), cclock);
+      
+      thread->computing_time->tv_sec = mts.tv_sec;
+      thread->computing_time->tv_nsec = mts.tv_nsec;
+#else
       clock_gettime(CLOCK_MONOTONIC, thread->computing_time);
+#endif
 
       g_atomic_int_and(&(thread->flags),
 		       (~AGS_THREAD_INITIAL_SYNC));
@@ -2623,7 +2862,17 @@ ags_thread_real_clock(AgsThread *thread)
       0,
     };
 
+#ifdef __APPLE__
+    host_get_clock_service(mach_host_self(), CALENDAR_CLOCK, &cclock);
+    
+    clock_get_time(cclock, &mts);
+    mach_port_deallocate(mach_task_self(), cclock);
+    
+    time_now.tv_sec = mts.tv_sec;
+    time_now.tv_nsec = mts.tv_nsec;
+#else
     clock_gettime(CLOCK_MONOTONIC, &time_now);
+#endif
       
     if(time_now.tv_sec == thread->computing_time->tv_sec + 1){
       time_spent = (time_now.tv_nsec) + (NSEC_PER_SEC - thread->computing_time->tv_nsec);
@@ -2658,7 +2907,17 @@ ags_thread_real_clock(AgsThread *thread)
 		       0);
     }
 
+#ifdef __APPLE__
+    host_get_clock_service(mach_host_self(), CALENDAR_CLOCK, &cclock);
+    
+    clock_get_time(cclock, &mts);
+    mach_port_deallocate(mach_task_self(), cclock);
+    
+    thread->computing_time->tv_sec = mts.tv_sec;
+    thread->computing_time->tv_nsec = mts.tv_nsec;
+#else
     clock_gettime(CLOCK_MONOTONIC, thread->computing_time);
+#endif
   }
 #endif
   
@@ -2674,7 +2933,7 @@ ags_thread_real_clock(AgsThread *thread)
   //  pthread_mutex_unlock(thread->mutex);
   /* increment delay counter and set run per cycle */
   if(thread->tic_delay == 0){
-    if(thread->freq >= AGS_THREAD_MAX_PRECISION){
+    if(thread->freq >= thread->max_precision){
       steps = 1.0 / thread->delay;
     }else{
       steps = 1;
@@ -2684,7 +2943,7 @@ ags_thread_real_clock(AgsThread *thread)
   }
 
   /* nth cycle */
-  if(thread->cycle_iteration != AGS_THREAD_MAX_PRECISION){
+  if(thread->cycle_iteration != thread->max_precision){
     thread->cycle_iteration++;
   }else{
     thread->cycle_iteration = 0;
@@ -2701,7 +2960,7 @@ ags_thread_real_clock(AgsThread *thread)
  *
  * Returns: the cycles to be performed
  * 
- * Since: 0.6.42
+ * Since: 1.0.0
  */
 guint
 ags_thread_clock(AgsThread *thread)
@@ -2709,12 +2968,7 @@ ags_thread_clock(AgsThread *thread)
   guint thread_signal;
   guint cycles;
   
-  pthread_mutex_lock(&class_mutex);
-
-  thread_signal = thread_signals[CLOCK];
-
-  pthread_mutex_unlock(&class_mutex);
-  
+  thread_signal = thread_signals[CLOCK];  
   g_return_val_if_fail(AGS_IS_THREAD(thread), 0);
 
   g_object_ref(thread);
@@ -2780,22 +3034,22 @@ ags_thread_real_start(AgsThread *thread)
  *
  * Add @child to @thread's start queue.
  *
- * Since: 0.7.56
+ * Since: 2.0.0
  */
 void
 ags_thread_add_start_queue(AgsThread *thread,
 			   AgsThread *child)
 {
-  GList *start_queue;
+  if(!AGS_IS_THREAD(thread) ||
+     !AGS_IS_THREAD(child)){
+    return;
+  }
   
   pthread_mutex_lock(thread->start_mutex);
   
-  start_queue = g_atomic_pointer_get(&(thread->start_queue));
-  start_queue = g_list_prepend(start_queue,
-			       child);
-
   g_atomic_pointer_set(&(thread->start_queue),
-		       start_queue);
+		       g_list_prepend(g_atomic_pointer_get(&(thread->start_queue)),
+				      child));
   
   pthread_mutex_unlock(thread->start_mutex);
 }
@@ -2807,7 +3061,7 @@ ags_thread_add_start_queue(AgsThread *thread,
  *
  * Add @child to @thread's start queue.
  *
- * Since: 0.7.56
+ * Since: 1.0.0
  */
 void
 ags_thread_add_start_queue_all(AgsThread *thread,
@@ -2827,25 +3081,38 @@ ags_thread_add_start_queue_all(AgsThread *thread,
   pthread_mutex_unlock(thread->start_mutex);
 }
 
+/**
+ * ags_thread_is_running:
+ * @thread: the #AgsThread instance
+ *
+ * Query running flag.
+ *
+ * Since: 2.0.0
+ */
+gboolean
+ags_thread_is_running(AgsThread *thread)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return(FALSE);
+  }
+  
+  return(((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) ? TRUE: FALSE));
+}
+
 /**
  * ags_thread_start:
  * @thread: the #AgsThread instance
  *
  * Start the thread.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_start(AgsThread *thread)
 {
   guint thread_signal;
   
-  pthread_mutex_lock(&class_mutex);
-
   thread_signal = thread_signals[START];
-  
-  pthread_mutex_unlock(&class_mutex);
-
   g_return_if_fail(AGS_IS_THREAD(thread));
 
   if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) != 0){
@@ -2920,9 +3187,24 @@ ags_thread_loop(void *ptr)
   /* get start computing time */
 #ifndef AGS_USE_TIMER
   if(g_atomic_pointer_get(&(thread->parent)) == NULL){
+#ifdef __APPLE__
+    clock_serv_t cclock;
+    mach_timespec_t mts;
+#endif
+
     thread->delay = 
-      thread->tic_delay = (AGS_THREAD_HERTZ_JIFFIE / thread->freq) / (AGS_THREAD_HERTZ_JIFFIE / AGS_THREAD_MAX_PRECISION);
+      thread->tic_delay = (AGS_THREAD_HERTZ_JIFFIE / thread->freq) / (AGS_THREAD_HERTZ_JIFFIE / thread->max_precision);
+#ifdef __APPLE__
+    host_get_clock_service(mach_host_self(), CALENDAR_CLOCK, &cclock);
+
+    clock_get_time(cclock, &mts);
+    mach_port_deallocate(mach_task_self(), cclock);
+
+    thread->computing_time->tv_sec = mts.tv_sec;
+    thread->computing_time->tv_nsec = mts.tv_nsec;
+#else
     clock_gettime(CLOCK_MONOTONIC, thread->computing_time);
+#endif
   }
 #endif
   
@@ -2958,7 +3240,7 @@ ags_thread_loop(void *ptr)
 	queued_thread->tic_delay = thread->tic_delay;
 
 	if((AGS_THREAD_INTERMEDIATE_PRE_SYNC & (g_atomic_int_get(&(queued_thread->flags)))) != 0){
-	  if(queued_thread->freq >= AGS_THREAD_MAX_PRECISION){
+	  if(queued_thread->freq >= queued_thread->max_precision){
 	    queued_thread->tic_delay = 0;
 	  }else{
 	    if(queued_thread->tic_delay < queued_thread->delay){
@@ -2970,7 +3252,7 @@ ags_thread_loop(void *ptr)
 	}
 	
 	if((AGS_THREAD_INTERMEDIATE_POST_SYNC & (g_atomic_int_get(&(queued_thread->flags)))) != 0){
-	  if(queued_thread->freq >= AGS_THREAD_MAX_PRECISION){
+	  if(queued_thread->freq >= queued_thread->max_precision){
 	    queued_thread->tic_delay = 0;
 	  }else{
 	    if(queued_thread->tic_delay > 0){
@@ -3005,7 +3287,7 @@ ags_thread_loop(void *ptr)
     g_list_free(start_start_queue);
     
     /* notify start */
-    /* signal AgsAudioLoop */
+    /* signal AgsMainLoop */
     pthread_mutex_lock(thread->start_mutex);
       
     g_atomic_int_set(&(thread->start_done),
@@ -3030,8 +3312,10 @@ ags_thread_loop(void *ptr)
 	ags_thread_unlock(thread);
 
 	/* init and wait */
+#ifndef __APPLE__
 	pthread_barrier_init(thread->barrier[0], NULL, wait_count);
 	pthread_barrier_wait(thread->barrier[0]);
+#endif
       }else{
 	/* retrieve wait count */
 	ags_thread_lock(thread);
@@ -3041,8 +3325,10 @@ ags_thread_loop(void *ptr)
 	ags_thread_unlock(thread);
 
 	/* init and wait */
+#ifndef __APPLE__
 	pthread_barrier_init(thread->barrier[1], NULL, wait_count);
 	pthread_barrier_wait(thread->barrier[1]);
+#endif
       }
     }
       
@@ -3223,11 +3509,13 @@ ags_thread_loop(void *ptr)
     }
     
     /* yield */
+#ifndef __APPLE__
     if(main_loop != NULL &&
        main_loop->tic_delay != main_loop->delay &&
        thread->cycle_iteration % (guint) floor(AGS_THREAD_HERTZ_JIFFIE / AGS_THREAD_YIELD_JIFFIE) == 0){
       pthread_yield();
-    }    
+    }
+#endif
   }
 
   //  g_object_ref(thread);
@@ -3247,7 +3535,7 @@ ags_thread_loop(void *ptr)
     thread->tic_delay = chaos_tree->tic_delay;
 
     if((AGS_THREAD_INTERMEDIATE_PRE_SYNC & (g_atomic_int_get(&(thread->flags)))) != 0){
-      if(thread->freq >= AGS_THREAD_MAX_PRECISION){
+      if(thread->freq >= thread->max_precision){
 	thread->tic_delay = 0;
       }else{
 	if(thread->tic_delay > 0){
@@ -3259,7 +3547,7 @@ ags_thread_loop(void *ptr)
     }
 	
     if((AGS_THREAD_INTERMEDIATE_POST_SYNC & (g_atomic_int_get(&(thread->flags)))) != 0){
-      if(thread->freq >= AGS_THREAD_MAX_PRECISION){
+      if(thread->freq >= thread->max_precision){
 	thread->tic_delay = 0;
       }else{
 	if(thread->tic_delay < thread->delay){
@@ -3353,19 +3641,14 @@ ags_thread_loop(void *ptr)
  * Only for internal use of ags_thread_loop but you may want to set the your very own
  * class function namely your thread's routine.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_run(AgsThread *thread)
 {
   guint thread_signal;
   
-  pthread_mutex_lock(&class_mutex);
-
   thread_signal = thread_signals[RUN];
-  
-  pthread_mutex_unlock(&class_mutex);
-
   g_return_if_fail(AGS_IS_THREAD(thread));
 
   g_object_ref(thread);
@@ -3496,19 +3779,14 @@ ags_thread_real_stop(AgsThread *thread)
  * 
  * Stop the threads loop by unsetting AGS_THREAD_RUNNING flag.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_stop(AgsThread *thread)
 {
   guint thread_signal;
 
-  pthread_mutex_lock(&class_mutex);
-
   thread_signal = thread_signals[STOP];
-
-  pthread_mutex_unlock(&class_mutex);
-
   g_return_if_fail(AGS_IS_THREAD(thread));
 
   if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) == 0){
@@ -3532,7 +3810,7 @@ ags_thread_stop(AgsThread *thread)
  * 
  * Returns: the time spent
  *
- * Since: 0.7.46
+ * Since: 1.0.0
  */
 guint
 ags_thread_interrupted(AgsThread *thread,
@@ -3544,12 +3822,7 @@ ags_thread_interrupted(AgsThread *thread,
 
   return(0);
   
-  pthread_mutex_lock(&class_mutex);
-
   thread_signal = thread_signals[INTERRUPTED];
-
-  pthread_mutex_unlock(&class_mutex);
-
   g_return_val_if_fail(AGS_IS_THREAD(thread),
 		       0);
 
@@ -3576,7 +3849,7 @@ ags_thread_interrupted(AgsThread *thread,
  *
  * Performs hangcheck of thread.
  *
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 void
 ags_thread_hangcheck(AgsThread *thread)
@@ -3734,7 +4007,7 @@ ags_thread_self(void)
  *
  * Returns: the first ordered thread
  *
- * Since: 0.7.40
+ * Since: 1.0.0
  */
 AgsThread*
 ags_thread_chaos_tree(AgsThread *thread)
@@ -3757,7 +4030,7 @@ ags_thread_chaos_tree(AgsThread *thread)
  *
  * Returns: %TRUE if tree is in chaos, otherwise %FALSE
  *
- * Since: 0.7.43
+ * Since: 1.0.0
  */
 gboolean
 ags_thread_is_chaos_tree(AgsThread *thread,
@@ -3830,20 +4103,16 @@ ags_thread_is_chaos_tree(AgsThread *thread,
  *
  * Returns: the new #AgsThread
  * 
- * Since: 0.4.0
+ * Since: 1.0.0
  */
 AgsThread*
 ags_thread_new(gpointer data)
 {
   AgsThread *thread;
 
-  pthread_mutex_lock(&class_mutex);
-
   thread = (AgsThread *) g_object_new(AGS_TYPE_THREAD,
 				      NULL);
 
-  pthread_mutex_unlock(&class_mutex);
-
   thread->data = data;
 
   return(thread);
